/***********************************************************************************************
*	FUNCTION: 	Cdisplay.c
*	LIBRARY:  	display
*	AUTHOR:	  	Scott Streit
*	DATE:     	11-03-1999
*	VERSION:  	2.0
*
*	PARAMETERS:	mem_fd   File descriptor that references the image.
*			rows	 The number of rows in the image.
*			cols	 The number of columns in the image.
*                                                                           			
*	DESCRIPTION:	The deinterlacing algorithms work on the principle
*			that the ccd/ir array will read out the data in a
*			predetermined order depending on the type of readout
*			being implemented.  Here's how they look:     			
*                                                                           			
*   split-parallel          split-serial              quad CCD                quad IR          	
*  ----------------       ----------------        ----------------        ----------------     	
* |     1  ------->|     |        |------>|      |<-----  |  ---->|      | -----> | ----> |    	
* |                |     |        |   1   |      |   3    |   2   |      |   0    |   1   |    	
* |                |     |        |       |      |        |       |      |        |       |    	
* |_______________ |     |        |       |      |________|_______|      |________|_______|    	
* |                |     |        |       |      |        |       |      |        |       |    	
* |                |     |        |       |      |        |       |      |        |       |    	
* |                |     |   0    |       |      |   0    |   1   |      |   3    |   2   |    	
* |<--------  0    |     |<------ |       |      |<-----  |  ---->|      | -----> | ----> |    	
*  ----------------       ----------------        ----------------        ----------------     	
***********************************************************************************************/
#include <stdio.h>
#include <jni.h>
#include "SetupCommand.h"

JNIEXPORT jint JNICALL
Java_SetupCommand_Cdeinterlace(JNIEnv *env, jobject obj, jint x, jint y, jint image_fd, jint algorithm)
{
	unsigned short *old_iptr;
	unsigned short *new_iptr;
	jclass newExcCls;
	char message[100];

	old_iptr = (unsigned short *)image_fd;
	
	/* Allocate a new buffer to hold the deinterlaced image. */
	if((new_iptr = (unsigned short *) malloc(x*y*sizeof(unsigned short))) == NULL) {
		newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        	/* Unable to find the new exception class, give up. */
        	if (newExcCls == 0)
        		return;

       		sprintf(message, "Error in allocating memory for deinterlacing. \nSaving interlaced image to disk.");
		free (new_iptr);
       		
       		(*env)->ThrowNew(env, newExcCls, message);
        }

	switch(algorithm) {
		case 1:
		/*-------------- SPLIT PARALLEL READOUT ---------------------------------------*/
		{
			int i;
			
			if (((float)y/2) != (int)y/2) {
				newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        			/* Unable to find the new exception class, give up. */
        			if (newExcCls == 0)
        				return;

       				sprintf(message, "Number of rows must be EVEN for split-parallel readout. \nSkipping deinterlace.");
				free (new_iptr);
       		
       				(*env)->ThrowNew(env, newExcCls, message);
        		}

			for (i=0;i<(x*y)/2;i++) {
        			*(new_iptr+i) = *(old_iptr+(2*i));
        			*(new_iptr+(x*y)-i-1) = *(old_iptr+(2*i)+1);
        		}

			memcpy(old_iptr, new_iptr, x*y*sizeof(unsigned short));
			free(new_iptr);

			return((int)old_iptr);
		}

		case 2:
		/*-------------- SPLIT SERIAL READOUT -----------------------------------------*/
		{
			int i;
			int j;
			int p1;
			int p2;
			int begin;
			int end;

			if ((float)x/2 != (int)x/2) {
				newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        			/* Unable to find the new exception class, give up. */
        			if (newExcCls == 0)
        				return;

       				sprintf(message, "Number of cols must be EVEN for split-serial readout. \nSkipping deinterlace.");
				free (new_iptr);
       		
       				(*env)->ThrowNew(env, newExcCls, message);
        		}
			for (i=0;i<y;i++) {
				/* leave in +0 for clarity */
        			p1      = i*x+0; /*position in raw image */
        			p2      = i*x+1;
        			begin   = i*x+0; /*position in interlaced image */
        			end     = i*x+x-1;
        			for (j=0;j<x;j+=2) {
                			*(new_iptr+begin) = *(old_iptr+p1);
                			*(new_iptr+end) = *(old_iptr+p2);
                			++begin; --end;
                			p1+=2; p2+=2;
                		}
        		}

			memcpy(old_iptr, new_iptr, x*y*sizeof(unsigned short));
			free(new_iptr);

			return((int)old_iptr);
		}

		case 3:
		/*--------------- SPLIT QUAD READOUT -----------------------------------------*/
		{
			int i = 0;
			int j = 0;
			int counter = 0;
			int end = 0;
			int begin = 0;

			if ( (float)x/2 != (int)x/2 || (float)y/2 != (int)y/2) {
				newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        			/* Unable to find the new exception class, give up. */
        			if (newExcCls == 0)
        				return;

       				sprintf(message, "Number of cols AND rows must be EVEN for quad readout. \nSkipping deinterlace.");
				free (new_iptr);
       		
       				(*env)->ThrowNew(env, newExcCls, message);
        		}
			
			while(i<x*y) {
        			if (counter%(x/2) == 0) {
                			end = (x*y)-(x*j)-1;
                			begin = (x*j)+0; 	/* left in 0 for clarity*/
                			j++; 			/*number of completed rows*/
                			counter=0; 		/*reset for next convergece*/
                		}
        			*(new_iptr+begin+counter)       = *(old_iptr+i++); /*front_row--->  */
        			*(new_iptr+begin+x-1-counter)   = *(old_iptr+i++); /*front_row<--   */
        			*(new_iptr+end-counter)         = *(old_iptr+i++); /*end_row<----   */
        			*(new_iptr+end-x+1+counter)     = *(old_iptr+i++); /*end_row---->   */
        			counter++;
        		}

			memcpy(old_iptr, new_iptr, x*y*sizeof(unsigned short));
			free(new_iptr);

			return((int)old_iptr);
		}

		case 4:
		/*--------------- IR SPLIT QUAD READOUT --------------------------------------*/
		{
			int i = 0;
			int j = y-1;
			int counter = 0;
			int end = 0;
			int begin = 0;

			if ( (float)x/2 != (int)x/2 || (float)y/2 != (int)y/2) {
				newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        			/* Unable to find the new exception class, give up. */
        			if (newExcCls == 0)
        				return;

       				sprintf(message, "Number of cols AND rows must be EVEN for quad readout. \nSkipping deinterlace.");
				free (new_iptr);
       		
       				(*env)->ThrowNew(env, newExcCls, message);
        		}
			
			while(i<x*y) {
        			if (counter%(x/2) == 0) {
                			end = (j-(y/2))*x;
                			begin = j*x;
                			j--; 			/*number of completed rows*/
                			counter=0; 		/*reset for next convergece*/
                		}

        			*(new_iptr+begin+counter)       = *(old_iptr+i++); /*front_row--->  */
        			*(new_iptr+begin+(x/2)+counter) = *(old_iptr+i++); /*front_row<--   */
        			*(new_iptr+end+(x/2)+counter)   = *(old_iptr+i++); /*end_row<----   */
        			*(new_iptr+end+counter)         = *(old_iptr+i++); /*end_row---->   */
        			counter++;
        		}

			memcpy(old_iptr, new_iptr, x*y*sizeof(unsigned short));
			free(new_iptr);

			return((int)old_iptr);
		}

		case 5:
		/*--------------- USER DEFINED READOUT -----------------------------------------*/
		{
			newExcCls = (*env)->FindClass(env, "java/lang/Exception");
        
        		/* Unable to find the new exception class, give up. */
        		if (newExcCls == 0)
        			return;

       			sprintf(message, "User Defined Readout... UNDER CONSTRUCTION.");
			free(new_iptr);
       		
       			(*env)->ThrowNew(env, newExcCls, message);
		}
	}


}
