/*******************************************************************************
*	FUNCTION: 	Cwrite_fits_data.c
*	LIBRARY:  	fits
*	AUTHOR:	  	Scott Streit
*	DATE:     	04-21-2000
*	VERSION:  	1
*
*	DESCRIPTION:	This native method is used to write image data to a
*			FITS file.
*
*	PARAMETERS:	file_name - The name of the FITS file to be saved.
*			image_fd  - A reference to the start of the image buffer.
*			row/cols  - The image row and columns size respectively.
*			jheader   - An array containing the fits header info.
*			bytes     - The number of bytes in the image.
*
*	RETURNS:	None.
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <jni.h>
#include "Fits.h"

#define HEADER_MAX	24

void putFitsHead(int cols, int rows, const char *header[], FILE *fd_out);
void write_card(char *str, FILE *fd_out);

JNIEXPORT void JNICALL Java_Fits_Cwrite_1fits_1data
(JNIEnv *env, jobject obj, jstring file_name, jint image_fd, jint rows, jint 
cols, jobjectArray jheader, jint bytes)
{
	FILE *fd_out;
	int fill_count = 0;
	int z, i;
	const char *header[HEADER_MAX];
	jstring test;
	short fill_data = 0;
	const char *filename = (*env)->GetStringUTFChars(env, file_name, 0);

	/* Initialize the header */
	for (i=0; i<HEADER_MAX; i++)
		header[i] = NULL;

	/* Map the Java header array to the C header array. */
	for (i=0; i<HEADER_MAX; i++) {
		test = (jstring)(*env)->GetObjectArrayElement(env, jheader, i);
		if (test != NULL)
			header[i] = (*env)->GetStringUTFChars(env, test, 0);
	}
	
	/* Fill the rest of the buffer with zeros. */
	fill_count = 2880 - (bytes - ((bytes/2880)*2880));
	
	fd_out = fopen(filename, "wb");
	
	/* Write the header. */
	putFitsHead( cols, rows, &header[0], fd_out);

	/* Seek to the end of the header. */
	fseek(fd_out, 2880, SEEK_SET);

	/* Write the image data. */
	for (i=0; i<bytes; i++) {
		fwrite(((caddr_t)image_fd+i), sizeof(unsigned char), 1, fd_out);
	}

	/* Fill the rest of the file with 0's for padding. */
	for (z=0; z<fill_count; z++)
		fwrite(&fill_data, sizeof(char), 1, fd_out);

	fclose(fd_out);
	
	(*env)->ReleaseStringUTFChars(env, file_name, filename);
}

void putFitsHead(int cols, int rows, const char *header[], FILE *fd_out)
{
        int i = 0;
        int j = 0;
        char card[81];	/* Array is 81 elements since sprintf appends a \0 */

	/* Initialize the card */
	for (j=0; j<80; j++)
		card[j] = ' ';

	/* Write the fits header */
        sprintf(card, "SIMPLE  =           %10s                                                  ", header[0]);
        write_card(card, fd_out); ++i;
        sprintf(card, "BITPIX  =           %10s                                                  ", header[1]);
        write_card(card, fd_out); ++i;
        sprintf(card, "NAXIS   =           %10s                                                  ", header[2]);
        write_card(card, fd_out); ++i;
        sprintf(card, "NAXIS1  =           %10s                                                  ", header[3]);
        write_card(card, fd_out); ++i;
        sprintf(card, "NAXIS2  =           %10s                                                  ", header[4]);
        write_card(card, fd_out); ++i;
        sprintf(card, "DATE    =           %10s                                                  ", header[5]);
        write_card(card, fd_out); ++i;
        sprintf(card, "TIME    =           %10s                                                  ", header[6]);
        write_card(card, fd_out); ++i;
        sprintf(card, "LOCATION= %20s                                                  ", header[7]);
        write_card(card, fd_out); ++i;
        sprintf(card, "SIDETIME= %20s                                                  ", header[8]);
        write_card(card, fd_out); ++i;
        sprintf(card, "EPOCH   = %20s                                                  ", header[9]);
        write_card(card, fd_out); ++i;
        sprintf(card, "AIRMASS = %20s                                                  ", header[10]);
        write_card(card, fd_out); ++i;
        sprintf(card, "EXP_TIME=           %10s                                                  ", header[11]);
        write_card(card, fd_out); ++i;
        sprintf(card, "IMG_TYPE= %20s                                                  ", header[12]);
        write_card(card, fd_out); ++i;
        sprintf(card, "TELESCPE= %20s                                                  ", header[13]);
        write_card(card, fd_out); ++i;
        sprintf(card, "INSTRMNT= %20s                                                  ", header[14]);
        write_card(card, fd_out); ++i;
        sprintf(card, "FILTER  = %20s                                                  ", header[15]);
        write_card(card, fd_out); ++i;
        sprintf(card, "OBJECT  = %20s                                                  ", header[16]);
        write_card(card, fd_out); ++i;
        sprintf(card, "RA      = %20s                                                  ", header[17]);
        write_card(card, fd_out); ++i;
        sprintf(card, "DEC     = %20s                                                  ", header[18]);
        write_card(card, fd_out); ++i;
        sprintf(card, "OBSERVER= %20s                                                  ", header[19]);
        write_card(card, fd_out); ++i;
        sprintf(card, "COMMENT1= %20s                                                  ", header[20]);
        write_card(card, fd_out); ++i;
        sprintf(card, "COMMENT2= %20s                                                  ", header[21]);
        write_card(card, fd_out); ++i;
        sprintf(card, "COMMENT3= %20s                                                  ", header[22]);
        write_card(card, fd_out); ++i;
        sprintf(card, "COMMENT4= %20s                                                  ", header[23]);
        write_card(card, fd_out); ++i;
        sprintf(card, "END                                                                             ");
        write_card(card, fd_out); ++i;
        sprintf(card, "                                                                          ");
        for (j=0; j<(2880-i*80); j++) {
                write_card(card, fd_out);
        }
}

void write_card(char *str, FILE *fd_out)
{
        fwrite(str, 80, 1, fd_out);
}

