/**
********************************************************************************
*	This class is used to calculate the current array temperature from the
*	thermistor adu value supplied by the controller.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
public class ArrayTemperature
{
	private static double temperature_coeff[];
	private static double adu_per_volt;
	private static double adu_offset;
	private static double vu;
	private static double vl;
	private static boolean linear_mode = false;

	/**
	*****************************************************************************
	*	Default constructor.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public ArrayTemperature() {}

	/**
	*****************************************************************************
	*	Determines whether to use the linear or non-linear algorithm.
	*
	*	@param algorithm  May be one of two Strings: "linear" or nonlinear".
	*
	*	@return The array temperature, in Celcius, is returned.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public static void set_algorithm(String algorithm)
	{
		if (algorithm.equals("linear")) {
			temperature_coeff = new double[2];
			temperature_coeff[0] = 0;
			temperature_coeff[1] = 0;
			linear_mode = true;
		}
		
		else {
			temperature_coeff = new double[11];
			temperature_coeff[0] = VoodooConstants.NONLINEAR_TEMP_COEFF0;
			temperature_coeff[1] = VoodooConstants.NONLINEAR_TEMP_COEFF1;
			temperature_coeff[2] = VoodooConstants.NONLINEAR_TEMP_COEFF2;
			temperature_coeff[3] = VoodooConstants.NONLINEAR_TEMP_COEFF3;
			temperature_coeff[4] = VoodooConstants.NONLINEAR_TEMP_COEFF4;
			temperature_coeff[5] = VoodooConstants.NONLINEAR_TEMP_COEFF5;
			temperature_coeff[6] = VoodooConstants.NONLINEAR_TEMP_COEFF6;
			temperature_coeff[7] = VoodooConstants.NONLINEAR_TEMP_COEFF7;
			temperature_coeff[8] = VoodooConstants.NONLINEAR_TEMP_COEFF8;
			temperature_coeff[9] = VoodooConstants.NONLINEAR_TEMP_COEFF9;
			temperature_coeff[10] = VoodooConstants.NONLINEAR_TEMP_COEFF10;
			adu_per_volt = VoodooConstants.NONLINEAR_ADU_PER_VOLT;
			adu_offset = VoodooConstants.NONLINEAR_ADU_OFFSET;
			vu = VoodooConstants.NONLINEAR_VU;
			vl = VoodooConstants.NONLINEAR_VL;
			linear_mode = false;
		}
	}

	/**
	*****************************************************************************
	*	Sets the coefficients to use with the linear algorithm. The input
	*	strings are parsed as double precision.
	*
	*	@param coeff1 The first coefficient.
	*	@param coeff2 The second coefficient.
	*
	*	@throws NumberFormatException If coefficients could not be parsed.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public static void set_linear_coefficients(String coeff1, String coeff2) throws NumberFormatException
	{
		try {
			temperature_coeff[0] = Double.parseDouble(coeff1);
			temperature_coeff[1] = Double.parseDouble(coeff2);
		}
		catch (NumberFormatException nfe) {
			temperature_coeff[0] = 0;
			temperature_coeff[1] = 0;
			throw new NumberFormatException("ERROR: Could not parse temperature coeffients");
		}
	}

	/**
	*****************************************************************************
	*	Calculates the array temperature for the nonlinear case, in Celcius,
	*	from the thermistor adu	value returned by the controller.
	*
	*	@param adu The array temperature in adu.
	*
	*	@return The array temperature, in Celcius, is returned.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public double calculate_temperature(int adu)
	{
		double temperature = -273.15;
		double voltage = 0.0;
		double tc[] = new double[temperature_coeff.length];
		double x = 0.0;
		
		if (linear_mode) {
			temperature = temperature_coeff[0] + temperature_coeff[1]*((double)adu);
		}
		
		else {
			// Convert adu's to voltage.
			voltage = (adu - (double) adu_offset) / adu_per_volt;

  			// Calculate dimensionless variable for the Chebychev series.
  			x = ((voltage - vl) - (vu - voltage)) / (vu - vl);

  			tc[0] = 1;
  			tc[1] = x;

       			temperature += temperature_coeff[0] + (temperature_coeff[1] * x);
       			for (int i=2; i<temperature_coeff.length; i++) {
               			tc[i] = 2.0 * x * tc[i-1] - tc[i-2];
       				temperature += temperature_coeff[i] * tc[i];
               		}
               	}
        	
        	return temperature;
	}

	/**
	*****************************************************************************
	*	Calculates the adu value from the temperature for the nonlinear case.
	*	The calculated adu value may be passed to the controller.
	*
	*	@param temperature The array temperature in adu.
	*
	*	@return The adu value is returned.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public int calculate_adu(double temperature)
	{
		boolean tolerance;
		int trials = 0;
		double adu = 0.0;
         	double vmid;
         	double lower;
         	double upper;
         	double voltage = 0.0;
         	double target_temp = -273.15;
         	double x;

		if (linear_mode) {
			adu = (temperature - temperature_coeff[0])/temperature_coeff[1];
		}
		
		else {
	  		lower = vl;
  			upper = vu;
			tolerance = false;
	  		vmid = (lower + upper)*0.5;

	  		/* Numerically determine v for a given temperature   */
	  		/* Choose an initial adu in the middle of vu and vl  */
	  		adu = vmid*adu_per_volt + (double)adu_offset;

	  		if (Math.abs(target_temp - temperature) > VoodooConstants.TOLERANCE)
	  			tolerance = true;
  		
	  		while (tolerance && (trials < VoodooConstants.MAX_TOLERANCE_TRIALS)) {
	    			target_temp = calculate_temperature((int)adu);

				if (Math.abs(target_temp - temperature) > VoodooConstants.TOLERANCE)
	  				tolerance = true;
  			
	    			if (tolerance) {
	      				if (target_temp < temperature) { 
        					upper = vmid;
        					vmid = (lower + upper)*0.5;
        					adu = vmid * adu_per_volt + (double)adu_offset;
      					}
      					
      					else if (target_temp > temperature) {
        					lower = vmid;
        					vmid = (lower + upper)*0.5;
        					adu = vmid * adu_per_volt + (double)adu_offset;
      					}
    				}

    				trials++;
			}
		}

  		return (int)adu;
	}
}