/**
********************************************************************************
*	A class to display a window from which the user may perform Controller
*       configuration setup tasks (obtaining the image size, choosing the binning
*       parameters and calculating the derived array size - image size x binning)
*
*	CVM - View
*
* 	@filename ControllerConfigDialog.java
*	@see JDialog
*	@version 1.00
*	@author Xiangyang (Helena) Xian
*
*	DATE		AUTHOR	DESCRIPTION
*	-----------------------------------
*	07/28/2000	sds	Removed all "Apply" buttons from the individual
*				panels and created one master "Apply" button.
*				Removed the "Auto Deinterlace" option.
*	08/14/2000	sds	Changed	most of the code to support new bit
*				definitions.
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import javax.swing.border.*;

public class ControllerConfigDialog extends JDialog 
{
	protected Container content_pane;
	protected JPanel top_both_readout_panel;
	protected JPanel top_serial_readout_panel;
	protected JPanel bottom_readout_panel;
	protected SetupDialog parent;

    	protected JComboBox binning_parameters_combo_Bx; 
	protected JComboBox binning_parameters_combo_By;
	protected JComboBox number_of_readout_channels_combo_box;

	private JButton apply_button;
	private JButton apply_all_button;

	protected Icon single_readout_A_icon;
	protected Icon single_readout_B_icon;
	protected Icon single_readout_C_icon;
	protected Icon single_readout_D_icon;
	protected Icon dual_readout_serial_L_icon;
	protected Icon dual_readout_serial_R_icon;
	protected Icon dual_readout_serial_LR_icon;
	protected Icon dual_readout_AB_icon;
	protected Icon dual_readout_CD_icon;
	protected Icon quad_readout_icon;
	
	protected JLabel readout_pics_label;
	protected JLabel both_readout_pics_label;
	protected JLabel serial_readout_pics_label;

	protected JLabel nc_label;
	protected JLabel range_of_nc_label;
	protected JLabel configuration_code_label;
	protected JLabel supported_capabilities_label;
	protected JLabel supported_bit_description_label;
	protected JLabel linear_coeff_description_label1;
	protected JLabel linear_coeff_description_label2;
	protected JLabel linear_coeff1_label;
	protected JLabel linear_coeff2_label;
	protected JLabel[] channel_label = new JLabel[32];

	protected VTextField image_size_ncols_txtfld;
	protected VTextField image_size_nrows_txtfld;
	protected VTextField derived_array_size_x_txtfld;
	protected VTextField derived_array_size_y_txtfld;
	protected VTextField readout_status_txtfld;
	protected VTextField readout_status_serial_txtfld;
	protected VTextField readout_status_parallel_txtfld;
	protected VTextField nc_txtfld;
	protected VTextField temperature_txtfld;
	protected VTextField linear_coeff1_txtfld;
	protected VTextField linear_coeff2_txtfld;
	protected VTextField[] channel_txtfld = new VTextField[32];

	protected JRadioButton single_readout_A_rbutton;
	protected JRadioButton single_readout_B_rbutton;
	protected JRadioButton single_readout_C_rbutton;
	protected JRadioButton single_readout_D_rbutton;
	protected JRadioButton dual_readout_AB_rbutton;
	protected JRadioButton dual_readout_CD_rbutton;
	protected JRadioButton quad_readout_ABCD_rbutton;
	protected JRadioButton serial_readout_L_rbutton;
	protected JRadioButton serial_readout_R_rbutton;
	protected JRadioButton serial_readout_LR_rbutton;
      	protected JRadioButton gain_1_rbutton;
      	protected JRadioButton gain_2_rbutton;
	protected JRadioButton gain_475_rbutton;
      	protected JRadioButton gain_95_rbutton;
	protected JRadioButton slow_speed_rbutton;
	protected JRadioButton fast_speed_rbutton;

	protected JCheckBox mpp_chkbox;
	protected JCheckBox number_of_coadder_chkbox;
	protected JCheckBox download_coadder_board_chkbox;
	protected JCheckBox array_temperature_chkbox;

	protected JTextArea supported_capabilities_txtarea;

	protected int number_of_controller_config_windows = 0;
	protected int image_ncols = 0;
	protected int image_nrows = 0;
	protected int derived_size_x = 0;
	protected int derived_size_y = 0;
	protected int config_data = 0;
	protected JTabbedPane controller_config_tabbed_pane;
	private JMenuBar menu_bar;

     	protected ControllerConfigListener controller_config_listener;

	private Font font;
	private Font summary_font;

	/**
	********************************************************************************
	*	Class constructor. The window is created in this method.
	*
	*	@param owner The window from which this dialog window was opened.
	*
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian  	
	********************************************************************************
	*/
	public ControllerConfigDialog(SetupDialog parent)
	{
		// Call the superclass (JDialog) constructor.
		super((Frame)null, "Controller Configuration", false);
		
		// Save the parent.
		this.parent = parent;
		
		// Set the dialog box size.
		setSize(400, 370);
		
		// Make the window non-resizable.
      		setResizable(false);
		
		// Get a handle on the dialog box's content pane and set its layout.
		content_pane = this.getContentPane();
		content_pane.setLayout(new BorderLayout());

		// Set the components font.
		font = new Font("LucidaSansRegular", Font.PLAIN, 11);

		// Create the event lister.
  		controller_config_listener = new ControllerConfigListener(this);	
		
		// Create and add the menu/toolbar panel.
     		menu_bar = create_menu_bar();
     		setJMenuBar(menu_bar);
		
                // Create the tabbed pane.
                JTabbedPane tp = create_tabbed_pane();
                content_pane.add(tp);
		
		// Add the button panel.
		content_pane.add(create_button_panel(), BorderLayout.SOUTH);

      		// Add a window listener.  This has to be here to allow the dialog box to be re-opened
      		// later when the "close" window menu item (from system, not java) is chosen.
      		addWindowListener(new WindowAdapter()
      		{
      			public void windowClosing(WindowEvent e) {}
		});
				
		// Initialize any variables.
		number_of_controller_config_windows = 0;
	}
	
        /**
        ********************************************************************************
        *       This method sets ControllerConfigDialog to the certain bit values.  
        *
        *       @param  config_data the controller configuration bits
        *
        *       @version 1.00
        *       @author Xiangyang (Helena) Xian
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	08/01/2000	sds	Integrated panels. Cleaned up code.
        ********************************************************************************
        */
	public void set_config_data(int config_data)
	{
		this.config_data = config_data;

		if ((number_of_controller_config_windows == 0)) {

			// Add the summary tab.
			Component summary_panel = create_summary_panel();
			controller_config_tabbed_pane.addTab("Configuration", summary_panel);

			//--------------------------------------------
			//	VIDEO PROCESSOR BITS
			//--------------------------------------------
			// Add the gain and video offsets tabs if CCD REV3B, otherwise, don't.
			if ((config_data & VoodooConstants.VIDEO_PROCESSOR_MASK) == VoodooConstants.CCD_REV3B) {
				Component gain_panel = create_gain_panel();
				controller_config_tabbed_pane.addTab("Gain", gain_panel);
				controller_config_listener.controller_config_cmd.set_gain(true);

				// Add the video offsets tab.
				//Component video_offsets_panel = create_video_offsets_panel();
				//controller_config_tabbed_pane.addTab("Video Offsets", video_offsets_panel);
				//controller_config_listener.controller_config_cmd.set_video_offsets(true);

				set_supported_text("Video processor Rev. 3B");
			}

			else if ((config_data & VoodooConstants.VIDEO_PROCESSOR_MASK) == VoodooConstants.CCD_GENI)
				set_supported_text("Video processor Generation I");

			// Set the configuration to IR.
			else if ((config_data & VoodooConstants.VIDEO_PROCESSOR_MASK) == VoodooConstants.IR_REV4C) {
				parent.parent.set_exposure_event_listener(VoodooConstants.IR);
				controller_config_listener.array_mode = VoodooConstants.IR;
				set_supported_text("Video processor IR Rev. 4C");
			}
			
			// Set the configuration to CCD.
			else if ((config_data & VoodooConstants.VIDEO_PROCESSOR_MASK) != VoodooConstants.IR_REV4C) {
				controller_config_listener.array_mode = VoodooConstants.CCD;
			}

			// Add the coadder tab.
			else if ((config_data & VoodooConstants.VIDEO_PROCESSOR_MASK) == VoodooConstants.IR_COADDER) {
				Component coadder_panel = create_coadder_panel();
				controller_config_tabbed_pane.addTab("Coadder", coadder_panel);
				controller_config_listener.controller_config_cmd.set_coadder(true);
				set_supported_text("Coadder board present");
			}

			//--------------------------------------------
			//	TIMING BOARD BITS
			//--------------------------------------------
			// Check for Rev4B timing board.
			if ((config_data & VoodooConstants.TIMING_BOARD_MASK) == VoodooConstants.TIM_REV4B) {
				// Set the "Reset Controller" options to visible.
				parent.reset_controller_chkbox.setVisible(true);
				parent.parent.reset_controller_button.setVisible(true);
				set_supported_text("Timing board Rev. 4B");
			}

			// Check for GenI timing board.
			else if ((config_data & VoodooConstants.TIMING_BOARD_MASK) == VoodooConstants.TIM_GENI) {
				// Set the "Reset Controller" options to invisible.
				parent.reset_controller_chkbox.setVisible(false);
				parent.parent.reset_controller_button.setVisible(false);
				set_supported_text("Timing board Generation I");
			}

			//--------------------------------------------
			//	UTILITY BOARD BITS
			//--------------------------------------------
			// Check for Rev.3 utility board. If it exists, add the temperature tab.
			if ((config_data & VoodooConstants.UTILITY_BOARD_MASK) == VoodooConstants.UTILITY_REV3) {
				set_supported_text("Utility board Rev. 3");
			}

			//--------------------------------------------
			//	SHUTTER BITS
			//--------------------------------------------
			// Add shutter control.
			if ((config_data & VoodooConstants.SHUTTER_MASK) == VoodooConstants.SHUTTER) {
				parent.parent.open_shutter_chkbox.setVisible(true);
				parent.parent.open_shutter_now_button.setVisible(true);
				set_supported_text("Shutter control");
			}

			// No shutter control.
			else {
				parent.parent.open_shutter_chkbox.setVisible(false);
				parent.parent.open_shutter_now_button.setVisible(false);
			}

			//--------------------------------------------
			//	TEMPERATURE READOUT BITS
			//--------------------------------------------
			// Add the temperature tab.
			if ((config_data & VoodooConstants.TEMPERATURE_READOUT_MASK) != VoodooConstants.NO_TEMPERTURE_CONTROL) {
				Component temperature_panel = create_temperature_panel();
				controller_config_tabbed_pane.addTab("Temperature", temperature_panel);
				controller_config_listener.controller_config_cmd.set_temperature(true);
				parent.parent.read_temperature_button.setEnabled(true);

				// Set temperature conversion algorithm.
				if ((config_data & VoodooConstants.TEMPERATURE_READOUT_MASK) == VoodooConstants.NONLINEAR_TEMP_CONV) {
					ArrayTemperature.set_algorithm("nonlinear");
					set_supported_text("Nonlinear temperature conversion");
					
					// Hide the linear coefficient components.
					linear_coeff1_txtfld.setVisible(false);
					linear_coeff2_txtfld.setVisible(false);
					linear_coeff_description_label1.setVisible(false);
					linear_coeff_description_label2.setVisible(false);
					linear_coeff1_label.setVisible(false);
					linear_coeff2_label.setVisible(false);
				}

				else if ((config_data & VoodooConstants.TEMPERATURE_READOUT_MASK) == VoodooConstants.LINEAR_TEMP_CONV) {
					ArrayTemperature.set_algorithm("linear");
					set_supported_text("Linear temperature conversion");
				}
			}
			
			else
				parent.parent.read_temperature_button.setEnabled(false);

			//--------------------------------------------
			//	SUBARRAY BITS
			//--------------------------------------------


			//--------------------------------------------
			//	BINNING BITS
			//--------------------------------------------
			// Add the binning tab.
			if ((config_data & VoodooConstants.BINNING_MASK) == VoodooConstants.BINNING) {
				Component binning_panel = create_binning_panel();
				controller_config_tabbed_pane.addTab("Binning", binning_panel);
				controller_config_listener.controller_config_cmd.set_binning(true);
				set_supported_text("Binning supported");
			}

			//--------------------------------------------
			//	READOUT BITS
			//--------------------------------------------
			// Add the Readout tab.
			if ((config_data & VoodooConstants.READOUT_MASK) == VoodooConstants.SERIAL_SPLIT) {
				Component serial_readout_panel = create_serial_readout_panel();
				controller_config_tabbed_pane.addTab("Readout", serial_readout_panel);
				controller_config_listener.controller_config_cmd.set_serial_readout(true);
				set_supported_text("Split serial clocking");
			}
			
			else if ((config_data & VoodooConstants.READOUT_MASK) == VoodooConstants.PARALLEL) {
				Component serial_readout_panel = create_serial_readout_panel();
				controller_config_tabbed_pane.addTab("Readout", serial_readout_panel);
				controller_config_listener.controller_config_cmd.set_serial_readout(true);
				set_supported_text("Split parallel clocking");
			}			
			
			else if ((config_data & VoodooConstants.READOUT_MASK) == VoodooConstants.BOTH_READOUTS) {
				Component readout_both_panel = create_readout_both_panel();
				controller_config_tabbed_pane.addTab("Readout", readout_both_panel);
				controller_config_listener.controller_config_cmd.set_both_readouts(true);
				set_supported_text("Split serial clocking");
				set_supported_text("Split parallel clocking");
			}

			//--------------------------------------------
			//	MPP BITS
			//--------------------------------------------
			// Check for the bit 16 for mpp_capable command.
			if ((config_data & VoodooConstants.MPP_MASK) == VoodooConstants.MPP_CAPABLE) {
				Component mpp_panel = create_mpp_panel();
				controller_config_tabbed_pane.addTab("MPP", mpp_panel);
				controller_config_listener.controller_config_cmd.set_mpp(true);
				set_supported_text("MPP capable");
			}

			number_of_controller_config_windows++;
		}

		else {
			// Check for the bit 18 for binning.
			if ((config_data & VoodooConstants.BINNING_MASK) == VoodooConstants.BINNING) {
				// Get the values for the image columns and rows from the Controller Setup Window.
				image_ncols = parent.setup_listener.setup_cmd.get_ccd_cols();
				image_nrows = parent.setup_listener.setup_cmd.get_ccd_rows();
				
				// Set the texts of values for the image size and the derived array size.
				image_size_ncols_txtfld.setText(Integer.toString(image_ncols));
				image_size_nrows_txtfld.setText(Integer.toString(image_nrows));
				derived_array_size_x_txtfld.setText(Integer.toString(image_ncols * (binning_parameters_combo_Bx.getSelectedIndex() + 1)));
				derived_array_size_y_txtfld.setText(Integer.toString(image_nrows * (binning_parameters_combo_By.getSelectedIndex() + 1)));
			}
		}
	}   

        /**
        *****************************************************************************
        *       Creates the "tabbed" panels portion of the window.
        *
        *       @return The created "tabbed pane" is returned.
        *
        *       @see JTabbedPane
        *       @version 1.00
        *       @author Xiangyang (Helena) Xian 
        *****************************************************************************
        */
        private JTabbedPane create_tabbed_pane()
	{
		// Create a new tabbed pane and set the tabs to be on top.
		controller_config_tabbed_pane = new JTabbedPane(JTabbedPane.TOP);

                // Set the tab event listener.
		controller_config_tabbed_pane.addChangeListener(controller_config_listener);

		return controller_config_tabbed_pane;
	}

	/**
	********************************************************************************
	*	This method creates the panel that contains the apply button.
	*
	*	@return Returns the panel that contains the apply button.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JPanel create_button_panel()
	{
		JPanel button_panel = new JPanel();

		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		button_panel.setLayout(gbl);

		// Create the buttons.
		apply_button = new JButton("Apply Above");
		apply_all_button = new JButton("Apply All");

		// Set the button size.
		apply_button.setPreferredSize(new Dimension(90, 26));
		apply_all_button.setPreferredSize(new Dimension(90, 26));

		// Set any needed action commands.
		apply_all_button.setActionCommand("Abort All");

		// Add the setup components to the panel.
		add_components(button_panel, apply_button, gbl, gbc, 0, 0, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 1, 1);
		add_components(button_panel, apply_all_button, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 1, 0, 1, 1);

		// Add the event listeners.
		apply_button.addActionListener(controller_config_listener);
		apply_all_button.addActionListener(controller_config_listener);

		return button_panel;
	}

	/**
	********************************************************************************
	*	This method creates the MPP (Multi-pin phased) control panel.
	*
	*	@return Returns the panel that contains the MPP controls.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JPanel create_mpp_panel()
	{
		JPanel mpp_panel = new JPanel();

		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		mpp_panel.setLayout(gbl);

		// Create the buttons.
		mpp_chkbox = new JCheckBox("Use MPP mode");

		// Add the setup components to the panel.
		add_components(mpp_panel, mpp_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 0, 1, 1);

		// Add the event listeners.
		mpp_chkbox.addItemListener(controller_config_listener);

		return mpp_panel;
	}

	/**
	********************************************************************************
	*	This method creates the summary ("Configuration") tab panel.
	*
	*	@return Returns the panel.
	*
	*	@see JTabbedPane JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_summary_panel() 
	{
		JPanel summary_panel = new JPanel();

		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		summary_panel.setLayout(gbl);

		// Create the components of labels.
		JLabel configuration_code_label = new JLabel("Configuration Code: 0x" + Integer.toHexString(config_data));
		JLabel supported_capabilities_label = new JLabel("Supported Capabilities");

		// Set the foreground color for the label components.
		configuration_code_label.setForeground(Color.black);
		supported_capabilities_label.setForeground(Color.blue);

		// Create the supported_capabilities textarea, and set the font, color, border and not editable.
		supported_capabilities_txtarea = new JTextArea("", 11, 20);
		supported_capabilities_txtarea.setFont(summary_font);
		supported_capabilities_txtarea.setLineWrap(true);
		supported_capabilities_txtarea.setEditable(false);
			
		// Create a scrollPane object and add the to textarea to it.
		JScrollPane supported_scrollPane = new JScrollPane(supported_capabilities_txtarea, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		// Add the components to the panel.
		add_components(summary_panel, configuration_code_label, gbl, gbc, 5, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 1, 1);
		add_components(summary_panel, supported_capabilities_label, gbl, gbc, 0, 0, 5, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 1, 1, 1);
		add_components(summary_panel, supported_scrollPane, gbl, gbc, 0, 0, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 3, 1, 1);

		return summary_panel;
	}

	/**
	********************************************************************************
	*	This method creates the panel that contains all the user input components.
	*
	*	@return Returns the panel that contains all the user input components.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_binning_panel()
	{
		JPanel binning_panel = new JPanel();
		
		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		binning_panel.setLayout(gbl);
      		
		// Create the label components.
		JLabel image_size_label = new JLabel("Image Size");
		JLabel ncols_label = new JLabel("ncols: ");
		JLabel nrows_label = new JLabel("nrows: "); 
		JLabel binning_parameters_label1 = new JLabel("Binning");
		JLabel binning_parameters_label2 = new JLabel("Parameters");
		JLabel binning_parameters__x_label = new JLabel("Bx: ");
		JLabel binning_parameters__y_label = new JLabel("By: ");
		JLabel derived_array_size_label1 = new JLabel("Derived");
		JLabel derived_array_size_label2 = new JLabel("Array Size");
		JLabel range_of_B_label = new JLabel(" 1 <= Bx/By <= 8 ");

		// Set the label colors.				
		image_size_label.setForeground(Color.black);
		ncols_label.setForeground(Color.black);
		nrows_label.setForeground(Color.black);
		binning_parameters_label1.setForeground(Color.black);
		binning_parameters_label2.setForeground(Color.black);
		binning_parameters__x_label.setForeground(Color.black);
		binning_parameters__y_label.setForeground(Color.black);
		derived_array_size_label1.setForeground(Color.black);
		derived_array_size_label2.setForeground(Color.black);
		range_of_B_label.setForeground(Color.black);
		
		// Create the textfield components.
		image_size_ncols_txtfld = new VTextField("", 4);
		image_size_nrows_txtfld = new VTextField("", 4);
		derived_array_size_x_txtfld = new VTextField("", 4);
		derived_array_size_y_txtfld = new VTextField("", 4);
		
		// Create a JComboBox for the the binning parameters for the columns.	
		binning_parameters_combo_Bx = new JComboBox();

		// Set JComboBox editable to true.
		binning_parameters_combo_Bx.setEditable(true);
		
		// Set the size of binning_parameters_combo_Bx to 1 character big.
		binning_parameters_combo_Bx.setPreferredSize(new Dimension(45, 20));

		// Create the JComboBox components.
		binning_parameters_combo_Bx.addItem("1");
		binning_parameters_combo_Bx.addItem("2");
		binning_parameters_combo_Bx.addItem("3");
		binning_parameters_combo_Bx.addItem("4");
		binning_parameters_combo_Bx.addItem("5");
		binning_parameters_combo_Bx.addItem("6");
		binning_parameters_combo_Bx.addItem("7");
		binning_parameters_combo_Bx.addItem("8");

		// Set the column binning action command.
		binning_parameters_combo_Bx.setActionCommand("Column Binning");
		
		// Add ItemListener to the binning_parameters_combo_Bx.
		binning_parameters_combo_Bx.addItemListener(controller_config_listener);

		// Create a JComboBox for the the binning parameters for the rows.
		binning_parameters_combo_By = new JComboBox();

		// Set JComboBox editable to true.
		binning_parameters_combo_By.setEditable(true);

		// Set the size of binning_parameters_combo_By to 1 character big.
		binning_parameters_combo_By.setPreferredSize(new Dimension(45, 20));

		// Create the JComboBox components.
		binning_parameters_combo_By.addItem("1");
		binning_parameters_combo_By.addItem("2");
		binning_parameters_combo_By.addItem("3");
		binning_parameters_combo_By.addItem("4");
		binning_parameters_combo_By.addItem("5");
		binning_parameters_combo_By.addItem("6");
		binning_parameters_combo_By.addItem("7");
		binning_parameters_combo_By.addItem("8");

		// Set the row binning action command.
		binning_parameters_combo_By.setActionCommand("Row Binning");
		
		// Add ItemListener to the binning_parameters_combo_By.
                binning_parameters_combo_By.addItemListener(controller_config_listener);

		// Get the values for columns and rows from the SetupCommand.
		image_ncols = parent.setup_listener.setup_cmd.get_ccd_cols();
		image_nrows = parent.setup_listener.setup_cmd.get_ccd_rows();

		// Set the values for the image size. 
		image_size_ncols_txtfld.setText(Integer.toString(image_ncols));
		image_size_nrows_txtfld.setText(Integer.toString(image_nrows));

		// Derived image size is initially assigned the same as the initial image size.  
		derived_size_x = image_ncols;
                derived_size_y = image_nrows;
                derived_array_size_x_txtfld.setText(Integer.toString(derived_size_x));
                derived_array_size_y_txtfld.setText(Integer.toString(derived_size_y));
	
		// Set the image size ncols and nrows application input textfields to uneditable,
		// since the file download is the default option.
		image_size_ncols_txtfld.setEditable(false);
		image_size_nrows_txtfld.setEditable(false);

		// Set derived array size application input textfields to uneditable.
		derived_array_size_x_txtfld.setEditable(false);
		derived_array_size_y_txtfld.setEditable(false);

		// Add the setup components to the panel.
		add_components(binning_panel, image_size_label, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 1, 2, 1);
		add_components(binning_panel, binning_parameters_label1, gbl, gbc, 0, 30, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 0, 2, 1);
		add_components(binning_panel, binning_parameters_label2, gbl, gbc, 0, 30, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 1, 2, 1);
		add_components(binning_panel, derived_array_size_label1, gbl, gbc, 0, 30, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 0, 2, 1);
		add_components(binning_panel, derived_array_size_label2, gbl, gbc, 0, 30, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 1, 2, 1);
		add_components(binning_panel, ncols_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 2, 1, 1);
		add_components(binning_panel, image_size_ncols_txtfld, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 2, 1, 1);
		add_components(binning_panel, binning_parameters__x_label, gbl, gbc, 0, 30, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 2, 1, 1);
		add_components(binning_panel, binning_parameters_combo_Bx, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 2, 1, 1);
		add_components(binning_panel, derived_array_size_x_txtfld, gbl, gbc, 0, 30, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 2, 1, 1);
		add_components(binning_panel, nrows_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 3, 1, 1);
		add_components(binning_panel, image_size_nrows_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 3, 1, 1);
		add_components(binning_panel, binning_parameters__y_label, gbl, gbc, 0, 30, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 3, 1, 1);	
		add_components(binning_panel, binning_parameters_combo_By, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 3, 1, 1);
		add_components(binning_panel, derived_array_size_y_txtfld, gbl, gbc, 0, 30, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 3, 1, 1);
		add_components(binning_panel, range_of_B_label, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 6, 8, 1);

		return binning_panel;
	}

	/**
	********************************************************************************
	*	This method creates the top amp split options panel for the dialog window
	*			when both serial and parallel are true.
	*
	*	@return Returns the constructed top amp split options panel as a JPanel
	*			class.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_top_readout_both_panel()
	{
		JPanel top_readout_both_panel = new JPanel();

		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
		top_readout_both_panel.setLayout(gbl);

		// Create the icons and set icon images.
		single_readout_A_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_A.gif");
		single_readout_B_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_B.gif");
		single_readout_C_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_C.gif");
		single_readout_D_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_D.gif");
		dual_readout_AB_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_AB.gif");
		dual_readout_CD_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_CD.gif");
		quad_readout_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_QUAD.gif");

		// Create the component of picture label.
		// Label icon is initiated as single readout. 
		both_readout_pics_label = new JLabel(single_readout_A_icon);

		// Create buttonGroup for display of radioButtons.
		ButtonGroup amp_readout_both_rbutton_group = new ButtonGroup();

		// Create the components of readout radioButtons.
		single_readout_A_rbutton = new JRadioButton("A");
		single_readout_B_rbutton = new JRadioButton("B");
		single_readout_C_rbutton = new JRadioButton("C");
		single_readout_D_rbutton = new JRadioButton("D");
		dual_readout_AB_rbutton = new JRadioButton("AB");
		dual_readout_CD_rbutton = new JRadioButton("CD");
		quad_readout_ABCD_rbutton = new JRadioButton("ALL");

		// Add the radioButtons to the amp_readout_both_rbutton_group buttonGroup.
		amp_readout_both_rbutton_group.add(single_readout_A_rbutton);
		amp_readout_both_rbutton_group.add(single_readout_B_rbutton);
		amp_readout_both_rbutton_group.add(single_readout_C_rbutton);
		amp_readout_both_rbutton_group.add(single_readout_D_rbutton);
		amp_readout_both_rbutton_group.add(dual_readout_AB_rbutton);
		amp_readout_both_rbutton_group.add(dual_readout_CD_rbutton);
		amp_readout_both_rbutton_group.add(quad_readout_ABCD_rbutton);

		// Set the foreground color for the radioButton components.
		single_readout_A_rbutton.setForeground(Color.black);
		single_readout_B_rbutton.setForeground(Color.black);
		single_readout_C_rbutton.setForeground(Color.black);
		single_readout_D_rbutton.setForeground(Color.black);
		dual_readout_AB_rbutton.setForeground(Color.black);
		dual_readout_CD_rbutton.setForeground(Color.black);
		quad_readout_ABCD_rbutton.setForeground(Color.black);

		// Set the action commands.
		dual_readout_AB_rbutton.setActionCommand("SPLIT BOTH");
		dual_readout_CD_rbutton.setActionCommand("SPLIT BOTH");
		quad_readout_ABCD_rbutton.setActionCommand("SPLIT BOTH");
		
                // Add the components to the panel.
		add_components(top_readout_both_panel, both_readout_pics_label, gbl, gbc, 0, 0, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 2, 2);
		add_components(top_readout_both_panel, single_readout_A_rbutton, gbl, gbc, 0, 10, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 0, 1, 1);
		add_components(top_readout_both_panel, single_readout_B_rbutton, gbl, gbc, 0, 0, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 0, 1, 1);
		add_components(top_readout_both_panel, single_readout_C_rbutton, gbl, gbc, 0, 0, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 0, 1, 1);
		add_components(top_readout_both_panel, single_readout_D_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 0, 1, 1);
		add_components(top_readout_both_panel, dual_readout_AB_rbutton, gbl, gbc, 0, 10, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 1, 1, 1);
		add_components(top_readout_both_panel, dual_readout_CD_rbutton, gbl, gbc, 0, 0, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 1, 1, 1);
		add_components(top_readout_both_panel, quad_readout_ABCD_rbutton, gbl, gbc, 0, 0, 0, 0,GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 1, 1, 1);

		// Add the actionListener to the radioButton components.
		single_readout_A_rbutton.addActionListener(controller_config_listener);
		single_readout_B_rbutton.addActionListener(controller_config_listener);
		single_readout_C_rbutton.addActionListener(controller_config_listener);
		single_readout_D_rbutton.addActionListener(controller_config_listener);
		dual_readout_AB_rbutton.addActionListener(controller_config_listener);
		dual_readout_CD_rbutton.addActionListener(controller_config_listener);
		quad_readout_ABCD_rbutton.addActionListener(controller_config_listener);

		return top_readout_both_panel;
	}

	/**
	********************************************************************************
	*	This method creates the top amp split options panel for the dialog window
	*			when only serial is true.
	*
	*	@return Returns the constructed top amp split options panel as a JPanel
	*			class.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_top_readout_serial_panel()
	{
		JPanel top_readout_serial_panel = new JPanel();

		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
		top_readout_serial_panel.setLayout(gbl);

		// Create the icons and set icon images.
		dual_readout_serial_L_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_L.gif");
		dual_readout_serial_R_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_R.gif");
		dual_readout_serial_LR_icon = new ImageIcon(VoodooConstants.PIC_DIR + "AMP_LR.gif");

		// Create the component of picture label.
		// Label icon is initiated as single readout. 
		serial_readout_pics_label = new JLabel(dual_readout_serial_L_icon);

		// Create buttonGroup for display of radioButtons.
		ButtonGroup amp_readout_serial_rbutton_group = new ButtonGroup();

		// Create the components of readout radioButtons.
		serial_readout_L_rbutton = new JRadioButton("LEFT");
		serial_readout_R_rbutton = new JRadioButton("RIGHT");
		serial_readout_LR_rbutton = new JRadioButton("LEFT/RIGHT");
		
		// Add the radioButtons to the amp_readout_serial_rbutton_group buttonGroup.
		amp_readout_serial_rbutton_group.add(serial_readout_L_rbutton);
		amp_readout_serial_rbutton_group.add(serial_readout_R_rbutton);
		amp_readout_serial_rbutton_group.add(serial_readout_LR_rbutton);

		// Set the foreground color for the radioButton components.
		serial_readout_L_rbutton.setForeground(Color.black);
		serial_readout_R_rbutton.setForeground(Color.black);
		serial_readout_LR_rbutton.setForeground(Color.black);

		// Add the components to the panel.
		add_components(top_readout_serial_panel, serial_readout_pics_label, gbl, gbc, 0, 0, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 1, 1);
		add_components(top_readout_serial_panel, serial_readout_L_rbutton, gbl, gbc, 0, 20, 0, 0,GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 0, 1, 1);
		add_components(top_readout_serial_panel, serial_readout_R_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 0, 1, 1);
		add_components(top_readout_serial_panel, serial_readout_LR_rbutton, gbl, gbc, 0, 0, 0, 0,GridBagConstraints.NONE, GridBagConstraints.WEST,  3, 0, 1, 1);

		// Add the actionListener to the radioButton components.
		serial_readout_L_rbutton.addActionListener(controller_config_listener);
		serial_readout_R_rbutton.addActionListener(controller_config_listener);
		serial_readout_LR_rbutton.addActionListener(controller_config_listener);

		return top_readout_serial_panel;
	}

	/**
	********************************************************************************
	*	This method creates the bottom amp split options panel for this dialog
	*			window.
	*
	*	@return Returns the constructed bottom amp split options panel as a JPanel
	*			class.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_bottom_readout_panel()
	{
		JPanel bottom_readout_panel = new JPanel();

		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
		bottom_readout_panel.setLayout(gbl);

		// Create the components of readout labels and textfields.
		JLabel readout_status_label = new JLabel("Readout Status: ");
		readout_status_label.setForeground(Color.black);
		readout_status_txtfld = new VTextField("Please select a readout mode.", 18);
		readout_status_txtfld.setEditable(false);
		
		JLabel readout_status_serial_label = new JLabel("Split Serial: ");
		readout_status_serial_label.setForeground(Color.black);
		readout_status_serial_txtfld = new VTextField("", 18);
		readout_status_serial_txtfld.setEditable(false);
		
		JLabel readout_status_parallel_label = new JLabel("Split Parallel: ");
		readout_status_parallel_label.setForeground(Color.black);
		readout_status_parallel_txtfld = new VTextField("", 18);
		readout_status_parallel_txtfld.setEditable(false);

		// Add the components to the panel.
		add_components(bottom_readout_panel, readout_status_label, gbl, gbc, 20, 20, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 4, 3, 1);
		add_components(bottom_readout_panel, readout_status_txtfld, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 4, 4, 1);
		add_components(bottom_readout_panel, readout_status_serial_label, gbl, gbc, 4, 20, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 5, 3, 1);
		add_components(bottom_readout_panel, readout_status_serial_txtfld, gbl, gbc, 4, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 5, 4, 1);
		add_components(bottom_readout_panel, readout_status_parallel_label,gbl, gbc, 4, 20, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 6, 3, 1);
		add_components(bottom_readout_panel, readout_status_parallel_txtfld, gbl, gbc, 4, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 6, 4, 1);

		return bottom_readout_panel;
	}

	/**
	********************************************************************************
	*	This method creates the serial/parallel readout panel.
	*
	*	@return Returns the created panel.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_readout_both_panel()
	{
		JPanel readout_both_panel = new JPanel();

                GridBagLayout gbl = new GridBagLayout();
                GridBagConstraints gbc = new GridBagConstraints();
                readout_both_panel.setLayout(gbl);

		// Add the top panel.
		top_both_readout_panel = create_top_readout_both_panel();
		add_components(readout_both_panel, top_both_readout_panel, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 0, 1, 1);
		
		// Add the bottom panel.
		bottom_readout_panel = create_bottom_readout_panel();
		add_components(readout_both_panel, bottom_readout_panel, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 1, 1, 1);

		// Initialize the radio buttons. This has to be done here since we are adding
		// two panels and not just one.
		single_readout_A_rbutton.doClick();
		
		return readout_both_panel;
	}		

	/**
	********************************************************************************
	*	This method creates the serial readout panel.
	*
	*	@return Returns the created panel.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_serial_readout_panel()
	{
		JPanel serial_readout_panel = new JPanel();

                GridBagLayout gbl = new GridBagLayout();
                GridBagConstraints gbc = new GridBagConstraints();
                serial_readout_panel.setLayout(gbl);

		// Add the top panel.
		top_serial_readout_panel = create_top_readout_serial_panel();
		add_components(serial_readout_panel, top_serial_readout_panel, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 0, 1, 1);
		
		// Add the bottom panel.
		bottom_readout_panel = create_bottom_readout_panel();
		add_components(serial_readout_panel, bottom_readout_panel, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 1, 1, 1);

		// Initialize the radio buttons. This has to be done here since we are adding
		// two panels and not just one.
		serial_readout_L_rbutton.doClick();
		
		return serial_readout_panel;
	}

	/**
	********************************************************************************
	*	This method creates the Coadder panel.
	*
	*	@return Returns the created panel.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_coadder_panel() 
	{
		JPanel coadder_panel = new JPanel();

		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		coadder_panel.setLayout(gbl);

		// Create the components of coadder checkboxes, labels and textfields.
		download_coadder_board_chkbox = new JCheckBox("Download Coadder Board DSP Code");
		number_of_coadder_chkbox = new JCheckBox("Set Number of Coadds per Readout");
		nc_label = new JLabel("Number of Coadds (NC):");
		range_of_nc_label = new JLabel("0 < NC < 65536");
		nc_txtfld = new VTextField("1", 5);
		nc_txtfld.setEditable(true);

		// Set the color of labels and checkboxes black.
		download_coadder_board_chkbox.setForeground(Color.black);
		number_of_coadder_chkbox.setForeground(Color.black);
		nc_label.setForeground(Color.black);
		range_of_nc_label.setForeground(Color.black);

		// Add the components to the panel.
		add_components(coadder_panel, download_coadder_board_chkbox, gbl, gbc, 0, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 2, 1);
		add_components(coadder_panel, number_of_coadder_chkbox, gbl, gbc, 0, 0, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 1, 2, 1);
		add_components(coadder_panel, nc_label, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 2, 1, 1);
		add_components(coadder_panel, nc_txtfld, gbl, gbc, 0, 10, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 2, 1, 1);
		add_components(coadder_panel, range_of_nc_label, gbl, gbc, 0, 0, 40, 20, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 3, 2, 1);

		// Add the event listeners.
		download_coadder_board_chkbox.addItemListener(controller_config_listener);
		number_of_coadder_chkbox.addItemListener(controller_config_listener);

		return coadder_panel;
	}

	/**
	********************************************************************************
	*	This method creates the Gain panel.
	*
	*	@return Returns the created panel.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_gain_panel() 
	{
		JPanel gain_panel = new JPanel();

		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		gain_panel.setLayout(gbl);

		// Create the components of labels.
		JLabel video_gain_label = new JLabel("Video Gain:");
		JLabel integrator_speed_label = new JLabel("Integrator Speed:");

		// Set the foreground color for the label components.
		video_gain_label.setForeground(Color.black);
		integrator_speed_label.setForeground(Color.black);

		// Create buttonGroup for display of radioButtons for the vidoe gain.
		ButtonGroup gain_rbutton_group = new ButtonGroup();

		// Create the components of labels and radiobuttons for the gain tab.
		gain_1_rbutton = new JRadioButton("1.0", false);
      	 	gain_2_rbutton = new JRadioButton("2.0", false);
		gain_475_rbutton = new JRadioButton("4.75", false);
      		gain_95_rbutton = new JRadioButton("9.5", true);

		// Add the radioButtons to the gain_rbutton_group buttonGroup.
		gain_rbutton_group.add(gain_1_rbutton);
		gain_rbutton_group.add(gain_2_rbutton);
		gain_rbutton_group.add(gain_475_rbutton);
		gain_rbutton_group.add(gain_95_rbutton);

		// Set the foreground color for the radioButton components.
		gain_1_rbutton.setForeground(Color.black);
		gain_2_rbutton.setForeground(Color.black);
		gain_475_rbutton.setForeground(Color.black);
		gain_95_rbutton.setForeground(Color.black);

		// Add the actionListener to the radioButton components.
		gain_1_rbutton.addActionListener(controller_config_listener);
		gain_2_rbutton.addActionListener(controller_config_listener);
		gain_475_rbutton.addActionListener(controller_config_listener);
		gain_95_rbutton.addActionListener(controller_config_listener);

		// Create buttonGroup for display of radioButtons for the integrator speed.
		ButtonGroup integrator_speed_rbutton_group = new ButtonGroup();

		// Create the components of labels and radiobuttons for the gain tab.
		slow_speed_rbutton = new JRadioButton("Slow", false);
      	 	fast_speed_rbutton = new JRadioButton("Fast", true);

		// Add the radioButtons to the gain_rbutton_group buttonGroup.
		integrator_speed_rbutton_group.add(slow_speed_rbutton);
		integrator_speed_rbutton_group.add(fast_speed_rbutton);

		// Set the foreground color for the radioButton components.
		slow_speed_rbutton.setForeground(Color.black);
		fast_speed_rbutton.setForeground(Color.black);

		// Add the components to the panel.
		add_components(gain_panel, video_gain_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 1, 1);
		add_components(gain_panel, gain_1_rbutton, gbl, gbc, 0, 5, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 0, 1, 1);
		add_components(gain_panel, gain_2_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 0, 1, 1);
		add_components(gain_panel, gain_475_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 0, 1, 1);
		add_components(gain_panel, gain_95_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 0, 1, 1);
		add_components(gain_panel, integrator_speed_label, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 1, 1, 1);
		add_components(gain_panel, slow_speed_rbutton, gbl, gbc, 10, 5, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 1, 1, 1);
		add_components(gain_panel, fast_speed_rbutton, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 1, 1, 1);

		// Add the actionListener to the radioButton components.
		slow_speed_rbutton.addActionListener(controller_config_listener);
		fast_speed_rbutton.addActionListener(controller_config_listener);


		// Initialize the components.

		gain_95_rbutton.doClick();

		fast_speed_rbutton.doClick();

		return gain_panel;
	}

        /**
        ********************************************************************************
        *       This method creates the Video Offsets panel.
        *
        *       @return Returns the created panel.
        *
        *       @see JPanel
        *       @version 1.00
        *       @author Xiangyang (Helena) Xian
        ********************************************************************************
        */
        private JPanel create_video_offsets_panel()
        {
                JPanel video_offsets_panel = new JPanel();

                // Set the panel layout.
                GridBagLayout gbl = new GridBagLayout();
                GridBagConstraints gbc = new GridBagConstraints();
                video_offsets_panel.setLayout(gbl);

                // Create the label components.
                JLabel number_of_readout_channels_label = new JLabel("Number of Readout Channels: ");
                JLabel range_of_number_of_readout_channels_label = new JLabel("(1 =< N <= 32)");
                JLabel offset_channel_label = new JLabel("Channel Numbers and Their Offset Values:");
                JLabel range_of_offsets_label = new JLabel("(0 =< offset <= 4095)");

                // Set the foreground color for the label components.
                number_of_readout_channels_label.setForeground(Color.black);
                range_of_number_of_readout_channels_label.setForeground(Color.black);
                offset_channel_label.setForeground(Color.black);
                range_of_offsets_label.setForeground(Color.black);

                // Create string texts for the number of readout channel comboBox.
                String[] number_of_readout_channels = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31", "32"};

                // Create a JComboBox for the the number of readout channel.
                // And add the string text to the comboBox.
                number_of_readout_channels_combo_box = new JComboBox(number_of_readout_channels);

                // Set JComboBox editable to true.
                number_of_readout_channels_combo_box.setEditable(true);

                // Set the size of the number_of_readout_channels_combo_box to 1 character big.
                number_of_readout_channels_combo_box.setPreferredSize(new Dimension(45, 20));

                // Set the video offset action command.
                number_of_readout_channels_combo_box.setActionCommand("Video Offsets");
	
		// Set the number of channel combobox to 2 as default. 
		number_of_readout_channels_combo_box.setSelectedItem("2");

		// Create the label and textfield components.
		for (int i = 0; i < channel_txtfld.length; i++) {
			channel_label[i] = new JLabel(i + ": ");
			channel_label[i].setForeground(Color.black);
			channel_txtfld[i] = new VTextField("0", 4);

			// Disable all channels.
			channel_label[i].setVisible(false);
			channel_txtfld[i].setVisible(false);
			channel_txtfld[i].setEnabled(false);
		}

		// Only first two channels are shown as default.
		channel_label[0].setVisible(true);
		channel_txtfld[0].setVisible(true);
		channel_txtfld[0].setEnabled(true);

		channel_label[1].setVisible(true);
		channel_txtfld[1].setVisible(true);
		channel_txtfld[1].setEnabled(true);

                // Add the components to the panel.
                add_components(video_offsets_panel, number_of_readout_channels_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 5, 1);
                add_components(video_offsets_panel, number_of_readout_channels_combo_box, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 0, 1, 1);
		add_components(video_offsets_panel, range_of_number_of_readout_channels_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 6, 0, 2, 1);
                add_components(video_offsets_panel, offset_channel_label, gbl, gbc, 20, 0, 5, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 1, 8, 1);
                add_components(video_offsets_panel, channel_label[0], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 2, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[0], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 2, 1, 1);
                add_components(video_offsets_panel, channel_label[1], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 2, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[1], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 2, 1, 1);
                add_components(video_offsets_panel, channel_label[2], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 2, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[2], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 2, 1, 1);
                add_components(video_offsets_panel, channel_label[3], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 6, 2, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[3], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 7, 2, 1, 1);
                add_components(video_offsets_panel, channel_label[4], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 3, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[4], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 3, 1, 1);
                add_components(video_offsets_panel, channel_label[5], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 3, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[5], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 3, 1, 1);
                add_components(video_offsets_panel, channel_label[6], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 3, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[6], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 3, 1, 1);
                add_components(video_offsets_panel, channel_label[7], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 6, 3, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[7], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 7, 3, 1, 1);
                add_components(video_offsets_panel, channel_label[8], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 4, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[8], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 4, 1, 1);
		add_components(video_offsets_panel, channel_label[9], gbl, gbc, 0, 7, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 4, 1, 1);                
		add_components(video_offsets_panel, channel_txtfld[9], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 4, 1, 1);
                add_components(video_offsets_panel, channel_label[10], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 4, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[10], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 4, 1, 1);
                add_components(video_offsets_panel, channel_label[11], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 6, 4, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[11], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 7, 4, 1, 1);
                add_components(video_offsets_panel, channel_label[12], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 5, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[12], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 5, 1, 1);
                add_components(video_offsets_panel, channel_label[13], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 5, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[13], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 5, 1, 1);
                add_components(video_offsets_panel, channel_label[14], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 5, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[14], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 5, 1, 1);
                add_components(video_offsets_panel, channel_label[15], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 6, 5, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[15], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 7, 5, 1, 1);
		add_components(video_offsets_panel, channel_label[16], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 6, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[16], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 6, 1, 1);
                add_components(video_offsets_panel, channel_label[17], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 6, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[17], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 6, 1, 1);
                add_components(video_offsets_panel, channel_label[18], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 6, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[18], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 6, 1, 1);
                add_components(video_offsets_panel, channel_label[19], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 6, 6, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[19], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 7, 6, 1, 1);
                add_components(video_offsets_panel, channel_label[20], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 7, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[20], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 7, 1, 1);
                add_components(video_offsets_panel, channel_label[21], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 7, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[21], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 7, 1, 1);
                add_components(video_offsets_panel, channel_label[22], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 7, 1, 1);
		add_components(video_offsets_panel, channel_txtfld[22], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 7, 1, 1);
                add_components(video_offsets_panel, channel_label[23], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 6, 7, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[23], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 7, 7, 1, 1);
                add_components(video_offsets_panel, channel_label[24], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 8, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[24], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 8, 1, 1);
                add_components(video_offsets_panel, channel_label[25], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 8, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[25], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 8, 1, 1);
                add_components(video_offsets_panel, channel_label[26], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 8, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[26], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 8, 1, 1);
                add_components(video_offsets_panel, channel_label[27], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 6, 8, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[27], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 7, 8, 1, 1);
                add_components(video_offsets_panel, channel_label[28], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 9, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[28], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 9, 1, 1);
		add_components(video_offsets_panel, channel_label[29], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 9, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[29], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 9, 1, 1);
                add_components(video_offsets_panel, channel_label[30], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 9, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[30], gbl, gbc, 0, 0, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 9, 1, 1);
                add_components(video_offsets_panel, channel_label[31], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 6, 9, 1, 1);
                add_components(video_offsets_panel, channel_txtfld[31], gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 7, 9, 1, 1);
		add_components(video_offsets_panel, range_of_offsets_label, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 10, 8, 1);

                // Add ItemListener to the number_of_readout_channels_combo_box.
                number_of_readout_channels_combo_box.addItemListener(controller_config_listener);

                return video_offsets_panel;
        }

	/**
	********************************************************************************
	*	This method creates a new tabbed pane - Temperature.
	*
	*	@return Returns the tabbed pane.
	*
	*	@see JTabbedPane JPanel
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JPanel create_temperature_panel() 
	{
		JPanel temperature_panel = new JPanel();

		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		temperature_panel.setLayout(gbl);

		// Create the checkbox components.
		array_temperature_chkbox = new JCheckBox("Array Temperature: ");

		// Create the label components.
		JLabel degree_C_label = new JLabel(" C");
		linear_coeff_description_label1 = new JLabel("Please supply the following values:");
		linear_coeff_description_label2 = new JLabel("T = (Coeff #1) + (Coeff #2)*adu");
		linear_coeff1_label = new JLabel("Coeff #1: ");
		linear_coeff2_label = new JLabel("Coeff #2: ");
		
		// Set the foreground color for the label components.
		degree_C_label.setForeground(Color.black);
		linear_coeff_description_label1.setForeground(Color.black);
		linear_coeff_description_label2.setForeground(Color.black);
		linear_coeff1_label.setForeground(Color.black);
		linear_coeff2_label.setForeground(Color.black);

		// Create the textfields.
		temperature_txtfld = new VTextField("0", 4);
		linear_coeff1_txtfld = new VTextField("0", 7);
		linear_coeff2_txtfld = new VTextField("0", 7);
		
		// Add the components to the panel.
		add_components(temperature_panel, array_temperature_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 1, 1);
		add_components(temperature_panel, temperature_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 0, 1, 1);
		add_components(temperature_panel, degree_C_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 0, 1, 1);
		add_components(temperature_panel, linear_coeff_description_label1, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 1, 3, 1);
		add_components(temperature_panel, linear_coeff_description_label2, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 2, 3, 1);
		add_components(temperature_panel, linear_coeff1_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 3, 1, 1);
		add_components(temperature_panel, linear_coeff1_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 3, 2, 1);
		add_components(temperature_panel, linear_coeff2_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 4, 1, 1);
		add_components(temperature_panel, linear_coeff2_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 4, 2, 1);

	
	array_temperature_chkbox.addItemListener(controller_config_listener);

		return temperature_panel;
	}

	/**
	********************************************************************************
	*	This method creates the menu bar.
	*
	*	@return Returns the menubar.
	*
	*	@see JMenuBar
	*	@version 1.00
	*	@author Xiangyang (Helena) Xian 
	********************************************************************************
	*/
	private JMenuBar create_menu_bar()
	{
		JMenuBar menu_bar = new JMenuBar();

		// Set the menubar's border.
		menu_bar.setBorder(new BevelBorder(BevelBorder.RAISED));
		menu_bar.setBorderPainted(true);
		
		// Add the Window menu to the menu bar.
		JMenu window_menu = new JMenu("Window");
		menu_bar.add(window_menu);
		
		// Add the submenus to the Window menu.
		JMenuItem close_item = new JMenuItem("Close", 'C');
		window_menu.add(close_item);
				
		// Set the menubar font.
		window_menu.setFont(font);
		close_item.setFont(font);
		
		// Add the event listeners to the menu items.
		close_item.addActionListener(controller_config_listener);
		
		return menu_bar;
	}

	/**
	*****************************************************************************
	*	Adds the specified component to the specified panel.
	*
	*	@param panel The panel to which the component will be added.
	*	@param compnt The component to add.
	*	@param gbl The GridBagLayout of the panel.
	*	@param gbl The GridBagConstraints of the panel.
	*	@param insetTOP The distance in pixels between the top of this
	*			component and the one above it.
	*	@param insetBOTTOM The distance in pixels between the bottom of
	*			this component and the one below it.
	*	@param insetLEFT The distance in pixels between the left side of
	*			this component and the one left of it.
	*	@param insetRIGHT The distance in pixels between the right side
	*			of this component and the one right of it.
	*	@param cfill Determines how the component "fills" the space of
	*		  	of its "cell".
	*	@param anchr The justification of the component within its "cell".
	*	@param col The x position of the component.
	*	@param row The y position of the component.
	*	@param gw The number of "cells" that the component spans in the
	*			width direction.
	*	@param gh The number of "cells" that the component spans in the
	*			height direction.
	*
	*	@see JPanel
	*	@see JComponent
	*	@see GridBagLayout
	*	@see GridBagConstraints
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
   	private void add_components(JPanel panel, JComponent compnt, GridBagLayout gbl, GridBagConstraints gbc, int insetTOP, int insetLEFT, int insetBOTTOM, int insetRIGHT, int cfill, int anchr, int col, int row, int gw, int gh)
   	{
      		// Set the grid layout constraints.
      		gbc.fill = cfill;
      		gbc.anchor = anchr;
      		gbc.insets = new Insets(insetTOP, insetLEFT, insetBOTTOM, insetRIGHT);
      		gbc.gridx = col;
      		gbc.gridy = row;
      		gbc.gridwidth = gw;
      		gbc.gridheight = gh;
      		
      		// Set the compontent font.
      		compnt.setFont(font);
      
      		// Add the component to the panel.
      		gbl.setConstraints(compnt, gbc);
      		panel.add(compnt);
      	}

	/**
	********************************************************************************
	*	This method sets the supported/unsupported configuration text to the
	*	specified string, which is constructed as follows:  bit#  text
	*
	*	@param bit The bit number of the supported/unsupported configuration.
	*	@param text The string to set.
	*	@param supported Determines whether the text goes into the supported
	*			 (true) or unsupported (false) text areas.
	*
	*	@version 1.00
	*	@author Scott Streit 
	********************************************************************************
	*/
	private void set_supported_text(String text)
	{
		supported_capabilities_txtarea.append(text + "\n");
	}

	/**
	********************************************************************************
	*	This method sets the apply button text to the specified string.
	*
	*	@param text The string to set the apply button text to.
	*
	*	@see JButton
	*	@version 1.00
	*	@author Scott Streit 
	********************************************************************************
	*/
      	public void set_apply_button_text(String text)
      	{
		apply_button.setText(text);
      	}

	/**
	********************************************************************************
	*	This method sets the apply all button text to the specified string.
	*
	*	@param text The string to set the apply all button text to.
	*
	*	@see JButton
	*	@version 1.00
	*	@author Scott Streit 
	********************************************************************************
	*/
      	public void set_apply_all_button_text(String text)
      	{
		apply_all_button.setText(text);
      	}

	/**
	********************************************************************************
	*	This method performs a software "click" of the "Apply All" button. It 
	*	does the same thing as if the button were "clicked" by
a user with a
	*	mouse.
	*	
	*	@see JButton
	*	@version 1.00
	*	@author Scott Streit 
	********************************************************************************
	*/
      	public void clickApplyAll()
      	{
		apply_all_button.doClick();
      	}
}
