/**
********************************************************************************
*	This class contains all the DSP commands used to control the
*	PCI, Timing, and Utility boards. Camera control operations
*	that provide the DSP with information, such as CCD dimensions,
*	are also contained within this class.
*
*	CVM - Model
*
*	@see DspCommand
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import javax.swing.*;

public class DspCommand2
{
	private int arg_and_board_dest;
	private int memory_type;
    	public static boolean abort;

	/**
	*****************************************************************************
	*	Default Constructor.
	*****************************************************************************
	*/	
	public DspCommand2() {}

	/**
	*****************************************************************************
	*	This method supplies the PCI board DSP with two pieces
	*	of information: 1) the number of arguments supplied with
	*	the next command to be issued, 2) the board destination
	*	(PCI, Timing or Utility).
	*
	*	@param number_of_args    The number of arguments supplied with
	*			         the next command to be issued.
	*	@param board_destination The board destination (PCI, Timing or Utility).
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON"
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void set_board_destination(int number_of_args, int board_destination) throws ReplyValueException, Exception
	{
		int reply_value = 0;

		if (abort) return;

		arg_and_board_dest = (number_of_args | board_destination);	// 0x30000 = Number of arguments (ARG#) << 16.
		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_DESTINATION, arg_and_board_dest);
		
		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			throw new ReplyValueException(rve.get_value());
		}
	}

	/**
	*****************************************************************************
	*	This method sets the DSP memory type for the board specified
	*	by the <I>set_board_destination()</I> method. The memory
	*	spaces are: D, P, R, X, Y.
	*
	*	@param type One of the following DSP memory types: D, P, R, X, Y.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void set_type(int type)
	{
		memory_type = type;
	}

	/**
	*****************************************************************************
	*	Set the HCTR to a new value.
	*
	*	@param value The value to be set.
	*
	*	@return None.
	*
	*	@throws Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@see Exception
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void set_hctr(int value) throws Exception
	{
		if (abort) return;

		// Set the HCTR to the new value.
		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCTR, value);
		
	}

	/**
	*****************************************************************************
	*	Get the current HCTR value.
	*
	*	@return The current value of the HCTR.
	*
	*	@throws Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@see Exception
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public int get_hctr() throws Exception
	{
		int reply_value = 0;
		
		if (abort) { return 0; }

		// Get the current HCTR value.
		reply_value = VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_GET_HCTR, 0);
		
		return reply_value;

	}

	/**
	*****************************************************************************
	*	Send a command to the PCI DSP via the vector register. The reply
	*	returned from the PCI DSP is checked against the specified expected
	*	reply value.
	*
	*	@param command The command to be sent to the PCI DSP.
	*
	*	@return None.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON"
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void hcvr(int command, int expected_reply) throws ReplyValueException, Exception
	{
		int reply_value = 0;
		
		if (abort) return;

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, command);
		
		if (abort) return;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(expected_reply);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) return;

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "SET HCVR", command, reply_value);
		}
	}

	/**
	*****************************************************************************
	*	Send a command to the PCI DSP via the vector register. This version
	*	of the method returns a requested value or ERR. No expected reply
	*	value checking is done with this method.
	*
	*	@param command The command to be sent to the PCI DSP.
	*
	*	@return A requested value or ERR.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON"
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public int hcvr(int command) throws ReplyValueException, Exception
	{
		int reply_value = 0;
		
		if (abort) return 0;

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, command);

		if (abort) return 0;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply();
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) return 0;

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "SET HCVR", command, reply_value);
		}
		
		return reply_value;
	}

	/**
	*****************************************************************************
	*	Set one of the five PCI DSP argument registers to the specified value.
	*
	*	@param arg_number The argument register number.
	*	@param arg_value The value to which the specified argument register
	*			 should be set.
	*
	*	@return None.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON"
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void arg(int arg_number, int arg_value) throws ReplyValueException, Exception
	{
		int reply_value = 0;
		
		if (abort) return;

		switch (arg_number) {
			case 1:
				VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, arg_value);
				break;
			case 2:
				VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG2, arg_value);
				break;
			case 3:
				VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG3, arg_value);
				break;
			case 4:
				VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG4, arg_value);
				break;
			case 5:
				VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG5, arg_value);
				break;
		}
		
		if (abort) return;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			throw new ReplyValueException(rve.get_value());
		}
	}

	/**
	*****************************************************************************
	*	Send a manual command to the PCI DSP.
	*
	*	@param command The command to be sent to the PCI DSP.
	*
	*	@return The value returned by sending the command.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board is in error or times out.
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public int cmdr(int command) throws ReplyValueException, Exception
	{
		int reply_value = 0;
		
		if (abort) return 0;

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_CMDR, command);		

		if (abort) return 0;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply();
		}
		catch (ReplyValueException rve) {
			throw new ReplyValueException(rve.get_value());
		}

		if (reply_value == VoodooConstants.ERR || reply_value == VoodooConstants.REPLY_BUFFER_EMPTY)
			throw new ReplyValueException(reply_value);

		return reply_value;
	}

	/**
	*****************************************************************************
	*	Test Data Link (TDL) method. Used to test communications to
	*	the PCI, Timing, and Utility boards. A data value is sent to
	*	the board specified by the <I>set_board_destination()</I>
	*	method. The return value is is checked against the sent value.
	*
	*	@param data Value to be sent to the specified board.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equal <I>data</I>.
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return Returns the ascii sequence "DON" if successful.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public int test_data_link(int data) throws ReplyValueException, Exception
	{
		int reply_value = 0;
				
		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, data);
		
		if (abort) { return 0; }

		// Read the reply.		
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();
			throw new ReplyValueException(rve.get_value());
		}
		
		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.TEST_DATA_LINK);
		
		if (abort) { return 0; }

		// Read the reply.		
		try {
			reply_value = ReplyBuffer.read_reply(data);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return 0; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "TEST DATA LINK", VoodooConstants.TEST_DATA_LINK, data, 0, 0, 0, 0, reply_value);
		}
		
		return reply_value;
	}

	/**
	*****************************************************************************
	*	Read Memory (RDM) method. Used to read a DSP memory location
	*	from one of the boards (PCI, Timing, or Utility).
	*
	*	@param address Address (in hex) within DSP of the specified
	*			board.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board equals the ascii sequence "ERR".
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return Returns the data value located at <I>address</I>.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public int read_memory(int address) throws ReplyValueException, Exception
	{
		int reply_value = 0;

		if (abort) { return 0; }
		
		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, memory_type);
		
		if (abort) { return 0; }

		// Read the reply.		
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();
			throw new ReplyValueException(rve.get_value());
		}
		
		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG2, address);

		if (abort) { return 0; }

		// Read the reply.		
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			throw new ReplyValueException(rve.get_value());
		}
		
		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.READ_MEMORY);

		if (abort) { return 0; }

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply();
		}
		catch (ReplyValueException rve) {}
		
		finally {
		
			// Check that the reply is not an ERRor.
			if  (reply_value == VoodooConstants.ERR)
				throw new ReplyValueException(reply_value);
		
			if (abort) { return 0; }

			// Print command description to Info/Status window if debug mode is "on".
			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "READ MEMORY", VoodooConstants.READ_MEMORY, memory_type, address, 0, 0, 0, reply_value);
		}
			
		return reply_value;
	}

	/**
	*****************************************************************************
	*	Write Memory (WRM) method. Used to write to a DSP memory location
	*	on one of the boards (PCI, Timing, or Utility). Writes the specified
	*	<I>data</I> value located at <I>address</I>. The memory type is not
	*	a parameter because on successive writes you do not need to repeat
	*	the memory type.
	*
	*	@param address Address (in hex) within DSP of the specified
	*			board.
	*
	*	@param data Value to be written to the specified board.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON".
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return Returns ascii sequence "DON" if successfull.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public int write_memory(int address, int data) throws ReplyValueException, Exception
	{
		int reply_value = 0;

		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, memory_type);
		
		if (abort) { return 0; }

		// Read the reply.		
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			throw new ReplyValueException(rve.get_value());
		}
		
		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG2, address);
		
		if (abort) { return 0; }

		// Read the reply.		
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			throw new ReplyValueException(rve.get_value());
		}
		
		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG3, data);
		
		if (abort) { return 0; }

		// Read the reply.		
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			throw new ReplyValueException(rve.get_value());
		}
		
		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.WRITE_MEMORY);
		
		if (abort) { return 0; }

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();
			throw new ReplyValueException(rve.get_value());
		}
			
		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return 0; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "WRITE MEMORY", VoodooConstants.WRITE_MEMORY, memory_type, address, data, 0, 0, reply_value);
		}
		
		return reply_value;
	}
	
	/**
	*****************************************************************************
	*	Read image (RDI) method. Used to readout an image from the controller
	*	into the user application.
	*
	*	@param byte_count. The number of bytes to readout.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON".
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return Returns ascii sequence "DON" if successfull.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public int read_image(int byte_count, int exposure_time, int mem_fd) throws ReplyValueException, Exception
	{
		int reply_value = 0;
				
		if (abort) { return 0; }

		// Check for exposure time greater than 5 seconds.
		if (exposure_time > 5000) {
			VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.READ_IMAGE);
		}
		
		if (abort) { return 0; }

		// Transfer the image from PCI into the image buffer.
		try {
				VoodooSetup.Cread_image(VoodooSetup.pci_fd, mem_fd, byte_count);
  		}
    		catch (Exception e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Device Driver Error", JOptionPane.ERROR_MESSAGE);
			return -1;
		}
		
		if (abort) { return 0; }

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return 0; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "READ IMAGE", VoodooConstants.READ_IMAGE, reply_value);
		}
	
		return reply_value;
	}

	/**
	*****************************************************************************
	*	Dimensions method. Used to set the image dimensions.
	*
	*	@param ccd_rows The number of image rows.
	*
	*	@param ccd_cols The number of image columns.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON".
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return None
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void dimensions(int array_rows, int array_columns) throws ReplyValueException, Exception
	{
		int reply_value = 0;
		
		if (abort) return;

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_NCOLS, array_columns);

		if (abort) return;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "SET COLUMNS", VoodooConstants.ASTROPCI_SET_NCOLS, reply_value);
		}
		if (abort) return;

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_NROWS, array_rows);

		if (abort) return;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "SET ROWS", VoodooConstants.ASTROPCI_SET_NROWS, reply_value);
		}
	}

	/**
	*****************************************************************************
	*	Binning method. Used to set the image binning parameters.
	*
	*	@param binning_rows The number of pixels to bin in the rows direction.
	*
	*	@param binning_cols The number of pixels to bin in the columns direction.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON".
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return None
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void binning(int binning_rows, int binning_columns) throws ReplyValueException, Exception
	{
		int reply_value = 0;
		
		if (abort) return;

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_BINNING_COLS, binning_columns);

		if (abort) return;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "SET BINNING COLS", VoodooConstants.ASTROPCI_SET_BINNING_COLS, reply_value);
		}

		if (abort) return;

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_BINNING_ROWS, binning_rows);

		if (abort) return;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "SET BINNING ROWS", VoodooConstants.ASTROPCI_SET_BINNING_ROWS, reply_value);
		}
	}

	/**
	*****************************************************************************
	*	Read exposure time method. Used to read the current elapsed exposure
	*	time.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board equals the ascii sequence "ERR".
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return Returns the current elapsed exposure time.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/		
	public int read_expose_time() throws ReplyValueException, Exception
	{
		int reply_value = 0;
		
		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.READ_EXPOSURE_TIME);

		if (abort) { return 0; }

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply();
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return 0; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "READ EXPOSURE TIME", VoodooConstants.READ_EXPOSURE_TIME, reply_value);
		}

		return reply_value;
	}

	/**
	*****************************************************************************
	*	Used to set the exposure time to a new value.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equal the ascii sequence "DON".
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return Returns the current elapsed exposure time.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/		
	public int set_exposure_time(int exposure_time) throws ReplyValueException, Exception
	{
		int reply_value = 0;
		
		if (abort) { return 0; }
		
		// Send the exposure time to the utility board.
		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_EXPTIME, exposure_time);

		if (abort) { return 0; }
			
		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return 0; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "SET EXPOSURE TIME", VoodooConstants.ASTROPCI_SET_EXPTIME, reply_value);
		}
		
		return reply_value;
	}

	/**
	*****************************************************************************
	*	Idle method. Used to put the controller into an idle state or take
	*	the controller out of the idle state.
	*
	*	@param mode True to put the controller into an idle state. False to
	*		    take the controller out of the idle state.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON"
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return None.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void idle(boolean mode) throws ReplyValueException, Exception
	{
		int reply_value = 0;
		
		if (abort) return;

		// Resume idle mode.
		if (mode)
			VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.RESUME_IDLE_MODE);

		// Stop idle mode.
		else
			VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.STOP_IDLE_MODE);

	
		if (abort) return;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) return;

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON) {
				if (mode)
					PrintMessage.debug_message(arg_and_board_dest, "RESUME IDLE MODE", VoodooConstants.RESUME_IDLE_MODE, reply_value); 
				else
					PrintMessage.debug_message(arg_and_board_dest, "STOP IDLE MODE", VoodooConstants.STOP_IDLE_MODE, reply_value); 
			}
		}
	}

	/**
	*****************************************************************************
	*	Gain method. Used to set the controller readout gain and A/D
	*	conversion speed.
	*
	*	@param gain The readout gain.
	*	@param speed The A/D conversion speed. Can be "fast" or "slow".
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON"
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return None.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_gain(int gain, int speed) throws ReplyValueException, Exception
	{
		ManualCommand manual_cmd = new ManualCommand();
		int reply = 0;
		
		if (abort) return;
		
		manual_cmd.set_board_destination(VoodooConstants.TIM_ID);
		manual_cmd.set_arg1(gain);
		manual_cmd.set_arg2(speed);
		manual_cmd.set_command(VoodooConstants.SGN);
		
		if (abort) return;

		// Send the gain command.
		manual_cmd.thread_it();

		if (abort) return;
		
		// Get the reply value and use only the lower 6 bits.
		Integer Integer_reply = (Integer)manual_cmd.get();
		reply = Integer_reply.intValue() & 0x00FFFFFF;

		// Set the reply textfield to the returned value.
		if (reply == VoodooConstants.ERR || reply == VoodooConstants.SYR)
			throw new ReplyValueException(reply);

		else if (reply == VoodooConstants.TOUT)
			throw new ReplyValueException(reply);

		else if (reply == VoodooConstants.REPLY_BUFFER_EMPTY)		
			throw new ReplyValueException(reply);
			
		manual_cmd = null;
	}
			
	/**
	*****************************************************************************
	*	Load application method. Used to load a timing or utility board
	*	application.
	*
	*	@param app_number The number of the application to load.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON"
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return None.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void load_application(int app_number) throws ReplyValueException, Exception
	{
		int reply_value = 0;

		if (abort) return;

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, app_number);

		if (abort) return;

		// Read the reply.		
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			throw new ReplyValueException(rve.get_value());
		}
		
		if (abort) { return; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.LOAD_APPLICATION);

		if (abort) return;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) return;

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "LOAD APPLICATION", VoodooConstants.LOAD_APPLICATION, app_number, 0, 0, 0, 0, reply_value);
		}
	}

	/**
	*****************************************************************************
	*	Set temperature method. Used to set the detectors target temperature.
	*
	*	@param temperature The detector temperature (in Celcius).
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON"
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return None.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_temperature(double temperature) throws ReplyValueException, Exception
	{
		int reply_value = 0;
		ArrayTemperature array_temperature = new ArrayTemperature();

		if (abort) return;

		reply_value = array_temperature.calculate_adu(temperature);
		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, reply_value);
		reply_value = 0;

		if (abort) return;

		// Read the reply.		
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			throw new ReplyValueException(rve.get_value());
		}
		
		if (abort) { return; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.SET_ARRAY_TEMPERATURE);

		if (abort) return;

		// Read the reply.
		try {
			reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			reply_value = rve.get_value();	// Used by debug (below)
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) return;

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "SET ARRAY TEMPERATURE", VoodooConstants.SET_ARRAY_TEMPERATURE, (int)temperature, 0, 0, 0, 0, reply_value);
		}
	}

	/**
	*****************************************************************************
	*	Get temperature method. Used to obtain the current array temperature
	*	from the controller.
	*
	*	@throws ReplyValueException If the value returned by the
	*		specified board does not equals the ascii sequence "DON"
	*		Exception Thrown if an ioctl error occurs.
	*
	*		Note: The exceptions are re-thrown because the debug
	*		      messages would not be printed in the correct
	*		      locations amoung the normal message otherwise.
	*
	*	@return Returns the current array temperature.
	*
	*	@see ReplyValueException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public double get_temperature() throws ReplyValueException, Exception
	{
		int adu = 0;
		ArrayTemperature array_temperature = new ArrayTemperature();
				
		if (abort) { return 0; }

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.READ_ARRAY_TEMPERATURE);
		
		if (abort) { return 0; }

		// Read the reply. The adu is 12-bits, so mask off to get the bits.
		try {
			adu = ReplyBuffer.read_reply() & 0x00000FFF;
		}
		catch (ReplyValueException rve) {
			rve.get_value();
			throw new ReplyValueException(rve.get_value());
		}

		// Print command description to Info/Status window if debug mode is "on".
		finally {
			if (abort) { return 0; }

			if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
				PrintMessage.debug_message(arg_and_board_dest, "READ ARRAY TEMPERATURE", VoodooConstants.READ_ARRAY_TEMPERATURE, adu); 
		}

		return array_temperature.calculate_temperature(adu);
	}

	/**
	*****************************************************************************
	*	Flush PCI board reply buffer. This method flushes the PCI board reply
	*	buffer.
	*
	*	@return None.
	*
	*	@see Exception
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public static void flush_reply_buffer() throws Exception
	{
		if (abort) return;

		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_FLUSH_REPLY_BUFFER, 0);
	}
}