/**
********************************************************************************
*	The class is responsible for taking an image exposure, reading out the
*	image, displaying the image, saving the image, and checking the image
*	data if the it is a DSP synthetically generated. This class is executed
*	inside of a SwingWorker thread class.
*
*	CVM - Model
*
*	@see VoodooMainWindow
*	@see SwingWorker
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import javax.swing.*;
import java.io.*;

public class ExposureCommand extends SwingWorker
{
	protected VoodooMainWindow parent = null;
	protected DspCommand2 dsp_cmd;
	protected Fits image;
	protected int byte_count;
	protected int rows;
	protected int cols;
	protected int elapsed_time;
	protected int start_increment = 1;
	protected int last_increment = 1;
	protected int increment = 1;
	protected int exposure_time;
	protected int image_resolution;
	protected String image_filename;
	protected String image_filename_prefix;
	protected String image_filename_suffix;
	protected boolean do_image_save;
	protected boolean do_multiple_exposures;
	protected boolean do_auto_increment;
	protected boolean do_delay;
	protected boolean do_display;
	protected boolean do_beep;
	protected boolean do_deinterlacing;
	protected boolean do_open_shutter = true;
	protected boolean pause;
	protected boolean change_exposure_time;
	protected static boolean current_shutter_position = false;	// Static to ensure current value.
	protected int deinterlace_value;
	protected int number_of_exposures;
	protected int delay;
	protected static boolean use_controller_exptime;
	
	/**
	*****************************************************************************
	*	Constructor.
	*
	*	@see VoodooMainWindow
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public ExposureCommand(VoodooMainWindow parent)
	{
		this.parent = parent;
		dsp_cmd = new DspCommand2();
		image = new Fits(parent.fits_dialog);
	}

	/**
	*****************************************************************************
	*	This method aborts the current exposure.
	*
	*	@see VoodooMainWindow
	*	@see SwingWorker
	*	@see DspCommand2
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void abort_exposure()
	{
		abort = true;
		dsp_cmd.abort = true;
		
		try {
			dsp_cmd.hcvr(VoodooConstants.ABORT_EXPOSURE, VoodooConstants.DON);
			dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			PrintMessage.message("ExposureCommand", 1, "ERROR: Could not abort exposure.\n", 0, true);
			PrintMessage.message("ExposureCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
			if (rve.get_value() == VoodooConstants.TOUT)
				PrintMessage.message("ExposureCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
			else
				PrintMessage.message("ExposureCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
		}
		catch (Exception e) {
			PrintMessage.message("ExposureCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
		}
	}

	/**
	*****************************************************************************
	*	This method pauses the current exposure.
	*
	*	@see VoodooMainWindow
	*	@see SwingWorker
	*	@see DspCommand2
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void pause_exposure(boolean pause)
	{
		try {
			if (pause)
				dsp_cmd.hcvr(VoodooConstants.PAUSE_EXPOSURE, VoodooConstants.DON);
			else
				dsp_cmd.hcvr(VoodooConstants.RESUME_EXPOSURE, VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			if (pause)
				PrintMessage.message("ExposureCommand", 1, "ERROR: Could not pause exposure.\n", 0, true);
			else
				PrintMessage.message("ExposureCommand", 1, "ERROR: Could not resume exposure.\n", 0, true);

			PrintMessage.message("ExposureCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
			if (rve.get_value() == VoodooConstants.TOUT)
				PrintMessage.message("ExposureCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
			else
				PrintMessage.message("ExposureCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
		}
		catch (Exception e) {
			PrintMessage.message("ExposureCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
		}
		
		this.pause = pause;
	}

	/**
	*****************************************************************************
	*	This method indicates that the current exposure time should be changed
	*	to the new value.
	*
	*	@see VoodooMainWindow
	*	@see SwingWorker
	*	@see DspCommand2
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void change_exposure_time()
	{
		exposure_time = (int)(1000*Float.parseFloat(parent.exposure_time_txtfld.getText()));
		change_exposure_time = true;
	}

	/**
	*****************************************************************************
	*	This method aborts the current image readout.
	*
	*	@see VoodooMainWindow
	*	@see SwingWorker
	*	@see DspCommand2
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void abort_readout()
	{
		try {
			abort = true;
			dsp_cmd.abort = true;
		
			dsp_cmd.hcvr(VoodooConstants.ABORT_READOUT, VoodooConstants.DON);
			dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			PrintMessage.message("ExposureCommand", 1, "ERROR: Could not abort readout.\n", 0, true);
			PrintMessage.message("ExposureCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
			if (rve.get_value() == VoodooConstants.TOUT)
				PrintMessage.message("ExposureCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
			else
				PrintMessage.message("ExposureCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
		}
		catch (Exception e) {
			PrintMessage.message("ExposureCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
		}
	}
	
	/**
	*****************************************************************************
	*	This method specifies whether or not to save the image.
	*
	*	@param save_to_disk If true, save image to disk. If false, do not
	*			    save the image.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_save_to_disk(boolean save_to_disk)
	{
		do_image_save = save_to_disk;
	}

	/**
	*****************************************************************************
	*	This method specifies whether or not to do multiple images.
	*
	*	@param multiple_exposures If true, do multiple exposures. If false,
	*				  do not do multiple exposures.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_multiple_exposures(boolean multiple_exposures)
	{
		do_multiple_exposures = multiple_exposures;
	}

	/**
	*****************************************************************************
	*	This method specifies whether or not to auto increment the image file.
	*
	*	@param auto_increment If true, auto increment filename. If false,
	*			      do not auto increment filename.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_auto_increment(boolean auto_increment)
	{
		do_auto_increment = auto_increment;
	}

	/**
	*****************************************************************************
	*	This method specifies whether or not to delay before each exposure.
	*
	*	@param delay If true, delay before each exposure. If false,
	*			      do not delay before each exposure.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_delay(boolean delay)
	{
		do_delay = delay;
	}

	/**
	*****************************************************************************
	*	This method specifies whether or not to beep when the exposure is
	*	complete.
	*
	*	@param beep If true, beep after exposure completes. If false, do not
	*		    beep after exposure completes.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_beep(boolean beep)
	{
		do_beep = beep;
	}

	/**
	*****************************************************************************
	*	This method specifies whether or not to display the image after the
	*	exposure completes.
	*
	*	@param display If true, display image after exposure completes. If
	*		       false, do not display image after exposure completes.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_display_image(boolean display)
	{
		do_display = display;
	}

	/**
	*****************************************************************************
	*	This method specifies whether or not to open the shutter during an
	*	exposure.
	*
	*	@param display If true, open the shutter during an exposure. If
	*		       false, do not open the shutter during an exposure.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_open_shutter(boolean open_shutter)
	{
		do_open_shutter = open_shutter;
	}
	
	/**
	*****************************************************************************
	*	This method sets which should be used to control the exposure time
	*	count, the controller or the host CPU.
	*
	*	@param use_controller True means use the controller for exposure time
	*			      count. False means use the host CPU.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public static void set_exposure_controller_mode(boolean use_controller)
	{
		use_controller_exptime = use_controller;
	}

	/**
	********************************************************************************
	*	This method sets the deinterlacing control to on/off.
	*	
	*	@param deinterlacing True to set the perform deinterlacing, false to not
	*			     perform deinterlacing.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_deinterlacing(boolean deinterlacing)
	{
		do_deinterlacing = deinterlacing;
	}

	/**
	********************************************************************************
	*	This method sets the deinterlace method to use. The methods are:
	*	
	*	<br><pre>
	*	1. Single readout from one output.
	*	2. Dual readout with parallel split from two outputs.
	*	3. Dual readout with serial split from two outputs.
	*	4. Quad readout from four outputs.
	*	</pre>
	*
	*	@param value The deinterlace method.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_deinterlace_value(int value)
	{
		deinterlace_value = value;
	}

	/**
	*****************************************************************************
	*	This method sets the exposure time. The input string is parsed as a
	*	float and multiplied by a 1000 to convert the exposure time to
	*	milliseconds.
	*
	*	@param exp_time The exposure time (as a String).
	*
	*	@Throws NumberFormatException If the text cannot be parsed.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_exposure_time(String exp_time) throws NumberFormatException
	{
		try {
			exposure_time = (int)(1000*Float.parseFloat(exp_time));
		}
		catch (NumberFormatException nfe) {
			exposure_time = 0;
			throw new NumberFormatException("ERROR: Cannot parse exposure time textfield");
		}
	}

	/**
	*****************************************************************************
	*	This method sets the pre-exposure delay. The input string is parsed
	*	as an integer and specifies how many seconds to delay before starting
	*	an exposure.
	*
	*	@param delay_time The pre-exposure delay time (as a String).
	*
	*	@Throws NumberFormatException If the text cannot be parsed.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_delay(String delay_time) throws NumberFormatException
	{
		if (do_delay) {
			try {
				delay = Integer.parseInt(delay_time);
			}
			catch (NumberFormatException nfe) {
				exposure_time = 0;
				throw new NumberFormatException("ERROR: Cannot parse delay textfield");
			}
		}
	}

	/**
	*****************************************************************************
	*	This method sets the number of exposure to take. The input string is
	*	parsed as an integer.
	*
	*	@param num_of_exp The number of exposures to take (as a String).
	*
	*	@Throws NumberFormatException If the text cannot be parsed.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_number_of_exposures(String num_of_exp) throws NumberFormatException
	{
		if (do_multiple_exposures) {
			try {
				number_of_exposures = Integer.parseInt(num_of_exp);
			}
			catch (NumberFormatException nfe) {
				exposure_time = 0;
				throw new NumberFormatException("ERROR: Cannot parse number of exp textfield");
			}
		}
	}					

	/**
	*****************************************************************************
	*	This method sets the image filename.
	*
	*	@param image_file The image filename.
	*
	*	@Throws Exception If the file path does not exist.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_image_filename(String image_file) throws Exception
	{
		// If saving file, check the directory existence and set the image filename.
		if (do_image_save) {
			int dir_index = image_file.lastIndexOf("/");
			
			if (dir_index != -1) {
				File file = new File(image_file.substring(0, dir_index));

				if (file.exists())			
					image_filename = image_file;
				else
					throw new Exception("ERROR: Specified image file path does not exist");
			}
		}
	}

	/**
	*****************************************************************************
	*	This method sets the image dimensions.
	*
	*	@param row_num The number of image rows.
	*	@param col_num The number of image columns.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_image_dimensions(int row_num, int col_num)
	{
		rows = row_num;
		cols = col_num;
	}

	/**
	*****************************************************************************
	*	This method sets the image resolution.
	*
	*	@param resolution The image resolution.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_image_resolution(int resolution)
	{
		image_resolution = resolution;
	}

	/**
	*****************************************************************************
	*	This method starts the SwingWorker thread.
	*
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public Object construct()
	{
		return expose();
	}

	/**
	*****************************************************************************
	*	This method closes the SwingWorker thread. This method overides the
	*	one in Swingworker.java.
	*
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void finished()
	{
		// Set the expose button text back to "Expose".
		parent.expose_button.setText("Expose");
		dsp_cmd.abort = false;
	}

	/**
	*****************************************************************************
	*	This method performs the exposure sequence.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public Object expose()
	{
		abort = false;
		pause = false;
		elapsed_time = 0;
			
		// Set the shutter position. The command reads the current controller
		// status from the timing board address X:0. If "open shutter", bit 11
		// of the status is set to a 1, otherwise, bit 11 is cleared (set to a 0).
		// The entire status word is then written back to the timing board address X:0.
		if (current_shutter_position != do_open_shutter) {
			if (set_shutter_postion() == VoodooConstants.FAILURE)
				return "FAILED";
		}

		// Check for abort.
		if (abort) return "DONE";
		
		// Set the exposure time.
		try {
			dsp_cmd.set_exposure_time(exposure_time);
		}
		catch (ReplyValueException rve) {
			PrintMessage.message("ExposureCommand", 1, "ERROR: Exposure time failed to be set.\n", 0, true);
			return "FAILED";
		}
		catch (Exception e) {
			PrintMessage.message("ExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
			return "FAILED";
		}

		// Check for abort.
		if (abort) return "DONE";
	
		// Get the image size in bytes.
		byte_count = (image_resolution/8)*rows*cols;
		
		// Check for multiple image exposures.
		if (!do_multiple_exposures)
			number_of_exposures = 1;

		// Check for abort.
		if (abort) return "DONE";

		for (int x=0; x<number_of_exposures; x++) {
			// Set the displayed elapsed time to zero.
			parent.set_elapsed_time(0);

			// Check for a delay before starting the exposure.
			if (do_delay && !abort) {
				try {
					Thread.sleep(delay*1000);
				}
				catch(InterruptedException ie) {}
			}

			// CALL A RUNNABLE METHOD THAT WILL CALL CIOCTL TO MONITOR THE
			// PROGRESS OF THE READOUT AND WILL CALL UPDATESTATUS().
//			monitor_readout();

			// Start the exposure.
			try {
				PrintMessage.message("ExposureCommand", 0, "\nStarting exposure " + (x+1) + "...waiting.\n", 0, true);
				dsp_cmd.hcvr(VoodooConstants.START_EXPOSURE, VoodooConstants.DON);
			}
			catch(ReplyValueException rve) {
				PrintMessage.message("ExposureCommand", 1, "failed.", 8, false);
				PrintMessage.message("ExposureCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ExposureCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ExposureCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);

				// Reset the PCI board DSP program counter.
				try {
					dsp_cmd.hcvr(VoodooConstants.ABORT_READOUT, VoodooConstants.DON);
					dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
				}
				catch(ReplyValueException RVE) {}
				catch(Exception E) {}		

				return "FAILED";
			}
			catch (Exception e) {
				PrintMessage.message("ExposureCommand", 1, "failed.", 8, false);
				PrintMessage.message("ExposureCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);

				// Reset the PCI board DSP program counter.
				try {
					dsp_cmd.hcvr(VoodooConstants.ABORT_READOUT, VoodooConstants.DON);
					dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
				}
				catch(ReplyValueException RVE) {}
				catch(Exception E) {}		

				return "FAILED";
			}

			// Wait for the exposure time to elapse.
			if (exposure_time > 5000 && !abort) {
				// Make sure the pause and change exposure time buttons visible.
				parent.pause_button.setEnabled(true);
				parent.change_exp_time_button.setEnabled(true);

				// Check for abort.
				if (abort) return "DONE";

				// Wait for the utility board timer to hit zero before monitoring elapsed time.
				if (use_controller_exptime) {
					try {
						//elapsed_time = dsp_cmd.read_expose_time();
						elapsed_time = dsp_cmd.hcvr(VoodooConstants.READ_EXPOSURE_TIME);
						while (elapsed_time < 0) {
							//elapsed_time = dsp_cmd.read_expose_time();
							elapsed_time = dsp_cmd.hcvr(VoodooConstants.READ_EXPOSURE_TIME);
						
							// Check for abort.
							if (abort) return "DONE";
						}
					}
					catch (ReplyValueException rve) {}
					catch (Exception e) {
						PrintMessage.message("ExposureCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
					}
				}
				
				try {
					elapsed_time = 0;
					parent.set_elapsed_time(exposure_time/1000);
					while ((elapsed_time) < ((exposure_time) - 5000)) {
						Thread.sleep(500);	// Sleep for 0.5 second.
						
						// Check for abort.
						if (abort) return "DONE";
		
						while (pause)
							Thread.sleep(250);
						
						if (change_exposure_time) {
							// Check for zero exposure time (or times < 5 sec) and set exposure time to 5100 ms
							// if true. This is to allow the PCI board to take control for the remaining 5 seconds.
							// The exposure time is set to 5100 ms to allow the READ_IMAGE command to be issued to
							// the PCI board. The PCI board does not accept commands once the elapsed exposure time
							// is less than 5 seconds.
							if (exposure_time <= 5000)
								exposure_time = 5100;

							dsp_cmd.set_exposure_time(exposure_time);
							
							parent.set_elapsed_time(exposure_time/1000);
							change_exposure_time = false;

							// Check for exposure time set to zero and exit if it is.
							if (exposure_time <= 5000)
								break;
						}
						if (use_controller_exptime)
							//elapsed_time = dsp_cmd.read_expose_time();
							elapsed_time = dsp_cmd.hcvr(VoodooConstants.READ_EXPOSURE_TIME);
						else
							elapsed_time += 500;
						parent.set_elapsed_time((exposure_time/1000) - (elapsed_time/1000));
					}
					// Make sure the display shows a 5.
					parent.set_elapsed_time(5);
				}
				catch (InterruptedException ie) {}
				catch (ReplyValueException rve) {}
				catch (Exception e) {
					PrintMessage.message("ExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					return "FAILED";
				}

				// Make sure the pause and change exposure time buttons are invisible.
				parent.pause_button.setEnabled(false);
				parent.change_exp_time_button.setEnabled(false);
			}

			// Inform user that exposure has complete.
			PrintMessage.message("ExposureCommand", 0, "done.", 8, false);

			if (do_beep && !abort) {
				Toolkit tk = Toolkit.getDefaultToolkit();
				tk.beep();
			}
			
			// Blank out all components during readout.
//			components_enabled(false);

			// Check for abort.
			if (abort) return "DONE";

//new DisplayZombie();

			parent.expose_button.setText("Abort Readout");
			
			// Readout the image. NOTE: Both exposure time > 5 seconds and exposure
			// time < 5 seconds, are handled within the DspCommand2.read_image() method.
			try {
				PrintMessage.message("ExposureCommand", 0, "Reading out image...waiting.\n", 0, true);
				dsp_cmd.read_image(byte_count, exposure_time, MemorySetup.mem_fd);
				PrintMessage.message("ExposureCommand", 0, "done.", 8, false);
			}
			catch(ReplyValueException rve) {
				PrintMessage.message("ExposureCommand", 1, "failed.", 8, false);
				PrintMessage.message("ExposureCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ExposureCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ExposureCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);

				// Reset the PCI board DSP program counter.
				try {
					dsp_cmd.hcvr(VoodooConstants.ABORT_READOUT, VoodooConstants.DON);
					dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
				}
				catch(ReplyValueException RVE) {}
				catch(Exception E) {}		
				
				return "Failed";
			}
			catch(Exception e) {
				PrintMessage.message("ExposureCommand", 1, "failed.", 8, false);
				PrintMessage.message("ExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);

				// Reset the PCI board DSP program counter.
				try {
					dsp_cmd.hcvr(VoodooConstants.ABORT_READOUT, VoodooConstants.DON);
					dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
				}
				catch(ReplyValueException RVE) {}
				catch(Exception E) {}		

				return "Failed";
			}
//			finally {
				// Re-display all components after readout.
//				components_enabled(true);
//			}

			if (do_beep && !abort) {
				Toolkit tk = Toolkit.getDefaultToolkit();
				tk.beep();
			}

			// Swap the data.
			MemorySetup.Cswap_memory(MemorySetup.mem_fd, byte_count);
				
			// Check for abort.
			if (abort) return "DONE";
		
			// Simulate the reading of the image buffer.
			/*
			try {
				for(int i = 0; i < parent_window.progress_bar.getMaximum(); i++) {
					updateStatus(i);
					Thread.sleep(100);
				}
			}
			catch(InterruptedException ie) { 
				updateStatus(0);
				return "Done";
			}
			*/
		
			// Check if the image needs to be deinterlaced.
			if (do_deinterlacing && !abort) {
				try {
					PrintMessage.message("ExposureCommand", 0, "Deinterlacing image data...waiting.\n", 0, true);
					SetupCommand.Cdeinterlace(cols, rows, MemorySetup.mem_fd, deinterlace_value);
					PrintMessage.message("ExposureCommand", 0, "done.", 8, false);
				}
				catch (Exception e) {
					PrintMessage.message("ExposureCommand", 1, "failed.", 8, false);
					JOptionPane.showMessageDialog(null, e.getMessage(), "Deinterlace Error", JOptionPane.ERROR_MESSAGE);
       				}
			}

			if (do_display && !abort) {
				try {
					PrintMessage.message("ExposureCommand", 0, "Displaying image...waiting.\n", 0, true);
					ImageDisplay.Cdisplay(MemorySetup.mem_fd, rows, cols);
					PrintMessage.message("ExposureCommand", 0, "done.", 8, false);
    				}
    				catch (Exception e) {
					PrintMessage.message("ExposureCommand", 1, "failed.", 8, false);
					JOptionPane.showMessageDialog(null, e.getMessage(), "Display Error", JOptionPane.ERROR_MESSAGE);
       				}

			}

			if (do_image_save && !abort) {
				// Update the FITS header.
				parent.fits_dialog.update_fits_info();

				// NOTE: the filename prefix is set to the correct string by removing any
				// previously set increment numbers (last_increment).
				if (do_auto_increment && !abort) {
					int prefix_index = image_filename.indexOf(Integer.toString(last_increment-1));
					int suffix_index = image_filename.indexOf('.');
			
					if (prefix_index != -1)
						image_filename_prefix = image_filename.substring(0, prefix_index);
					else {
						if (suffix_index != -1)
							image_filename_prefix = image_filename.substring(0, suffix_index);
						else
							image_filename_prefix = image_filename;
					}

					// Set the image filename.
					if (suffix_index != -1) {
						image_filename_suffix = image_filename.substring(suffix_index);
						image_filename = image_filename_prefix.concat(Integer.toString(increment) + image_filename_suffix);
					}
					
					else
						image_filename = image_filename_prefix.concat(Integer.toString(increment));
					
					parent.image_filename_txtfld.setText(image_filename);
					increment++;
					parent.image_incr_start_number_txtfld.setText(Integer.toString(increment));
					last_increment = increment;
				}

				// Try to write the FITS file from the image data.
				try {
					PrintMessage.message("ExposureCommand", 0, "Writing FITS file...waiting.\n", 0, true);
					image.write_fits_file(image_filename, MemorySetup.mem_fd, rows, cols, byte_count);
					PrintMessage.message("ExposureCommand", 0, "done.", 8, false);
				}
				catch (FileNotFoundException fnfe) {
					PrintMessage.message("ExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("ExposureCommand", 1, "ERROR: No FITS file specified.\n", 0, true);
				}
				catch (IOException ie) {
					PrintMessage.message("ExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("ExposureCommand", 1, "ERROR: IO Exception (FITS File).\n", 0, true);
				}
		
			}

			// Check the returned WSI data for errors.
			if (OptionsCommand.get_image_check_mode() && !abort) {
				int word_count = 0;

				try {
					PrintMessage.message("ExposureCommand", 0, "Checking image data...waiting.\n", 0, true);
					word_count = MemorySetup.Cdata_check(MemorySetup.mem_fd, byte_count/4);
					PrintMessage.message("ExposureCommand", 0, "done.", 8, false);
				}
   				catch (Exception e) {
					PrintMessage.message("ExposureCommand", 1, "failed. (Data Error).\n", 8, false);
					PrintMessage.message("ExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				}
			}

		} // end for

		return "Done";
	}

	/**
	*****************************************************************************
	*	This method sets the current shutter position that is to used during
	*	the exposure. The command reads the current controller status from
	*	the timing board address X:0. If "open shutter", bit 11 of the status
	*	is set to a 1, otherwise, bit 11 is cleared (set to a 0). The entire
	*	status word is then written back to the timing board address X:0.
	*
	*	@return Returns SUCCESS (0) if there are no problems. Otherwise,
	*		FAILURE (1) is sent back.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	protected int set_shutter_postion()
	{
		int current_status = 0;

		// Set the shutter position. The command reads the current controller
		// status from the timing board address X:0. If "open shutter", bit 11
		// of the status is set to a 1, otherwise, bit 11 is cleared (set to a 0).
		// The entire status word is then written back to the timing board address X:0.
		current_shutter_position = do_open_shutter;

		if (abort) return VoodooConstants.FAILURE;

		try {
			// Ask for the current controller status.
			current_status = dsp_cmd.hcvr(VoodooConstants.READ_CONTROLLER_STATUS);
		}
		catch (ReplyValueException rve) {
			//InfoStatusWindow.message(InfoStatusWindow.ERROR,  "ERROR: Could not read status word X0.\n");
			return VoodooConstants.FAILURE;
		}
		catch (Exception e) {
			//InfoStatusWindow.message(InfoStatusWindow.ERROR,  "ERROR: " + e.getMessage() + ".\n");
			return VoodooConstants.FAILURE;
		}

		if (abort) return VoodooConstants.FAILURE;

		try {
			if (do_open_shutter) {
				// Set the arg1 register to open/close shutter status.
				dsp_cmd.arg(1, (current_status | VoodooConstants._OPEN_SHUTTER_));
			}

			else if (!do_open_shutter) {
				// Set the arg1 register to memory type X.
				dsp_cmd.arg(1, (current_status & VoodooConstants._CLOSE_SHUTTER_));
			}

			// Open/close the shutter.
			dsp_cmd.hcvr(VoodooConstants.WRITE_CONTROLLER_STATUS, VoodooConstants.DON);
		}
		catch (ReplyValueException rve) {
			//InfoStatusWindow.message(InfoStatusWindow.ERROR,  "ERROR: Shutter position failed to be set.\n");
			return VoodooConstants.FAILURE;
		}
		catch (Exception e) {
			//InfoStatusWindow.message(InfoStatusWindow.ERROR,  "ERROR: " + e.getMessage() + ".\n");
			return VoodooConstants.FAILURE;
		}

		return VoodooConstants.SUCCESS;
	}	

	private void updateStatus(final int i)
	{
		//Runnable doSetProgressBarValue = new Runnable() {
	   	//	public void run() {
				parent.set_progress_bar(i);
	    	//	}
		//};
		//SwingUtilities.invokeLater(doSetProgressBarValue);
    	}
  	
    	private void monitor_readout()
    	{
    		Runnable monitor_readout_value = new Runnable() {
    			
    			public void run() {
				int reply_value = 0;
				updateStatus(0);
				System.out.println("\n Progress bar max = " + parent.get_progress_bar_max_value());
				for(int x=0; x<parent.get_progress_bar_max_value(); x++) {
					try {
						VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_GET_PROGRESS, reply_value);
//						System.out.println("\n progress buffer = " + reply_value);
						updateStatus(reply_value);
						Thread.sleep(100);
					}
					catch(InterruptedException ie) {}
				}
    			}
    		};
    		SwingUtilities.invokeLater(monitor_readout_value);
    	}

	/**
	*****************************************************************************
	*	This method enables/disables the components on the main window.
	*
	*	@param mode True means enable the components. False means disable
	*		    the components.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	protected void components_enabled(boolean mode)
	{
		// Set the camera options mode.
		parent.open_shutter_now_button.setEnabled(mode);
		parent.clear_array_button.setEnabled(mode);
		parent.read_temperature_button.setEnabled(mode);

		// Set the image options mode.
		parent.save_disk_chkbox.setEnabled(mode);
		parent.display_image_chkbox.setEnabled(mode);
		parent.delay_chkbox.setEnabled(mode);
		parent.multi_exp_chkbox.setEnabled(mode);
		parent.beep_chkbox.setEnabled(mode);
		parent.deinterlace_combo_box.setEnabled(mode);
		parent.open_shutter_chkbox.setEnabled(mode);
				
		// Only set save to disk options if it's selected.
		if (parent.save_disk_chkbox.isSelected()) {
			parent.autoincr_chkbox.setEnabled(mode);
			parent.image_filename_txtfld.setEnabled(mode);
			parent.image_incr_start_number_txtfld.setEnabled(mode);
			parent.image_incr_start_number_label.setEnabled(mode);
		}
		
		// Only set delay textfield if it's selected.
		if (parent.delay_chkbox.isSelected())
			parent.delay_txtfld.setEnabled(mode);
			
		// Only set multiple exposure textfield if it's selected.
		if (parent.multi_exp_chkbox.isSelected())
			parent.mult_exp_txtfld.setEnabled(mode);
	}
}
