/**
********************************************************************************
*	The class is responsible for responding to events from the exposure
*	control section of the main application window.
*
*	CVM - Controller
*
*	@see VoodooMainWindow
*	@see ActionListener
*	@see ItemListener
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;

public class ExposureControlListener implements ActionListener, ItemListener, CaretListener
{
	protected VoodooMainWindow parent;
	protected float exposure_time;
	protected int number_of_exposures;
	protected int exposure_status;
	protected String image_filename;
	protected ExposureCommand exp_cmd;
	
	/**
	*****************************************************************************
	*	Constructor.
	*
	*	@param parent The window for which this controller listens.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public ExposureControlListener(VoodooMainWindow parent)
	{
		this.parent = parent;
		exp_cmd = new ExposureCommand(parent);
		exposure_time = 0;
		exposure_status = 0;
	}

	/**
	*****************************************************************************
	*	This method handles item selection events from the exposure control
	*	section of the main application window.
	*
	*	@param ie The selection event.
	*
	*	@see ItemListener
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void itemStateChanged(ItemEvent ie)
	{
		AbstractButton active_button = (AbstractButton) ie.getSource();
		
		// Set the number of exposures.
		if (active_button.getText().equals("Multiple Exp: ")) {
			if (ie.getStateChange() == ItemEvent.SELECTED) {
				exp_cmd.set_multiple_exposures(true);
				parent.mult_exp_txtfld.setEnabled(true);
			}
			else {
				exp_cmd.set_multiple_exposures(false);
				parent.mult_exp_txtfld.setEnabled(false);
			}
		}

		// Display the image.
		else if (active_button.getText().equals("Display Image")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				exp_cmd.set_display_image(true);
			else
				exp_cmd.set_display_image(false);
		}
				
		// Save the image to disk.
		else if (active_button.getText().equals("Save To Disk: ")) {
			if (ie.getStateChange() == ItemEvent.SELECTED) {
				exp_cmd.set_save_to_disk(true);
				parent.autoincr_chkbox.setEnabled(true);
				parent.image_filename_txtfld.setEnabled(true);
				parent.image_incr_start_number_label.setEnabled(true);
				parent.image_incr_start_number_txtfld.setEnabled(true);
			}
			else {
				exp_cmd.set_save_to_disk(false);
				parent.autoincr_chkbox.setEnabled(false);
				parent.image_filename_txtfld.setEnabled(false);
				parent.image_incr_start_number_label.setEnabled(false);
				parent.image_incr_start_number_txtfld.setEnabled(false);
			}
		}

		// Auto increment the image filename.
		else if (active_button.getText().equals("Auto Incr")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				exp_cmd.set_auto_increment(true);
			else
				exp_cmd.set_auto_increment(false);
		}

		// Set the delay between exposures.
		else if (active_button.getText().equals("Delay (secs): ")) {
			if (ie.getStateChange() == ItemEvent.SELECTED) {
				exp_cmd.set_delay(true);
				parent.delay_txtfld.setEnabled(true);
			}
			else {
				exp_cmd.set_delay(false);
				parent.delay_txtfld.setEnabled(false);
			}
		}

		// Beep when exposure completes.
		else if (active_button.getText().equals("Beep")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				exp_cmd.set_beep(true);
			else
				exp_cmd.set_beep(false);
		}

		// Open the shutter during an exposure.
		else if (active_button.getText().equals("Open Shutter")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				exp_cmd.set_open_shutter(true);
			else
				exp_cmd.set_open_shutter(false);
		}

		// Set the array deinterlacing mode.
		else if (active_button.getText().equals("Deinterlacing:")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				exp_cmd.set_deinterlacing(true);
			else
				exp_cmd.set_deinterlacing(false);
		}	
	}
	
	/**
	*****************************************************************************
	*	This method handles action events from the exposure control section
	*	of the main application window. A minimum setup of the array dimensions
	*	must be performed before an exposure can be taken.
	*
	*	@param e The event.
	*
	*	@see ActionListener
	*	@version 1.01
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void actionPerformed(ActionEvent e)
	{
		AbstractButton active_button = (AbstractButton) e.getSource();
		
		// Take an image exposure.
		if (active_button.getText().equals("Expose")) {
			// Check that the minimum setup has been applied before exposing.
			if (!parent.setup_dialog.setup_listener.setup_cmd.setup_applied()) {
				JOptionPane.showMessageDialog(null, "Cannot take an exposure until a " +
						"minimum \nsetup of the dimensions has been applied.",
						"WARNING", JOptionPane.WARNING_MESSAGE);
				return;
			}

			// Set the image dimension and resolution (already integers).
			exp_cmd.set_image_dimensions(parent.setup_dialog.setup_listener.setup_cmd.get_ccd_rows(), parent.setup_dialog.setup_listener.setup_cmd.get_ccd_cols());
			exp_cmd.set_image_resolution(parent.setup_dialog.setup_listener.setup_cmd.get_image_resolution());

			// Set and check all textfields.
			try {
				exp_cmd.set_exposure_time(parent.exposure_time_txtfld.getText());
				exp_cmd.set_delay(parent.delay_txtfld.getText());
				exp_cmd.set_number_of_exposures(parent.mult_exp_txtfld.getText());
				exp_cmd.set_image_filename(parent.image_filename_txtfld.getText());
			}
			catch (NumberFormatException nfe) {
				PrintMessage.message("ExposureControlListener", 1, nfe.getMessage() + ".\n", 0, true);
				parent.expose_button.setText("Expose");
				return;
			}
			catch (Exception exp) {
				PrintMessage.message("ExposureControlListener", 1, exp.getMessage() + ".\n", 0, true);
				parent.expose_button.setText("Expose");
				return;
			}

			// Set the expose button text to "Abort".
			parent.expose_button.setText("Abort Exposure");
			
			// Take the exposure(s).
			exp_cmd.thread_it();
		}
		
		// Abort the current image exposure.
		else if (active_button.getText().equals("Abort Exposure")) {
			//exp_cmd.stop();
			exp_cmd.abort_exposure();
			//exp_cmd.interrupt();

			try {
				DspCommand2.flush_reply_buffer();
			}
			catch (Exception ex) {}
			
			parent.expose_button.setText("Expose");
			parent.pause_button.setText("Pause ");

			// Make sure the pause and change exposure time buttons are invisible.
			parent.pause_button.setEnabled(false);
			parent.change_exp_time_button.setEnabled(false);
			
			PrintMessage.message("ExposureControlListener", 0, "\nExposure aborted.\n", 0, true);
		}

		// Change the current exposure time.
		else if (active_button.getText().equals("Change Exp Time")) {
			exp_cmd.change_exposure_time();
		}
	
		// Pause the current image exposure.
		else if (active_button.getText().equals("Pause ")) {
			parent.pause_button.setText("Resume");
			exp_cmd.pause_exposure(true);
		}
		
		// Resume the current image exposure.
		else if (active_button.getText().equals("Resume")) {
			parent.pause_button.setText("Pause ");
			exp_cmd.pause_exposure(false);
		}
		
		// Abort the current image readout.
		else if (active_button.getText().equals("Abort Readout")) {
			exp_cmd.abort_readout();
			//exp_cmd.stop();
			//exp_cmd.interrupt();
			
			try {
				DspCommand2.flush_reply_buffer();
			}
			catch (Exception ex) {}

			parent.expose_button.setText("Expose");
			PrintMessage.message("ExposureControlListener", 0, "Readout aborted.\n", 0, true);
		}
	}

	/**
	*****************************************************************************
	*	This method handles text action events from the "Save To Disk"
	*	textfield in the image options section of the main application
	*	window. Any change in the text will fire this event.
	*
	*	@param ce The text event caused by a change in text.
	*
	*	@see ActionListener
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void caretUpdate(CaretEvent ce)
	{
		int new_increment = 0;

		// Set the new image filename increment. Do nothing if the user clicked
		// in the textfield, but made no changes to the increment value.
		try {
			new_increment = Integer.parseInt(parent.image_incr_start_number_txtfld.getText());
			
			if (new_increment != exp_cmd.start_increment) {
				exp_cmd.increment = new_increment;
				exp_cmd.start_increment = new_increment;
			}
		}
		catch (NumberFormatException nfe) {}
	}
}