/**
********************************************************************************
*	The class is responsible for taking an image exposure, reading out the
*	image, displaying the image, saving the image, and checking the image
*	data if the it is a DSP synthetically generated. This class is executed
*	inside of a SwingWorker thread class.
*
*	CVM - Model
*
*	@see VoodooMainWindow
*	@see SwingWorker
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import javax.swing.*;
import java.io.*;

public class IRExposureCommand extends ExposureCommand
{
	/**
	*****************************************************************************
	*	Constructor.
	*
	*	@see VoodooMainWindow
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public IRExposureCommand(VoodooMainWindow parent)
	{
		// Superclass is automatically called, which starts the SwingWorker thread.
		super(parent);
		dsp_cmd = new DspCommand2();
		image = new Fits(parent.fits_dialog);
	}

	/**
	*****************************************************************************
	*	This method starts the SwingWorker thread.
	*
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public Object construct()
	{
		return irexpose();
	}

	/**
	*****************************************************************************
	*	This method performs the exposure sequence.
	*
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public Object irexpose()
	{
		int current_status = 0;

		abort = false;
		pause = false;
		increment = 1;

		// Set the shutter position. The command reads the current controller
		// status from the timing board address X:0. If "open shutter", bit 11
		// of the status is set to a 1, otherwise, bit 11 is cleared (set to a 0).
		// The entire status word is then written back to the timing board address X:0.
		if (current_shutter_position != do_open_shutter) {
			if (set_shutter_postion() == VoodooConstants.FAILURE)
				return "FAILED";
		}

		// Check for abort.
		if (abort) return "DONE";

		// Set the exposure time.
		try {
			dsp_cmd.set_exposure_time(exposure_time);
		}
		catch (ReplyValueException rve) {
			PrintMessage.message("IRExposureCommand", 1, "ERROR: Exposure time failed to be set.\n", 0, true);
			return "FAILED";
		}
		catch (Exception e) {
			PrintMessage.message("IRExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
			return "FAILED";
		}

		// Check for abort.
		if (abort) return "DONE";

		// Get the image size, number of bytes, and image resolution.
		byte_count = (image_resolution/8)*rows*cols;

		// Reset the image dimensions (half for exposure time > 5 sec).
		try {
			if (exposure_time > 5000) {
				dsp_cmd.dimensions(rows/2, cols*(image_resolution/16));	// This is how the DSP handles different bits per pixel.
			}
			else {
				dsp_cmd.dimensions(rows, cols*(image_resolution/16));	// This is how the DSP handles different bits per pixel.
			}
		}
		catch (ReplyValueException rve) {}
		catch (Exception e) {}
			
		// Check for multiple image exposures.
		if (!do_multiple_exposures)
			number_of_exposures = 1;

		// Check for abort.
		if (abort) return "DONE";

		for (int x=0; x<number_of_exposures; x++) {
			// Set the displayed elapsed time to zero.
			parent.set_elapsed_time(0);

			// Check for a delay before starting the exposure.
			if (do_delay && !abort) {
				try {
					Thread.sleep(delay*1000);
				}
				catch(InterruptedException ie) {}
			}

			// CALL A RUNNABLE METHOD THAT WILL CALL CIOCTL TO MONITOR THE
			// PROGRESS OF THE READOUT AND WILL CALL UPDATESTATUS().
//			monitor_readout();

			// Start the exposure.
			try {
				PrintMessage.message("IRExposureCommand", 0, "\nStarting exposure " + (x+1) + "...waiting.\n", 0, true);
				dsp_cmd.hcvr(VoodooConstants.START_EXPOSURE, VoodooConstants.DON);
			}
			catch(ReplyValueException rve) {
				PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
				PrintMessage.message("IRExposureCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("IRExposureCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("IRExposureCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				return "FAILED";
			}
			catch (Exception e) {
				PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
				PrintMessage.message("IRExposureCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			}
				
			// Inform user that exposure has complete.
			PrintMessage.message("IRExposureCommand", 0, "done.", 8, false);

			// Check if the exposure time < 5 seconds, if it is, call the DspCommand2.read_image()
			// command, which will handle that case.
			if (exposure_time < 5000) {
				try {
					PrintMessage.message("IRExposureCommand", 0, "Reading out image...waiting.\n", 0, true);
					parent.expose_button.setText("Abort Readout");

					// Disable the main window components.
//					components_enabled(false);

					dsp_cmd.read_image(byte_count, exposure_time, MemorySetup.mem_fd);
					PrintMessage.message("IRExposureCommand", 0, "done.", 8, false);
				}
				catch(ReplyValueException rve) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("IRExposureCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
					if (rve.get_value() == VoodooConstants.TOUT)
						PrintMessage.message("IRExposureCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
					else
						PrintMessage.message("IRExposureCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
					return "Failed";
				}
				catch(Exception e) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("IRExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					return "Failed";
				}
//				finally {
					// Re-enable the main window components.
//					components_enabled(true);
//				}
			}

			// Else, exposure_time > 5000.
			else {
				// Check for abort.
				if (abort) return "DONE";

				try {
					// Make sure the pause and change exposure time buttons are invisible.
					parent.pause_button.setEnabled(false);
					parent.change_exp_time_button.setEnabled(false);
					
					// Disable the main window components.
//					components_enabled(false);

					PrintMessage.message("IRExposureCommand", 0, "Reading out image...waiting.\n", 0, true);
					parent.expose_button.setText("Abort Readout");
					dsp_cmd.read_image(byte_count/2, exposure_time, MemorySetup.mem_fd);
					PrintMessage.message("IRExposureCommand", 0, "done.", 8, false);
				}
				catch(ReplyValueException rve) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("IRExposureCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
					if (rve.get_value() == VoodooConstants.TOUT)
						PrintMessage.message("IRExposureCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
					else
						PrintMessage.message("IRExposureCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
					return "Failed";
				}
				catch(Exception e) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("IRExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					return "Failed";
				}
//				finally {
					// Re-enable the main window components.
//					components_enabled(true);
//				}

				// Make sure the pause and change exposure time buttons visible.
				parent.pause_button.setEnabled(true);
				parent.change_exp_time_button.setEnabled(true);

				parent.expose_button.setText("Abort Exposure");
 
				// Wait for the utility board timer to hit zero before monitoring elapsed time.
				if (use_controller_exptime) {
					try {
						elapsed_time = dsp_cmd.read_expose_time();
						while (elapsed_time < 0) {
							elapsed_time = dsp_cmd.read_expose_time();
						
							// Check for abort.
							if (abort) return "DONE";
						}
					}
					catch (ReplyValueException rve) {}
					catch (Exception e) {
						PrintMessage.message("IRExposureCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
					}
				}
				
				try {
					elapsed_time = 0;
					parent.set_elapsed_time(exposure_time/1000);
					while ((elapsed_time) < ((exposure_time) - 5000)) {
						Thread.sleep(500);	// Sleep for 0.5 second.
						
						// Check for abort.
						if (abort) return "DONE";
		
						while (pause)
							Thread.sleep(250);
						
						if (change_exposure_time) {
							// Check for zero exposure time (or times < 5 sec) and set exposure time to 5100 ms
							// if true. This is to allow the PCI board to take control for the remaining 5 seconds.
							// The exposure time is set to 5100 ms to allow the READ_IMAGE command to be issued to
							// the PCI board. The PCI board does not accept commands once the elapsed exposure time
							// is less than 5 seconds.
							if (exposure_time <= 5000)
								exposure_time = 5100;

							dsp_cmd.set_exposure_time(exposure_time);
							
							parent.set_elapsed_time(exposure_time/1000);
							change_exposure_time = false;

							// Check for exposure time set to zero and exit if it is.
							if (exposure_time <= 5000)
								break;
						}
						if (use_controller_exptime)
							elapsed_time = dsp_cmd.read_expose_time();
						else
							elapsed_time += 500;
						parent.set_elapsed_time((exposure_time/1000) - (elapsed_time/1000));
					}
					// Make sure the display shows a 5.
					parent.set_elapsed_time(5);
				}
				catch (InterruptedException ie) {}
				catch (ReplyValueException rve) {}
				catch (Exception e) {
					PrintMessage.message("IRExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					return "FAILED";
				}

				if (do_beep && !abort) {
					Toolkit tk = Toolkit.getDefaultToolkit();
					tk.beep();
				}

				// Make sure the pause and change exposure time buttons are invisible.
				parent.pause_button.setEnabled(false);
				parent.change_exp_time_button.setEnabled(false);

				// Disable the main window components.
//				components_enabled(false);

				// Check for abort.
				if (abort) return "DONE";

				// Readout the image. NOTE: The second parameter to the read_image() command is
				// forced to a value greater than 5 seconds so the read image vector
				// command will be issued. The value of 7 seconds is meaningless, the
				// value just has to be greater than 5 seconds.
				try {
					PrintMessage.message("IRExposureCommand", 0, "Reading out image...waiting.\n", 0, true);
					parent.expose_button.setText("Abort Readout");
					dsp_cmd.read_image(byte_count/2, exposure_time, MemorySetup.mem_fd+(byte_count/2));
					PrintMessage.message("IRExposureCommand", 0, "done.", 8, false);
				}
				catch(ReplyValueException rve) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("IRExposureCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
					if (rve.get_value() == VoodooConstants.TOUT)
						PrintMessage.message("IRExposureCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
					else
						PrintMessage.message("IRExposureCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
					return "Failed";
				}
				catch(Exception e) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("IRExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					return "Failed";
				}
//				finally {
					// Re-enable the main window components.
//					components_enabled(true);
//				}
			}

			if (do_beep && !abort) {
				Toolkit tk = Toolkit.getDefaultToolkit();
				tk.beep();
			}
				
			// Swap the data.
			MemorySetup.Cswap_memory(MemorySetup.mem_fd, byte_count);

			// Check for abort.
			if (abort) return "DONE";
		
			// Check if the image needs to be deinterlaced.
			if (do_deinterlacing && !abort) {
				try {
					PrintMessage.message("IRExposureCommand", 0, "Deinterlacing image data...waiting.\n", 0, true);
					SetupCommand.Cdeinterlace(cols, rows/2, MemorySetup.mem_fd, deinterlace_value);
					SetupCommand.Cdeinterlace(cols, rows/2, MemorySetup.mem_fd+(byte_count/2), deinterlace_value);
					PrintMessage.message("IRExposureCommand", 0, "done.", 8, false);
				}
				catch (Exception e) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					JOptionPane.showMessageDialog(null, e.getMessage(), "Deinterlace Error", JOptionPane.ERROR_MESSAGE);
       				}
			}

			if (do_display && !abort) {
				try {
					PrintMessage.message("IRExposureCommand", 0, "Displaying image...waiting.\n", 0, true);
					ImageDisplay.Cdisplay(MemorySetup.mem_fd, rows, cols);
					PrintMessage.message("IRExposureCommand", 0, "done.", 8, false);
    				}
    				catch (Exception e) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					JOptionPane.showMessageDialog(null, e.getMessage(), "Display Error", JOptionPane.ERROR_MESSAGE);
       				}

			}

			if (do_image_save && !abort) {
				// Update the FITS header.
				parent.fits_dialog.update_fits_info();
		
				// NOTE: the filename prefix is set to the correct string by removing any
				// previously set increment numbers (last_increment).
				if (do_auto_increment && !abort) {
					int index = image_filename.indexOf(Integer.toString(last_increment-1));
			
					if (index != -1)
						image_filename_prefix = image_filename.substring(0, index);
					else
						image_filename_prefix = image_filename.substring(0, image_filename.indexOf('.'));

					image_filename = image_filename_prefix.concat(Integer.toString(increment) + ".fts");
					parent.image_filename_txtfld.setText(image_filename);
					increment++;
					last_increment = increment;
				}

				// Try to write the FITS file from the image data.
				try {
					PrintMessage.message("IRExposureCommand", 0, "Writing FITS file...waiting.\n", 0, true);
					image.write_fits_file(image_filename, MemorySetup.mem_fd, rows, cols, byte_count);
					PrintMessage.message("IRExposureCommand", 0, "done.", 8, false);
				}
				catch (FileNotFoundException fnfe) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("IRExposureCommand", 1, "ERROR: No FITS file specified.\n", 0, true);
				}
				catch (IOException ie) {
					PrintMessage.message("IRExposureCommand", 1, "failed.", 8, false);
					PrintMessage.message("IRExposureCommand", 1, "ERROR: IO Exception (FITS File).\n", 0, true);
				}
		
			}

			// Check the returned WSI data for errors.
			if (OptionsCommand.get_image_check_mode() && !abort) {
				int word_count = 0;

				try {
					PrintMessage.message("IRExposureCommand", 0, "Checking image data...waiting.\n", 0, true);
					word_count = MemorySetup.Cdata_check(MemorySetup.mem_fd, byte_count/4);
					PrintMessage.message("IRExposureCommand", 0, "done.", 8, false);
				}
   				catch (Exception e) {
					PrintMessage.message("IRExposureCommand", 1, "failed. (Data Error).\n", 8, false);
					PrintMessage.message("IRExposureCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				}
			}

		} // end for

		return "Done";
	}

	private void updateStatus(final int i)
	{
		//Runnable doSetProgressBarValue = new Runnable() {
	   	//	public void run() {
				parent.set_progress_bar(i);
	    	//	}
		//};
		//SwingUtilities.invokeLater(doSetProgressBarValue);
    	}
  	
    	private void monitor_readout()
    	{
    		System.out.println("\n In Monitor Readout Function.");
    		
    		Runnable monitor_readout_value = new Runnable() {
    			
    			public void run() {
				int reply_value = 0;
				updateStatus(0);
				System.out.println("\n Progress bar max = " + parent.get_progress_bar_max_value());
				for(int x=0; x<parent.get_progress_bar_max_value(); x++) {
					try {
						VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_GET_PROGRESS, reply_value);
//						System.out.println("\n progress buffer = " + reply_value);
						updateStatus(reply_value);
						Thread.sleep(100);
					}
					catch(InterruptedException ie) {}
				}
    			}
    		};
    		SwingUtilities.invokeLater(monitor_readout_value);
    	}
}
