/**
********************************************************************************
*	A class to display a dialog window that will allow the user to dynamically
*	change the image buffer size.
*
*	CVM - View
*
*	@see JDialog
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

class ImageBuffersDialog extends JDialog
{
	private Container content_pane;
	private ImageBuffersDialogListener image_buffers_dialog_listener;
	private JPanel new_image_buffer_size_panel;
	private JMenuBar new_menu_bar;
	private Font font;
	public VTextField image_buffer_size_txtfld;
	public VTextField image_buffer_rowSize_txtfld;
	public VTextField image_buffer_colSize_txtfld;
	public VTextField image_buffer_totalSize_txtfld;
	public int number_of_options_windows;
	
	/**
	********************************************************************************
	*	Constructor. Creates and sets all dialog window properties.
	*
	*	@param owner The parent window for this dialog box.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public ImageBuffersDialog(Frame owner)
	{
		// Call the superclass (JDialog) constructor.
		super(owner, "Image Buffers", false);
		
		// Set the dialog box size.
		setSize(375, 260);
		
		// Make the window non-resizable.
      		setResizable(false);
		
		// Get a handle on the dialog box's content pane.
		content_pane = this.getContentPane();

		// Set the components font.
		font = new Font("LucidaSansRegular", Font.PLAIN, 11);
		
		// Create the event listener for this dialog box.
		image_buffers_dialog_listener = new ImageBuffersDialogListener(this);

		// Create the menu bar and add it to the dialog box.
		new_menu_bar = create_menu_bar();
		setJMenuBar(new_menu_bar);
		
		// Create the image buffers panel.
		new_image_buffer_size_panel = create_image_buffer_size_panel();
		
		// Add the fits panel to the dialog box.
		content_pane.add(new_image_buffer_size_panel);
		
      		// Add a window listener.  This has to be here to allow the
      		// dialog box to be re-opened later when the "close" window
      		// menu item (from system, not java) is chosen.
      		addWindowListener(new WindowAdapter()
      		{
      			public void windowClosing(WindowEvent e)
      			{
      				number_of_options_windows = 0;
			}
		});
		
		// Initialize any variables.
		number_of_options_windows = 0;
	}

	/**
	********************************************************************************
	*	This function creates the menu bar for this dialog window.
	*
	*	@return Returns the constructed menubar as a JMenuBar class.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JMenuBar create_menu_bar()
	{
		JMenuBar menu_bar = new JMenuBar();

		// Set the menubar's border.
		menu_bar.setBorder(new BevelBorder(BevelBorder.RAISED));
		menu_bar.setBorderPainted(true);
		
		// Add the Window menu to the menu bar.
		JMenu window_menu = new JMenu("Window");
		menu_bar.add(window_menu);

		// Add the submenus to the Window menu.
		JMenuItem close_item = new JMenuItem("Close", 'C');
		window_menu.add(close_item);
		
		// Set the menubar font.
		window_menu.setFont(font);
		close_item.setFont(font);

		// Add the event listeners to the menu items.
		close_item.addActionListener(image_buffers_dialog_listener);
		
		return menu_bar;
	}

	/**
	********************************************************************************
	*	This function creates the image buffer options panel for this dialog
	*	window.
	*
	*	@return Returns the constructed image buffer options panel as a JPanel
	*		class.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public JPanel create_image_buffer_size_panel()
	{
     		JPanel image_buffer_size_panel = new JPanel();
		
		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		image_buffer_size_panel.setLayout(gbl);

      		// Create the image buffer size radio buttons.
      		JRadioButton x512_rbutton = new JRadioButton("512x512", false);
      		JRadioButton x1024_rbutton = new JRadioButton("1024x1024", false);
      		JRadioButton x2200_rbutton = new JRadioButton("2200x2200", true);
      		JRadioButton xCustom_rbutton = new JRadioButton("Custom", false);

      		// Group the radio buttons together so only one at a time may be selected.
      		ButtonGroup buffer_size_rbutton_group = new ButtonGroup();
      		buffer_size_rbutton_group.add(x512_rbutton);
      		buffer_size_rbutton_group.add(x1024_rbutton);
      		buffer_size_rbutton_group.add(x2200_rbutton);
      		buffer_size_rbutton_group.add(xCustom_rbutton);

     		// Create the button components.
    		JButton set_size_button = new JButton("Apply");
     		
		// Set the button size.
		set_size_button.setPreferredSize(new Dimension(90, 26));

     		// Create the label components.
      		JLabel image_buffer_rowSize_label = new JLabel("Rows:");
      		JLabel image_buffer_colSize_label = new JLabel("Cols:");
      		JLabel image_buffer_totalSize_label = new JLabel("Total Size (bytes)");
    		JLabel warning1_label = new JLabel("   WARNING: The buffer size should only be");
    		JLabel warning2_label = new JLabel("   changed before the first image is taken,");
    		JLabel warning3_label = new JLabel("   otherwise, result is undefined");
    		 		     		
     		// Create the textfield components.
		image_buffer_rowSize_txtfld = new VTextField("", 6);
		image_buffer_colSize_txtfld = new VTextField("", 6);
		image_buffer_totalSize_txtfld = new VTextField("9680000", 12);

		// Set the total buffer size textfield to uneditable.
		image_buffer_totalSize_txtfld.setEditable(false);
		
		// Set the label component color.
		warning1_label.setForeground(Color.red);
		warning2_label.setForeground(Color.red);
		warning3_label.setForeground(Color.red);
		image_buffer_rowSize_label.setForeground(Color.black);
		image_buffer_colSize_label.setForeground(Color.black);
		image_buffer_totalSize_label.setForeground(Color.black);
		
		// Add the components to the panel.
      		add_components(image_buffer_size_panel, warning1_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 3, 1);
      		add_components(image_buffer_size_panel, warning2_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 1, 3, 1);
      		add_components(image_buffer_size_panel, warning3_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 2, 3, 1);
      		add_components(image_buffer_size_panel, x512_rbutton, gbl, gbc, 0, 10, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 5, 1, 1);
      		add_components(image_buffer_size_panel, x1024_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 5, 1, 1);
      		add_components(image_buffer_size_panel, x2200_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 5, 1, 1);
      		add_components(image_buffer_size_panel, xCustom_rbutton, gbl, gbc, 0, 10, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 6, 1, 1);
      		add_components(image_buffer_size_panel, image_buffer_rowSize_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 8, 1, 1);
      		add_components(image_buffer_size_panel, image_buffer_rowSize_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 8, 1, 1);
      		add_components(image_buffer_size_panel, image_buffer_colSize_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 9, 1, 1);
      		add_components(image_buffer_size_panel, image_buffer_colSize_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 9, 1, 1);
      		add_components(image_buffer_size_panel, image_buffer_totalSize_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 8, 1, 1);
      		add_components(image_buffer_size_panel, image_buffer_totalSize_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 9, 1, 1);
      		add_components(image_buffer_size_panel, set_size_button, gbl, gbc, 10, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 11, 3, 1);

		// Add the event listener to the components.
		x512_rbutton.addActionListener(image_buffers_dialog_listener);
		x1024_rbutton.addActionListener(image_buffers_dialog_listener);
		x2200_rbutton.addActionListener(image_buffers_dialog_listener);
		xCustom_rbutton.addActionListener(image_buffers_dialog_listener);
		set_size_button.addActionListener(image_buffers_dialog_listener);
		
    		return image_buffer_size_panel;
     	}

	/**
	*****************************************************************************
	*	Adds the specified component to the specified panel.
	*
	*	@param panel The panel to which the component will be added.
	*	@param compnt The component to add.
	*	@param gbl The GridBagLayout of the panel.
	*	@param gbl The GridBagConstraints of the panel.
	*	@param insetTOP The distance in pixels between the top of this
	*			component and the one above it.
	*	@param insetBOTTOM The distance in pixels between the bottom of
	*			this component and the one below it.
	*	@param insetLEFT The distance in pixels between the left side of
	*			this component and the one left of it.
	*	@param insetRIGHT The distance in pixels between the right side
	*			of this component and the one right of it.
	*	@param cfill Determines how the component "fills" the space of
	*		  	of its "cell".
	*	@param anchr The justification of the component within its "cell".
	*	@param col The x position of the component.
	*	@param row The y position of the component.
	*	@param gw The number of "cells" that the component spans in the
	*			width direction.
	*	@param gh The number of "cells" that the component spans in the
	*			height direction.
	*
	*	@see JPanel
	*	@see JComponent
	*	@see GridBagLayout
	*	@see GridBagConstraints
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
      	private void add_components(JPanel panel, JComponent compnt, GridBagLayout gbl, GridBagConstraints gbc, int insetTOP, int insetLEFT, int insetBOTTOM, int insetRIGHT, int cfill, int anchr, int col, int row, int gw, int gh)
   	{
      		// Set the grid layout constraints.
      		gbc.fill = cfill;
      		gbc.anchor = anchr;
      		gbc.insets = new Insets(insetTOP, insetLEFT, insetBOTTOM, insetRIGHT);
      		gbc.gridx = col;
      		gbc.gridy = row;
      		gbc.gridwidth = gw;
      		gbc.gridheight = gh;
      		
      		// Set the component font.
      		compnt.setFont(font);
      
      		// Add the component to the panel.
      		gbl.setConstraints(compnt, gbc);
      		panel.add(compnt);
      	}
}
