/**
********************************************************************************
*	This class provides initialization for all non-window objects that need 
*	initialization. In particular, the device driver C library functions are
*	enabled here along with any global constants and variables that are
*	needed.
*
*	Not a thread because the three threads (tim, util, and SetupCommand) all
*	try to execute simoultaneously. The tim and util threads fight over the
*	static variables (filename and type), while the SetupCommand continues on
*	with setting power on, dimensions, etc. The SetupCommand could be maid to
*	"wait", however.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
  
import java.io.*;
import java.util.*;

public class LoadDspFile
{
	private DspCommand2 dsp_command;
	private String full_filename;
	private String expected_file_type;
	private Thread thread;
	private static boolean busy;

	/**
	********************************************************************************
	*	Class Constructor.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public LoadDspFile()
	{
		dsp_command = new DspCommand2();
	}

	/**
	********************************************************************************
	*	This method loads a file into the DSP of the specified controller board.
	*
	*	@param full_filename The name of the file to load.
	*	@param expected_file_type The controller board to load the file into.
	*				  May be one of the strings "Timing" or "Utility".
	*				  The input strings are not case sensitive.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void load_file(String full_filename, String expected_file_type) throws IOException, NoSuchElementException, ReplyValueException, Exception
	{
		String input_value = " ";
		String input_line = " ";
		String type_string = "__";
		String addr_string = " ";
		String data_string = " ";
		String file_type = " ";
		String path = "./";
		String name = " ";
		boolean end_of_file = false;
		boolean done_reading_data = false;
		int type = 0;
		int addr = 0;
		int data = 0;
		int n = -1;
		long file_position = 0;
		RandomAccessFile inFile;
		int reply_value = 0;
		
		// Separate the file path and name (This is currently operating system dependent,
		// but this can and should be fixed).
		n = full_filename.lastIndexOf((int)'/');
		if (n != -1)
			path = full_filename.substring(0, n);
		name = full_filename.substring(n+1);
				
		// Set the file path and name.
		File file_name = new File(path, name);
		
		// Check for the files existence.
		if (!file_name.exists()) {
			PrintMessage.message("LoadDspFile", 1, "ERROR: " + expected_file_type + " file does not exist.\n", 0, true);
			throw new IOException();
		}
		
		// Open the file for reading.
		inFile = new RandomAccessFile(file_name, "r");

		// Read in the file one line at a time.	
		while (!end_of_file) {
			input_line = inFile.readLine();
			
			// Break the input line into pieces.
			StringTokenizer tokenizer = new StringTokenizer(input_line, " ");
			
			// This block will catch an exception on a blank input line.
			// Because of this, no action is taken. The rest of the code
			// assumes there will be no blank lines amoung the data elements.
			try {
				input_value = tokenizer.nextToken();
			} catch (NoSuchElementException nsee) {}
			
			// Check for the start of valid data.			
			if (input_value.equals("_DATA")) {
				
				type_string = tokenizer.nextToken();
				addr_string = tokenizer.nextToken();

				// Convert the "address" string to the correct data type.
				addr = Integer.parseInt(addr_string, 16);
				
				// A valid start address must be less than MAX_DSP_LOAD_ADDRESS.				
				if (addr < 0x4000) {
					// Set the "type" string to the correct ASCII sequence.
					if (type_string.equals("X"))
						dsp_command.set_type(VoodooConstants.X);
					else if (type_string.equals("Y"))
						dsp_command.set_type(VoodooConstants.Y);
					else if (type_string.equals("P"))
						dsp_command.set_type(VoodooConstants.P);
					else if (type_string.equals("R"))
						dsp_command.set_type(VoodooConstants.R);
					else if (type_string.equals("D"))
						dsp_command.set_type(VoodooConstants.D);
					
					// Read in the data block.
					while (!done_reading_data) {
						input_line = inFile.readLine();
						tokenizer = new StringTokenizer(input_line, " ");
								
						while (tokenizer.hasMoreTokens()) {
							data_string = tokenizer.nextToken();
								
							// Check for the end of the data block by searching the current data string
							// for a "_" character. If found, reposition the file pointer to the
							// beginning of the line.
							if (data_string.indexOf("_") != -1) {
								done_reading_data = true;
								file_position = inFile.getFilePointer();
								inFile.seek(file_position - (input_line.length() + 2));
								break;
							}
							
							// Data is valid, so send it to the DSP.
							else {
								try {
									data = Integer.parseInt(data_string, 16);
									reply_value = dsp_command.write_memory(addr, data);
									addr++;
								}
								catch (Exception e) {
									PrintMessage.message("LoadDspFile", 1, "\nERROR: Data error.\n Exception: " + e + "\n", 0, true);
								}
							}
							
							// Exit if DON not received.
							if  (reply_value != VoodooConstants.DON)
								throw new ReplyValueException(reply_value);
						}
					}
				}
			}
			
			// Check the file validity and set the board destination.
			if (input_value.equals("_START")) {
				try {
					file_type = tokenizer.nextToken();
				} catch (NoSuchElementException nsee) {}
				
				if (file_type.equals("TIMBOOT") && expected_file_type.equalsIgnoreCase("timing")) {
					try {
						PrintMessage.message("LoadDspFile", 0, "Timing file .. OK.\n", 0, true);
						dsp_command.set_board_destination(VoodooConstants.WRM_ARG_NUM, VoodooConstants.TIM_ID);
					}
					catch (ReplyValueException rve) {
						PrintMessage.message("LoadDspFile", 1, "ERROR: Board destination not set.\n", 0, true);
						throw new ReplyValueException(rve.get_value());
					}				
				}
				else if (file_type.equals("UTILBOOT") && expected_file_type.equalsIgnoreCase("utility")) {
					try {
						PrintMessage.message("LoadDspFile", 0, "Utility file .. OK.\n", 0, true);
						dsp_command.set_board_destination(VoodooConstants.WRM_ARG_NUM, VoodooConstants.UTIL_ID);
					}
					catch (ReplyValueException rve) {
						PrintMessage.message("LoadDspFile", 1, "ERROR: Board destination not set.\n", 0, true);
						throw new ReplyValueException(rve.get_value());
					}				
				}
				else
					PrintMessage.message("LoadDspFile", 1, "\nERROR: Invalid file type.\n", 0, true);
			}
			
			// Check for the end of file.
			else if (input_value.equals("_END")) {
				end_of_file = true;
			}
						
			// Re-initialize variables.
			done_reading_data = false;
			type_string = "__";
			input_value = "";
		}
	}
}