/**
********************************************************************************
*	This class handles all manual commands sent to the DSP. All the manual
*	commands input are from the DSP manual commands window.
*
*	CVM - Model
*
*	@version 1.1
*	@see SwingWorker
*	@author Scott Streit
*
*	DATE		WHO	DESCRIPTION
*	----------	---	------------------------------------------------
*	02/16/2000	sds	Upgraded to function within a SwingWorker thread.
*	07/13/2000	sds	Converted from static functions to instantiated.
*	08/11/2000	sds	Moved DSP manual command window text-to-int
*				conversions to functions here.
********************************************************************************
*/
import java.sql.Types;
import java.io.*;

public class ManualCommand extends SwingWorker
{
	private DspCommand2 dsp_cmd = null;
	private DspDialog parent = null;
	private boolean abort;
	private int command;
	private int argument1;
	private int argument2;
	private int argument3;
	private int argument4;
	private int argument5;
	private int board_destination;
	private int number_of_args;
	private	int reply_value;
	
	/**
	********************************************************************************
	*	Class constructor. This constructor is only used when a manual command
	*	needs to be issued unthreaded.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public ManualCommand()
	{
		dsp_cmd = new DspCommand2();
		
		command = 0;
		argument1 = VoodooConstants.NULL;
		argument2 = VoodooConstants.NULL;
		argument3 = VoodooConstants.NULL;
		argument4 = VoodooConstants.NULL;
		argument5 = VoodooConstants.NULL;
		board_destination = 0;
	}

	/**
	********************************************************************************
	*	Class constructor. This constructor is only used when a manual command
	*	is issued from the DspDialog window "Manual Command" tab.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public ManualCommand(DspDialog parent)
	{
		this.parent = parent;
		dsp_cmd = new DspCommand2();

		command = 0;
		argument1 = VoodooConstants.NULL;
		argument2 = VoodooConstants.NULL;
		argument3 = VoodooConstants.NULL;
		argument4 = VoodooConstants.NULL;
		argument5 = VoodooConstants.NULL;
		board_destination = 0;
	}

	/**
	*****************************************************************************
	*	This method starts the SwingWorker thread.
	*
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public Object construct()
	{
		return apply();
	}

	/**
	*****************************************************************************
	*	This method closes the SwingWorker thread.
	*
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void finished()
	{
		// Set the expose button text back to "Send Command".
		if (parent != null)
			parent.set_command_button_text("Send Command", "Manual");

		// Re-initialize argument data members.
		argument1 = VoodooConstants.NULL;
		argument2 = VoodooConstants.NULL;
		argument3 = VoodooConstants.NULL;
		argument4 = VoodooConstants.NULL;
		argument5 = VoodooConstants.NULL;
	}

	/**
	********************************************************************************
	*	This method aborts the current thread. This method overides the one in
	*	Swingworker.java.
	*	
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
    	public void stop() {
		abort = true;

		// Reset the PCI board DSP program counter.
		try {
			dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
		}
		catch(ReplyValueException RVE) {}
		catch(Exception E) {}		
    	}

	/**
	********************************************************************************
	*	This method sets the command to be issued to the specified DSP controller.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_command(int new_command)
	{
		command = new_command;
	}
	
	/**
	*****************************************************************************
	*	This method sets the command to be issued to the specified DSP
	*	controller. This version is used to parse the text values from the
	*	DSP Manual Command Window.
	*
	*	@throws ArrayIndexOutOfBoundsException Thrown if the command
	*		textfield is left blank by the user.
	*
	*	@see VTextField
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/	
	public void set_command(String command_string) throws ArrayIndexOutOfBoundsException
	{
		byte manual_command_bytes[] = new byte[3];
		command = 0;

		// Read in the first 3 ascii characters that represent the command.
		manual_command_bytes = command_string.toUpperCase().getBytes();
		
		// Check that command does not contain any characters other than A to Z.
		if (((manual_command_bytes[0] < 0x41) || (manual_command_bytes[0] > 0x5A)) ||
		   ((manual_command_bytes[1] < 0x41) || (manual_command_bytes[1] > 0x5A)) ||
		   ((manual_command_bytes[2] < 0x41) || (manual_command_bytes[2] > 0x5A))) {
			PrintMessage.message("DspDialog", 1, "ERROR: Command can only contain characters between A & Z\n", 0, true);
			manual_command_bytes[0] = (byte)'X';
			manual_command_bytes[1] = (byte)'X';
			manual_command_bytes[2] = (byte)'X';
		}
			
		// Convert the input bytes into a single integer.
		command = (command | manual_command_bytes[0]) << 8;
		command = (command | manual_command_bytes[1]) << 8;
		command = (command | manual_command_bytes[2]);
	}

	/**
	********************************************************************************
	*	This method sets the argument 1 register to the specified value.
	*
	*	@param new_argument1 The argument value.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_arg1(int new_argument1)
	{
		argument1 = new_argument1;
	}

	/**
	********************************************************************************
	*	This method sets the argument 2 register to the specified value.
	*
	*	@param new_argument2 The argument value.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_arg2(int new_argument2)
	{
		argument2 = new_argument2;
	}

	/**
	********************************************************************************
	*	This method sets the argument 3 register to the specified value.
	*
	*	@param new_argument3 The argument value.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_arg3(int new_argument3)
	{
		argument3 = new_argument3;
	}

	/**
	********************************************************************************
	*	This method sets the argument 4 register to the specified value.
	*
	*	@param new_argument4 The argument value.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_arg4(int new_argument4)
	{
		argument4 = new_argument4;
	}

	/**
	********************************************************************************
	*	This method sets the argument 5 register to the specified value.
	*
	*	@param new_argument5 The argument value.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_arg5(int new_argument5)
	{
		argument5 = new_argument5;
	}

	/**
	********************************************************************************
	*	This method sets the specified argument register to the specified value.
	*	The input string is converted to an integer. This version is used to
	*	parse the text values from the DSP Manual Command Window.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_argument(int arg, String txtfld_string)
	{
		int manual_args = 0;

		// Check that the input string is not null.
		if (txtfld_string.compareTo("") != 0) {
			
			// Check if the argument string is already a hex number.
			// If not, convert it from an ascii string to a hex integer.
			try {
				manual_args = Integer.parseInt(txtfld_string, 16);
			} catch (Exception e) {
				// Convert the input bytes into a single hex integer.
				if (txtfld_string.length() == 1)
					manual_args = (manual_args | txtfld_string.toUpperCase().charAt(0));
				else if (txtfld_string.length() == 2) {
					manual_args = (manual_args | txtfld_string.toUpperCase().charAt(0)) << 8;
					manual_args = (manual_args | txtfld_string.toUpperCase().charAt(1));
				}
				else if (txtfld_string.length() == 3) {
					manual_args = (manual_args | txtfld_string.toUpperCase().charAt(0)) << 8;
					manual_args = (manual_args | txtfld_string.toUpperCase().charAt(1)) << 8; 
					manual_args = (manual_args | txtfld_string.toUpperCase().charAt(2));
				}
			}
			
			// Set the value.
			switch (arg) {
				case 1:
					argument1 = manual_args;
					break;
				case 2:
					argument2 = manual_args;
					break;
				case 3:
					argument3 = manual_args;
					break;
				case 4:
					argument4 = manual_args;
					break;
				case 5:
					argument5 = manual_args;
					break;
			}				
		}
	}
	
	/**
	********************************************************************************
	*	This method sets the board destination register to the board that is to
	*	receive the command. May be one of the following: VoodooConstants.TIM_ID
	*	VoodooConstants.UTIL_ID. 
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_board_destination(int new_board_destination)
	{
		board_destination = new_board_destination;
	}
	
	/**
	********************************************************************************
	*	This method sends the command and any non-null argument registers to the
	*	specified controller board.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public Object apply()
	{
		reply_value = 0;
		number_of_args = 0;
		abort = false;

		PrintMessage.message("ManualCommand", 0, "Sending manual command " + ReplyBuffer.to_ascii(command) + "...waiting.\n", 0, true);

		// Determine the number of arguments in use.
		if (argument1 != VoodooConstants.NULL)
			number_of_args++;
		if (argument2 != VoodooConstants.NULL)
			number_of_args++;
		if (argument3 != VoodooConstants.NULL)
			number_of_args++;
		if (argument4 != VoodooConstants.NULL)
			number_of_args++;
		if (argument5 != VoodooConstants.NULL)
			number_of_args++;

		if (abort) { return "Done"; }
		
		// Set the number of arguments (0x?0000 = Number of arguments (ARG#) << 16)
		// and "OR" the result with the board destination.
		try {
			dsp_cmd.set_board_destination((number_of_args << 16), board_destination);
		}
		catch (ReplyValueException rve) {
			PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
			PrintMessage.message("ManualCommand", 1, "ERROR: Board destination not set.\n", 0, true);
			print_debug();
			return "FAILED";
		}
		catch (Exception e) {
			PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
			PrintMessage.message("ManualCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			print_debug();
			return "FAILED";
		}

		if (abort) { return "Done"; }

		// If argument1 not null, set it.
		if (argument1 != VoodooConstants.NULL) {
			try {
				dsp_cmd.arg(1, argument1);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: Cannot set the arg1 register.\n", 0, true);
				PrintMessage.message("ManualCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ManualCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ManualCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				print_debug();
				return "FAILED";
			}
			catch (Exception e) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				print_debug();
				return "FAILED";
			}
		}

		if (abort) { return "Done"; }
		
		// If argument2 not null, set it.
		if (argument2 != VoodooConstants.NULL) {
			try {
				dsp_cmd.arg(2, argument2);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: Cannot set the arg2 register.\n", 0, true);
				PrintMessage.message("ManualCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ManualCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ManualCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				print_debug();
				return "FAILED";
			}
			catch (Exception e) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				print_debug();
				return "FAILED";
			}
		}
		
		if (abort) { return "Done"; }
		
		// If argument3 not null, set it.
		if (argument3 != VoodooConstants.NULL) {
			try {
				dsp_cmd.arg(3, argument3);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: Cannot set the arg3 register.\n", 0, true);
				PrintMessage.message("ManualCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ManualCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ManualCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				print_debug();
				return "FAILED";
			}
			catch (Exception e) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				print_debug();
				return "FAILED";
			}
		}
		
		if (abort) { return "Done"; }
		
		// If argument4 not null, set it.
		if (argument4 != VoodooConstants.NULL) {
			try {
				dsp_cmd.arg(4, argument4);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: Cannot set the arg4 register.\n", 0, true);
				PrintMessage.message("ManualCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ManualCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ManualCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				print_debug();
				return "FAILED";
			}
			catch (Exception e) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				print_debug();
				return "FAILED";
			}
		}
		
		if (abort) { return "Done"; }
		
		// If argument5 not null, set it.
		if (argument5 != VoodooConstants.NULL) {
			try {
				dsp_cmd.arg(5, argument5);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: Cannot set the arg5 register.\n", 0, true);
				PrintMessage.message("ManualCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("ManualCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("ManualCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				print_debug();
				return "FAILED";
			}
			catch (Exception e) {
				PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
				PrintMessage.message("ManualCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				print_debug();
				return "FAILED";
			}
		}
		
		if (abort) { return "Done"; }
		
		// Set the command.
		try {
			reply_value = dsp_cmd.cmdr(command);
			PrintMessage.message("ManualCommand", 0, "done.", 8, false);
		}
		catch (ReplyValueException rve) {
			if (abort) { return "Done"; }
			reply_value = rve.get_value();
			PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
			set_reply_text();
			print_debug();
			return "FAILED";
		}
		catch (Exception e) {
			if (abort) { return "Done"; }
			PrintMessage.message("ManualCommand", 1, "failed.", 8, false);
			PrintMessage.message("ManualCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			print_debug();
			return "FAILED";
		}
		
		if (abort) { return "Done"; }

		set_reply_text();

		print_debug();

		return (new Integer(reply_value));
	}

	/**
	********************************************************************************
	*	This method sets the reply message string in the specified parent window.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private void set_reply_text()
	{
		// Set the reply textfield to the returned value.
		if (parent != null) {
			if (reply_value == VoodooConstants.DON || reply_value == VoodooConstants.ERR || reply_value == VoodooConstants.SYR)
				parent.set_manual_reply_txtfld(ReplyBuffer.to_ascii(reply_value));

			else if (reply_value == VoodooConstants.TOUT)
				parent.set_manual_reply_txtfld("TIME OUT");

			else if (reply_value == VoodooConstants.REPLY_BUFFER_EMPTY)		
				parent.set_manual_reply_txtfld("EMPTY BUFF");

			else
				parent.set_manual_reply_txtfld(Integer.toHexString(reply_value));
		}
	}

	/**
	********************************************************************************
	*	This method prints the debug message to the info/status window.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private void print_debug()
	{
		if (abort) { return; }

		// Determine the number of arguments in use.
		if (argument1 == VoodooConstants.NULL)
			argument1 = 0;
		if (argument2 == VoodooConstants.NULL)
			argument2 = 0;
		if (argument3 == VoodooConstants.NULL)
			argument3 = 0;
		if (argument4 == VoodooConstants.NULL)
			argument4 = 0;
		if (argument5 == VoodooConstants.NULL)
			argument5 = 0;

		// Print command description to Info/Status window if debug mode is "on".
		if (OptionsCommand.get_debug_mode() == VoodooConstants.DEBUG_ON)
			PrintMessage.debug_message(((number_of_args << 16) | board_destination), command, argument1,
			                              argument2, argument3, argument4, argument5, reply_value); 
	}
}