/**
********************************************************************************
*	This class stores the application option settings. The current options
*	are:	1. Turn debug mode on/off
*		2. Turn synthetic image checking on/off
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
public class OptionsCommand
{
	private static boolean debug_mode = false;
	private static boolean image_check_mode = false;
	
	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public OptionsCommand() {}
	
	/**
	********************************************************************************
	*	This method returns the current debug mode (on or off).
	*
	*	@return Returns the current debug mode (on or off).
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static boolean get_debug_mode()
	{
		return debug_mode;
	}
	

	/**
	********************************************************************************
	*	This method sets the current debug mode (on or off).
	*
	*	@param mode The current debug mode (on or off).
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static void set_debug_mode(boolean mode)
	{
		debug_mode = mode;
	}
	
	/**
	********************************************************************************
	*	This method returns whether synthetic image checking is on or off.
	*
	*	@return Returns whether synthetic image checking is on or off.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static boolean get_image_check_mode()
	{
		return image_check_mode;
	}
	
	/**
	********************************************************************************
	*	This method turns the synthetic image checking on or off.
	*
	*	@mode The current synthetic image checking mode (on or off).
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static void set_image_check_mode(boolean mode)
	{
		int status = 0;
		int synthetic_clear_bit_mask = 0x00000400;
		DspCommand2 dsp_cmd = new DspCommand2();		

		image_check_mode = mode;
		
		// Set the board destination to Timing and memory type to X.
		try {
			dsp_cmd.set_board_destination(VoodooConstants.RDM_ARG_NUM, VoodooConstants.TIM_ID);
			dsp_cmd.set_type(VoodooConstants.X);
		}
		catch (ReplyValueException rve) {
				PrintMessage.message("OptionsCommand", 1, "ERROR: Board destination not set.\n", 0, true);
		}
		catch (Exception e) {
				PrintMessage.message("OptionsCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
		}

		// Read the controller status word from the Timing board.
		try {
			status = dsp_cmd.read_memory(0);
		}
		catch (ReplyValueException rve) {
				PrintMessage.message("OptionsCommand", 1, "ERROR: Could not read controller status word at X0.\n", 0, true);
				PrintMessage.message("OptionsCommand", 1, "     Value read = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
		}
		catch (Exception e) {
				PrintMessage.message("OptionsCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
		}

		// Set the board destination to Timing and memory type to X.
		try {
			dsp_cmd.set_board_destination(VoodooConstants.WRM_ARG_NUM, VoodooConstants.TIM_ID);
			dsp_cmd.set_type(VoodooConstants.X);
		}
		catch (ReplyValueException rve) {
				PrintMessage.message("OptionsCommand", 1, "ERROR: Board destination not set.\n", 0, true);
		}
		catch (Exception e) {
				PrintMessage.message("OptionsCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
		}

		// Set the PCI board to synthetic image mode. Set/clear bit 10 of X0 on Timing board.
		try {
			if (image_check_mode)
				dsp_cmd.write_memory(0, (status | synthetic_clear_bit_mask));
			else
				dsp_cmd.write_memory(0, (status & ~synthetic_clear_bit_mask));
		}
		catch (ReplyValueException rve) {
				if (image_check_mode)
					PrintMessage.message("OptionsCommand", 1, "ERROR: PCI board not set to synthetic image mode\n", 0, true);
				else
					PrintMessage.message("OptionsCommand", 1, "ERROR: PCI board not set to normal image mode\n", 0, true);
				PrintMessage.message("OptionsCommand", 1, "     Expected reply = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				PrintMessage.message("OptionsCommand", 1, "     Actual reply = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);	
		}
		catch (Exception e) {
				PrintMessage.message("OptionsCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
		}
		
		dsp_cmd = null;
	}
}