/**
********************************************************************************
*	This class downloads PCI boot code from a file to the PCI board DSP.
*	The file download must be performed before any other controller actions
*	occur, otherwise, the PCI board's actions are undefined.
*
*	NOTE: This class does not use the DspCommand class methods because they
*	      all require return values, which the PCI board cannot do at the
*	      time of this download. The PCI board can only reply after this
*	      download is complete.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/

import java.io.*;
import java.util.*;

public class PCIBoot
{
	private RandomAccessFile bootFile;
		
	// Define constants.
	private final String PCI_STRING = "PCIBOOT";
	private final String SEARCH_STRING = "_DATA P";
	private final String END_STRING = "_END";
	private final int HTF_MASK = 0x200;
	private final int HTF_CLEAR_MASK = 0xFFFFFCFF;		// Bits 8 and 9.
	private final int BIT3_CLEAR_MASK = 0xFFFFFFF7;
	private final int BIT3_SET_MASK = 0x00000008;
	
	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public PCIBoot() {}

	/**
	********************************************************************************
	*	This methods downloads the specified boot file into the PCI DSP. Once
	*	loaded, the PCI can begin command processing.
	*
	*	@param filename The name of the file to download.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void boot(String filename) throws ReplyValueException, Exception, FileNotFoundException, IOException, NullPointerException
	{
		boolean end_of_file;
		boolean done_reading_data;
		String input_line;
		String words_string;
		String addr_string;
		String data_string;
		String temp_string;
		long file_position;
		int data;
		int reply_value = 0;
		int word_total = 0;
		int word_count = 0;
		int address = 0;
		int count = 0;
		
		bootFile = new RandomAccessFile(filename, "r");
		
		// Check the file validity.
		input_line = bootFile.readLine();
		if (input_line.indexOf(PCI_STRING) == -1) {
			bootFile.close();
			throw new Exception("Invalid PCI file, no PCIBOOT string found.");
		}

		// Set the PCI board to slave mode by clearing bit 3 and setting bits 8 & 9 (HTF bits).
		reply_value = VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_GET_HCTR, 0);

		// Clear the HTF bits (8 and 9) and bit3.
		reply_value = reply_value & HTF_CLEAR_MASK & BIT3_CLEAR_MASK;
		
		// Set the HTF bits.
		reply_value = reply_value | HTF_MASK;
		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCTR, reply_value);

		// Inform the DSP that new pci boot code will be downloaded.
		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCVR, VoodooConstants.PCI_DOWNLOAD);

		// Set the magic number that says this is a pci boot load.
		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, 0x00555AAA);
						
		end_of_file = false;
		done_reading_data = false;
		file_position = 0;
		
		while(!done_reading_data) {
			input_line = bootFile.readLine();

			if (input_line.indexOf(SEARCH_STRING) != -1) {
				// Get the next line.
				input_line = bootFile.readLine();
				
				// Break the input line into pieces.
				StringTokenizer tokenizer = new StringTokenizer(input_line, " ");
				
				// Get the number of words and starting address.
				words_string = tokenizer.nextToken();
				addr_string = tokenizer.nextToken();
							
				word_total = Integer.parseInt(words_string, 16);
				VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, word_total);

				address = Integer.parseInt(addr_string, 16);
				VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, address);
				
				// Throw away the next line (example: _DATA P 000002).
				input_line = bootFile.readLine();
				
				// Load the data.
				while (word_count < word_total) {
					// Get the next line, this is the data start.
					input_line = bootFile.readLine();

					// Check for intermixed "_DATA" strings.
					if (input_line.indexOf(SEARCH_STRING) == -1) {
						tokenizer = new StringTokenizer(input_line, " ");
					
						while (tokenizer.hasMoreTokens()) {
							// Check the word count to prevent going over.
							if (word_count >= word_total)
								break;

							data_string = tokenizer.nextToken();
							word_count++;

							// Send command.
							data = Integer.parseInt(data_string, 16);
							VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_ARG1, data);
						}
					}
				}
				done_reading_data = true;
			}
		}
		reply_value = 0;
		// Set the PCI board data transfer format (Set HTF bits to 00).
		reply_value = VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_GET_HCTR, 0);
		reply_value = reply_value & HTF_CLEAR_MASK;
		VoodooSetup.Cioctl(VoodooSetup.pci_fd, VoodooConstants.ASTROPCI_SET_HCTR, reply_value);

		// Read the reply.		
		reply_value = ReplyBuffer.read_reply(VoodooConstants.DON);

		bootFile.close();

		try {
			DspCommand2.flush_reply_buffer();
		}
		catch (Exception e) {}
	}
}