/**
********************************************************************************
*	This class sends commands to the PCI interface board, Timing, and Utility
*	controller boards.
*
*	CVM - Model
*
*	@see SwingWorker
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.io.*;
import java.util.*;
import javax.swing.*;

public class PciTimUtilCommand extends SwingWorker
{
	private DspDialog parent = null;
	private DspCommand2 dsp_cmd;
	private int address;
	private int data;
	private int board;
	private int command;
	private int type;
	
	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public PciTimUtilCommand(DspDialog parent)
	{
		this.parent = parent;
		dsp_cmd = new DspCommand2();
	}

	/**
	********************************************************************************
	*	This method initiates the download. It is automatically called by the
	*	SwingWorker class constructor.
	*	
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public Object construct()
	{
		return send_command();
	}

	/**
	********************************************************************************
	*	This method sets the "Apply" button back to "Apply". This method is
	*	automatically called by the SwingWorker class.
	*	
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void finished()
	{
		// Set the apply button text back to "Send Command".
		parent.set_command_button_text("Send Command", "TimUtilPci");
		dsp_cmd.abort = false;
	}

	/**
	********************************************************************************
	*	This method aborts the current thread. This method overides the one in
	*	Swingworker.java.
	*	
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
    	public void stop()
    	{
		abort = true;
		dsp_cmd.abort = true;

		// Reset the PCI board DSP program counter.
		try {
			dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
		}
		catch(ReplyValueException RVE) {}
		catch(Exception E) {}		
    	}
	
	/**
	********************************************************************************
	*	This method sets the address of the memory location on the specified
	*	board. Only the lower 24 bits of the input string are parsed, all
	*	others are ignored. This is because the DSP only takes 24 bit data.
	*	
	*	@param address The address of the memory location.
	*
	*	@throws NumberFormatException Thrown if the command textfield is left
	*				      blank by the user or if an invalid
	*				      character is supplied by the user.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_address(String addr_string) throws NumberFormatException
	{
		try {
			address = Integer.parseInt(addr_string, 16) & 0x00FFFFFF;
		}
		catch (NumberFormatException nfe) {
			throw new NumberFormatException("ERROR: Cannot parse address textfield");
		}
	}

	/**
	********************************************************************************
	*	This method sets the data to be sent to the specified board. Only the
	*	lower 24 bits of the input string are parsed, all others are ignored.
	*	This is because the DSP only takes 24 bit data.
	*	
	*	@param data The data to send.
	*
	*	@throws NumberFormatException Thrown if the command textfield is left
	*				      blank by the user or if an invalid
	*				      character is supplied by the user.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_data(String data_string) throws NumberFormatException
	{
		try {
			data = Integer.parseInt(data_string, 16) & 0x00FFFFFF;
		}
		catch (NumberFormatException nfe) {
			throw new NumberFormatException("ERROR: Cannot parse address textfield");
		}
	}

	/**
	********************************************************************************
	*	This method sets the board to send the command too.
	*	
	*	@param board The board to send the command too.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_board(int new_board)
	{
		board = new_board;
	}

	/**
	********************************************************************************
	*	This method sets the memory type to be written too on the specified board.
	*	
	*	@param type The memory type.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_type(int new_type)
	{
		type = new_type;
	}

	/**
	********************************************************************************
	*	This method sets the command to be sent to the specified board.
	*	
	*	@param command The command to send.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_command(int new_command)
	{
		command = new_command;
	}
	
		
	/**
	********************************************************************************
	*	This method sends all the setup information to the PCI interface board.
	*	
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	private Object send_command()
	{
		int reply_value = 0;
		abort = false;
		
		if (abort) { return "Done"; }
		
		// Set the memory type.
		dsp_cmd.set_type(type);

		switch(command) {
			case VoodooConstants.TEST_DATA_LINK:
				try {
					dsp_cmd.set_board_destination(VoodooConstants.TDL_ARG_NUM, board);
				}
				catch(ReplyValueException rve) {
					if (abort) { return "Done"; }
					PrintMessage.message("PciTimUtilCmdListener", 1, "ERROR: Board destination not set.\n", 0, true);
					return "FAILED";
				}
				catch(Exception e) {
					if (abort) { return "Done"; }
					PrintMessage.message("PciTimUtilCmdListener", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					return "FAILED";
				}

				if (abort) { return "Done"; }

				try {
					PrintMessage.message("PciTimUtilCmdListener", 0, "Sending TEST_DATA_LINK command...waiting.\n", 0, true);
					reply_value = dsp_cmd.test_data_link(data);
					PrintMessage.message("PciTimUtilCmdListener", 0, "done.", 8, false);
					if (abort) { return "Done"; }
				}
				catch(ReplyValueException rve) {
					if (abort) { return "Done"; }
					reply_value = rve.get_value();
					PrintMessage.message("PciTimUtilCmdListener", 1, "failed.", 8, false);
					PrintMessage.message("PciTimUtilCmdListener", 1, "     Expected value = 0x" + Integer.toHexString(data) + "\n", 0, true);
					if (rve.get_value() == VoodooConstants.TOUT)
						PrintMessage.message("PciTimUtilCmdListener", 1, "     Actual value = TIMEOUT\n", 0, true);
					else
						PrintMessage.message("PciTimUtilCmdListener", 1, "     Actual value = 0x" + Integer.toHexString(reply_value) + "\n", 0, true);
				}
				catch(Exception e) {
					if (abort) { return "Done"; }
					PrintMessage.message("PciTimUtilCmdListener", 1, "failed.", 8, false);
					PrintMessage.message("PciTimUtilCmdListener", 1, "PCI board not ready to process commands.\n", 0, true);
				}
				break;
			
			case VoodooConstants.READ_MEMORY:
				try {
					dsp_cmd.set_board_destination(VoodooConstants.RDM_ARG_NUM, board);
				}
				catch(ReplyValueException rve) {
					PrintMessage.message("PciTimUtilCmdListener", 1, "ERROR: Board destination not set.\n", 0, true);
					return "FAILED";
				}
				catch(Exception e) {
					if (abort) { return "Done"; }
					PrintMessage.message("PciTimUtilCmdListener", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					return "FAILED";
				}

				if (abort) { return "Done"; }

				try {
					PrintMessage.message("PciTimUtilCmdListener", 0, "Sending READ_MEMORY command...waiting.\n", 0, true);
					reply_value = dsp_cmd.read_memory(address);
					PrintMessage.message("PciTimUtilCmdListener", 0, "done.", 8, false);
					if (abort) { return "Done"; }
				}
				catch(ReplyValueException rve) {
					if (abort) { return "Done"; }
					reply_value = rve.get_value();
					PrintMessage.message("PciTimUtilCmdListener", 1, "failed.", 8, false);
					if (rve.get_value() == VoodooConstants.TOUT)
						PrintMessage.message("PciTimUtilCmdListener", 1, "ERROR: Returned value = TIMEOUT\n", 0, true);
					else
						PrintMessage.message("PciTimUtilCmdListener", 1, "ERROR: Returned value = 0x" + Integer.toHexString(reply_value) + "\n", 0, true);
				}
				catch(Exception e) {
					if (abort) { return "Done"; }
					PrintMessage.message("PciTimUtilCmdListener", 1, "failed.", 8, false);
					PrintMessage.message("PciTimUtilCmdListener", 1, "PCI board not ready to process commands.\n", 0, true);
				}
				break;
			
			case VoodooConstants.WRITE_MEMORY:
				try {
					dsp_cmd.set_board_destination(VoodooConstants.WRM_ARG_NUM, board);
				}
				catch(ReplyValueException rve) {
					PrintMessage.message("PciTimUtilCmdListener", 1, "ERROR: Board destination not set.\n", 0, true);
					return "FAILED";
				}
				catch(Exception e) {
					if (abort) { return "Done"; }
					PrintMessage.message("PciTimUtilCmdListener", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					return "FAILED";
				}

				if (abort) { return "Done"; }

				try {
					PrintMessage.message("PciTimUtilCmdListener", 0, "Sending WRITE_MEMORY command...waiting.\n", 0, true);
					reply_value = dsp_cmd.write_memory(address, data);
					PrintMessage.message("PciTimUtilCmdListener", 0, "done.", 8, false);
					if (abort) { return "Done"; }
				}
				catch(ReplyValueException rve) {
					if (abort) { return "Done"; }
					reply_value = rve.get_value();
					PrintMessage.message("PciTimUtilCmdListener", 1, "failed.", 8, false);
					PrintMessage.message("PciTimUtilCmdListener", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
					if (rve.get_value() == VoodooConstants.TOUT)
						PrintMessage.message("PciTimUtilCmdListener", 1, "     Actual value = TIMEOUT\n", 0, true);
					else
						PrintMessage.message("PciTimUtilCmdListener", 1, "     Actual value = 0x" + Integer.toHexString(reply_value) + "\n", 0, true);
				}
				catch(Exception e) {
					if (abort) { return "Done"; }
					PrintMessage.message("PciTimUtilCmdListener", 1, "failed.", 8, false);
					PrintMessage.message("PciTimUtilCmdListener", 1, "PCI board not ready to process commands.\n", 0, true);
				}
				break;
		}

		if (abort) { return "Done"; }
		
		// Set the reply value text.
		if (reply_value == VoodooConstants.DON || reply_value == VoodooConstants.ERR || reply_value == VoodooConstants.SYR)
			parent.set_timing_reply_txtfld(ReplyBuffer.to_ascii(reply_value));

		else if (reply_value == VoodooConstants.TOUT)
			parent.set_timing_reply_txtfld("TIMEOUT");

		else if (reply_value == VoodooConstants.REPLY_BUFFER_EMPTY)		
			parent.set_timing_reply_txtfld("REPLY BUFFER EMPTY");

		else {
			// Use only the lower 6 bits of the reply value.
			reply_value = reply_value & 0x00FFFFFF;
			parent.set_timing_reply_txtfld(Integer.toHexString(reply_value));
		}

		return (new Integer(reply_value));
	}
}