/**
********************************************************************************
*	This class reads the current value of the reply buffer.
*
*	CVM - Model
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import javax.swing.*;
import java.awt.event.*;

public class ReplyBuffer
{
	private static int reply_value = 0;
    	
	/**
	********************************************************************************
	*	Default constructor.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
    	public ReplyBuffer() {}
    
	/**
	********************************************************************************
	*	This method reads the current value in the reply buffer. If a hardware
	*	reset (RST/SYR) is found, the reply buffer is flushed, a warning dialog
	*	is displayed, and the reset reply value is returned.
	*
	*	@throws ReplyValueException If the buffer fails to contain a value after
	*				    10 reads or the buffer is empty.
	*
	*	@version 1.3
	*	@author Scott Streit	
	********************************************************************************
	*/
   	public static int read_reply() throws ReplyValueException
    	{
		reply_value = 0;

		// Read the reply.
    		try {
			reply_value = VoodooSetup.Cread_reply(VoodooSetup.pci_fd, 0);
		}
		catch (Exception e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Device Driver Error", JOptionPane.ERROR_MESSAGE);
			return reply_value;
		}

		// Check for a timeout to prevent a hangup.
		if (reply_value == VoodooConstants.TOUT) {
			try {
				DspCommand2.flush_reply_buffer();
				throw new ReplyValueException(reply_value);
			}
			catch (Exception e) {
				PrintMessage.message("ReplyBuffer", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			}
		}
		
		// Check for a "hardware button" reset.
		if (reply_value == VoodooConstants.RST || reply_value == VoodooConstants.SYR) {
			try {
				DspCommand2.flush_reply_buffer();
				String message = "The contoller has experienced an external reset\nand Voodoo is now out of sync with the hardware,\n please re-apply the contoller setup.";
				JOptionPane.showMessageDialog(null, message, "Reply Warning", JOptionPane.WARNING_MESSAGE);
			}
			catch (Exception e) {
				PrintMessage.message("ReplyBuffer", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			}		
		}

	
		return reply_value;
    	}
    
	/**
	********************************************************************************
	*	This method reads the current value in the reply buffer and compares it
	*	to the <I>expected value</I>. If a hardware reset (RST/SYR) is found,
	*	the reply buffer is flushed, a warning dialog is displayed, and the
	*	reset reply value is returned.
	*
	*	@param expected_value The expected reply value. This value is compared
	*			      against the actual reply value.
	*
	*	@throws ReplyValueException If the buffer fails to contain a value after
	*				    10 reads, the buffer is empty, or the value
	*				    does not match the expected value.
	*
	*	@version 1.3
	*	@author Scott Streit	
	********************************************************************************
	*/
    	public static int read_reply(int expected_value) throws ReplyValueException
    	{
		reply_value = 0;
		
		// Read the reply.
		try {
			reply_value = VoodooSetup.Cread_reply(VoodooSetup.pci_fd, 0);
		}
		catch (Exception e) {
			JOptionPane.showMessageDialog(null, e.getMessage(), "Device Driver Error", JOptionPane.ERROR_MESSAGE);
			return reply_value;
		}

		// If there is no buffer value, set reply value to ERR. Also check
		// for a timeout to prevent a hangup.
		if (reply_value == VoodooConstants.TOUT) {
			try {
				DspCommand2.flush_reply_buffer();
				throw new ReplyValueException(reply_value);
			}
			catch (Exception e) {
				PrintMessage.message("ReplyBuffer", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			}
		}

		// Check for a "hardware button" reset.
		if ((reply_value == VoodooConstants.RST || reply_value == VoodooConstants.SYR) && (reply_value != expected_value)) {
			try {
				DspCommand2.flush_reply_buffer();
				String message = "The contoller has experienced an external reset\nand Voodoo is now out of sync with the hardware,\n please re-apply the contoller setup.";
				JOptionPane.showMessageDialog(null, message, "Reply Warning", JOptionPane.WARNING_MESSAGE);
			}
			catch (Exception e) {
				PrintMessage.message("ReplyBuffer", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			}
		}
		
		// Compare the reply value against the user's expected value.
		if (reply_value != expected_value) {
			try {
				DspCommand2.flush_reply_buffer();
				throw new ReplyValueException(reply_value);
			}
			catch (Exception e) {
				PrintMessage.message("ReplyBuffer", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			}
		}
		
		return reply_value;   
	}

	/**
	********************************************************************************
	*	Converts a 24-bit integer into it's 3 character ascii equivalent. Used
	*	to output more descriptive DSP return values.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public static String to_ascii(int value)
	{
		char ascii_value[] = new char[3];
		ascii_value[0] = (char)((value & 0xff0000) >> 16);
		ascii_value[1] = (char)((value & 0x00ff00) >> 8);
		ascii_value[2] = (char)(value & 0x0000ff);

		return (new String(ascii_value));
	}
}
