/**
********************************************************************************
*	This class sends all the Controller Setup information to the PCI
*	interface board.
*
*	CVM - Model
*
*	@see SwingWorker
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.io.*;
import java.util.*;
import javax.swing.*;

public class SetupCommand extends SwingWorker
{
	protected SetupDialog parent;
	private boolean do_reset_controller;
	private boolean do_power_on;
	private boolean do_hardware_test;
	private boolean do_pci_hardware_test;
	private boolean do_timing_hardware_test;
	private boolean do_utility_hardware_test;
	private boolean do_pci;
	private boolean do_timing;
	private boolean do_timing_file;
	private boolean do_timing_app;
	private boolean do_utility;
	private boolean do_utility_file;
	private boolean do_utility_app;
	private boolean do_dimensions;
	private boolean did_timing;
	private boolean setup_applied;
	private int image_bytes;
	private String pci_filename;
	private String timing_filename;
	private String utility_filename;
	private int tim_app_number;
	private int util_app_number;
	private int number_of_pci_tests;
	private int number_of_timing_tests;
	private int number_of_utility_tests;
	private PCIBoot pci_boot;
	private LoadDspFile dsp_file;
	private DspCommand2 dsp_cmd;
	private int tim_done;
	private int util_done;
	private int bits_per_pixel;
	private int array_columns;
	private int array_rows;
	protected int config_word;
	
  	// C function prototypes.
  	public native static int Cdeinterlace(int rows, int cols, int image_fd, int algorithm);

  	// Load the library that contains the C functions.
  	static
  	{
    		System.loadLibrary("setupcmdlibc");
  	}
		
	/**
	********************************************************************************
	*	Class constructor. The default configuration word contains: shutter.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public SetupCommand(SetupDialog parent) 
	{
		this.parent = parent;
		config_word = 0;
		dsp_file = new LoadDspFile();
		dsp_cmd = new DspCommand2();
		pci_boot = new PCIBoot();

		// Initialize class data members.
		do_reset_controller = false;
		do_power_on = false;
		do_hardware_test = false;
		do_pci_hardware_test = false;
		do_timing_hardware_test = false;
		do_utility_hardware_test = false;
		do_pci = false;
		do_timing = false;
		do_timing_file = true;
		do_timing_app = false;
		do_utility = false;
		do_utility_file = true;
		do_utility_app = false;
		do_dimensions = false;
		did_timing = false;
		image_bytes = 0;
		pci_filename = null;
		timing_filename = null;
		utility_filename = null;
		tim_app_number = 3;
		util_app_number = 0;
		number_of_pci_tests = 0;
		number_of_timing_tests = 0;
		number_of_utility_tests = 0;
		setup_applied = false;
		bits_per_pixel = 16;
		array_columns = 0;
		array_rows = 0;
	}

	/**
	********************************************************************************
	*	This method initiates the download. It is automatically called by the
	*	SwingWorker class constructor.
	*	
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public Object construct()
	{
		return apply();
	}

	/**
	********************************************************************************
	*	This method sets the "Apply" button back to "Apply". This method is
	*	automatically called by the SwingWorker class.
	*	
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void finished()
	{
		// Set the apply button text back to "Apply".
		parent.apply_button.setText("Apply");
		dsp_cmd.abort = false;
	}

	/**
	********************************************************************************
	*	This method aborts the current thread. This method overides the one in
	*	Swingworker.java.
	*	
	*	@see SwingWorker
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
    	public void stop() {
		abort = true;
		dsp_cmd.abort = true;
		
		// Reset the PCI board DSP program counter.
		try {
			dsp_cmd.hcvr(VoodooConstants.PCI_PC_RESET, VoodooConstants.DON);
		}
		catch(ReplyValueException RVE) {}
		catch(Exception E) {}
    	}

	/**
	********************************************************************************
	*	This method sets the controller reset to on/off.
	*	
	*	@param reset_controller True to reset controller, false to not reset the
	*				controller.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_reset_controller(boolean reset_controller)
	{
		do_reset_controller = reset_controller;
	}
	
	/**
	********************************************************************************
	*	This method sets the power on to on/off.
	*	
	*	@param power_on True to power on the controller, false to not power on
	*			the controller.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_power_on(boolean power_on)
	{
		do_power_on = power_on;
	}
	
	/**
	********************************************************************************
	*	This method sets the hardware tests to on/off. The tests consist of a
	*	series of Test Data Link commands.
	*	
	*	@param hardware_test True to perform the controller hardware tests, false
	*			     to not perform the tests.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_hardware_test(boolean hardware_test)
	{
		do_hardware_test = hardware_test;
	}

	/**
	********************************************************************************
	*	This method sets the PCI hardware tests to on/off. The tests consist of
	*	a series of Test Data Link commands.
	*	
	*	@param do_pci_test True to perform the PCI hardware tests, false to not
	*			   perform the tests.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_pci_test(boolean do_pci_test)
	{
		do_pci_hardware_test = do_pci_test;
	}

	/**
	********************************************************************************
	*	This method sets the Timing hardware tests to on/off. The tests consist
	*	of a series of Test Data Link commands.
	*	
	*	@param do_timing_test True to perform the Timing hardware tests, false
	*			      to not perform the tests.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_timing_test(boolean do_timing_test)
	{
		do_timing_hardware_test = do_timing_test;
	}

	/**
	********************************************************************************
	*	This method sets the Utility hardware tests to on/off. The tests consist
	*	of a series of Test Data Link commands.
	*	
	*	@param do_utility_test True to perform the Utility hardware tests, false
	*			       to not perform the tests.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_utility_test(boolean do_utility_test)
	{
		do_utility_hardware_test = do_utility_test;
	}

	/**
	********************************************************************************
	*	This method sets the PCI download to on/off.
	*	
	*	@param pci True to perform the PCI download, false to not perform
	*		   the download.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_pci(boolean pci)
	{
		do_pci = pci;
	}

	/**
	********************************************************************************
	*	This method sets the Timing download to on/off.
	*	
	*	@param timing True to perform the Timing download, false to not
	*		      perform the download.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_timing(boolean timing)
	{
		do_timing = timing;
	}

	/**
	********************************************************************************
	*	This method sets the Timing file download to on/off.
	*	
	*	@param timing_file True to perform the Timing file download, false to not
	*		      	   perform the file download.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_timing_file(boolean timing_file)
	{
		do_timing_file = timing_file;
	}

	/**
	********************************************************************************
	*	This method sets the Timing application download to on/off.
	*	
	*	@param timing_app True to perform the Timing application download, false to
	*		      	  not perform the application download.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_timing_app(boolean timing_app)
	{
		do_timing_app = timing_app;
	}
	
	/**
	********************************************************************************
	*	This method sets the Utility download to on/off.
	*	
	*	@param utility True to perform the Utility download, false to not
	*		      perform the download.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_utility(boolean utility)
	{
		do_utility = utility;
	}

	/**
	********************************************************************************
	*	This method sets the Utility file download to on/off.
	*	
	*	@param utility_file True to perform the Utility file download, false to not
	*		      	    perform the file download.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_utility_file(boolean utility_file)
	{
		do_utility_file = utility_file;
	}

	/**
	********************************************************************************
	*	This method sets the Utility application download to on/off.
	*	
	*	@param utility_app True to perform the Utility application download, false to
	*		      	   not perform the application download.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_utility_app(boolean utility_app)
	{
		do_utility_app = utility_app;
	}
	
	/**
	********************************************************************************
	*	This method sets the array dimensions control to on/off.
	*	
	*	@param dimensions True to set the array dimensions, false to not set the
	*		    	  array dimensions.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_dimensions(boolean dimensions)
	{
		do_dimensions = dimensions;
	}
	
	/**
	********************************************************************************
	*	This method sets the PCI file to download.
	*	
	*	@param pci_file The PCI file to download.
	*
	*	@throws Exception If no file is specified.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_pci_filename(String pci_file) throws Exception
	{
		if (do_pci) {
			// Check for empty textfield.
			if (pci_file.equals(""))
				throw new Exception("ERROR: No Pci file specified");
		
			pci_filename = pci_file;
		}
	}
	
	/**
	********************************************************************************
	*	This method sets the Timing file to download.
	*	
	*	@param timing_file The Timing file to download.
	*
	*	@throws Exception If no file is specified.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_timing_filename(String timing_file) throws Exception
	{
		if (do_timing && do_timing_file) {
			// Check for empty textfield.
			if (timing_file.equals(""))
				throw new Exception("ERROR: No timing file specified");
		
			timing_filename = timing_file;
		}
	}

	/**
	********************************************************************************
	*	This method sets the Utility file to download.
	*	
	*	@param utility_file The Utility file to download.
	*
	*	@throws Exception If no file is specified.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_utility_filename(String utility_file) throws Exception
	{
		if (do_utility && do_utility_file) {
			// Check for empty textfield.
			if (utility_file.equals(""))
				throw new Exception("ERROR: No utility file specified");
		
			utility_filename = utility_file;
		}
	}

	/**
	********************************************************************************
	*	This method sets the Timing application to download. The application
	*	number is parsed from a String to an int.
	*	
	*	@param timing_app The Timing application to download.
	*
	*	@throws NumberFormatException If textfield could not be parsed, if
	*		parsed value is out of range, or if no application was specified.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_timing_application(String timing_app) throws NumberFormatException
	{
		if (do_timing && do_timing_app) {
			try {
				// Check for empty textfield.
				if (timing_app.equals("")) {
					tim_app_number = -1;
					throw new NumberFormatException("ERROR: No Timing application specified");
				}

				tim_app_number = Integer.parseInt(timing_app);

				// Check that the application number is within range.
				if (tim_app_number < 0 || tim_app_number > 3) {
					JOptionPane.showMessageDialog(null, "Error: Timing application number " +
						tim_app_number + " out of range (0-3)", "ERROR", JOptionPane.ERROR_MESSAGE);
					throw new NumberFormatException();
				}
			}
			catch (NumberFormatException nfe) {
				tim_app_number = -1;
				throw new NumberFormatException("ERROR: Could not parse timing app textfield");
			}
		}
	}
	
	/**
	********************************************************************************
	*	This method sets the Utility application to download. The application
	*	number is parsed from a String to an int.
	*	
	*	@param utility_app The Utility application to download.
	*
	*	@throws NumberFormatException If textfield could not be parsed, if
	*		parsed value is out of range, or if no application was specified.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_utility_application(String utility_app) throws NumberFormatException
	{
		if (do_utility && do_utility_app) {
			try {
				// Check for empty textfield.
				if (utility_app.equals("")) {
					util_app_number = -1;
					throw new NumberFormatException("ERROR: No utility application specified");
				}

				util_app_number = Integer.parseInt(utility_app);

				// Check that the application number is within range.
				if (util_app_number < 0 || util_app_number > 3) {
					JOptionPane.showMessageDialog(null, "Error: Utility application number " +
						util_app_number + " out of range (0-3)", "ERROR", JOptionPane.ERROR_MESSAGE);
					throw new NumberFormatException();
				}
			}
			catch (NumberFormatException nfe) {
				util_app_number = -1;
				throw new NumberFormatException("ERROR: Could not parse utility app textfield");
			}
		}
	}

	/**
	********************************************************************************
	*	This method sets the number of hardware tests to perform. The tests
	*	consist of the specified number of Test Data Link commands.
	*	
	*	@param board A int specifying the board to set the number of tests
	*		     for. Can be 0 (VoodooConstants.PCI_ID), 1
	*		     (VoodooConstants.TIM_ID), or 2 (VoodooConstants.UTIL_ID).
	*	@param number_of_tests The number of tests to perform for the specified
	*			       board.
	*
	*	@throws NumberFormatException If textfield could not be parsed or if
	*		the board id is invalid. Valid board id's are
	*		VoodooConstants.PCI_ID, VoodooConstants.TIM_ID, or
	*		VoodooConstants.UTIL_ID.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_hardware_test(int board, String number_of_tests) throws NumberFormatException
	{
		if (do_hardware_test) {
			try {
				switch (board) {
					case VoodooConstants.PCI_ID:
						number_of_pci_tests = Integer.parseInt(number_of_tests);
						break;
					case VoodooConstants.TIM_ID:
						number_of_timing_tests = Integer.parseInt(number_of_tests);
						break;
					case VoodooConstants.UTIL_ID:
						number_of_utility_tests = Integer.parseInt(number_of_tests);
						break;
					default:
						throw new NumberFormatException("ERROR: Invalid board id: " + Integer.toString(board) + "\n");
				}
			}
			catch (NumberFormatException nfe) {
				throw new NumberFormatException("ERROR: Could not parse a hardware test textfield");
			}
		}
	}
	
	/**
	********************************************************************************
	*	This method gets the number of array rows (in pixels).
	*	
	*	@return Returns the number of array rows (in pixels).
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public int get_ccd_rows()
	{
		return array_rows;
	}
	
	/**
	********************************************************************************
	*	This method gets the number of array columns (in pixels).
	*	
	*	@return Returns the number of array columns (in pixels).
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public int get_ccd_cols()
	{
		return array_columns;
	}

	/**
	********************************************************************************
	*	This method sets the image resolution in bits/pixel.
	*	
	*	@param bpp The image resolution in bits/pixel.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_image_resolution(int bpp)
	{
		bits_per_pixel = bpp;
	}

	/**
	********************************************************************************
	*	This method gets the image resolution in bits/pixel.
	*	
	*	@return Returns the image resolution in bits/pixel.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public int get_image_resolution()
	{
		return bits_per_pixel;
	}

	/**
	********************************************************************************
	*	This method sets the number of bytes in the image.
	*	
	*	@param columns The number of columns in the image (in pixels).
	*	@param rows The number of rows in the image (in pixels).
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_image_bytes(int columns, int rows)
	{
		image_bytes = columns*rows*2;
	}

	/**
	********************************************************************************
	*	This method sets the number of array rows (in pixels).
	*	
	*	@param rows The number of array rows (in pixels).
	*
	*	@throws NumberFormatException If the textfield could not be parsed.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_array_rows(String rows) throws NumberFormatException
	{
		if (do_dimensions) {
			try {
				array_rows = Integer.parseInt(rows);
			}
			catch (NumberFormatException nfe) {
				array_rows = 0;
				throw new NumberFormatException("ERROR: Could not parse rows textfield");
			}
		}
	}
	
	/**
	********************************************************************************
	*	This method sets the number of array columns (in pixels).
	*	
	*	@param columns The number of array columns (in pixels).
	*
	*	@throws NumberFormatException If the textfield could not be parsed.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void set_array_cols(String columns) throws NumberFormatException
	{
		if (do_dimensions) {
			try {
				array_columns = Integer.parseInt(columns);
			}
			catch (NumberFormatException nfe) {
				array_columns = 0;
				throw new NumberFormatException("ERROR: Could not parse columns textfield");
			}
		}
	}

	/**
	********************************************************************************
	*	This method gets the number of bytes in the image.
	*	
	*	@return Returns the number of bytes in the image.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public int get_image_bytes()
	{
		return image_bytes;
	}

	/**
	********************************************************************************
	*	This method checks that the apply button has been pushed and that the
	*	dimensions have been set (actually that they have been attempted to be
	*	set.
	*
	*	@return Returns true if the dimensions have been set and the apply button
	*		has been pushed. Else, returns false.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public boolean setup_applied()
	{
		return setup_applied;
	}

	/**
	********************************************************************************
	*	This method sets all the SetupDialog class user input components to
	*	their default values and clears all textfields.
	*	
	* 	@see SetupDialog
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public void clear()
	{
		// Clear all values.
		do_reset_controller = false;
		do_power_on = false;
		do_hardware_test = false;
		do_timing_hardware_test = false;
		do_utility_hardware_test = false;
		do_pci = false;
		do_timing = false;
		do_timing_file = false;
		do_timing_app = false;
		do_utility = false;
		do_utility_file = false;
		do_utility_app = false;
		do_dimensions = false;
		array_columns = 0;
		array_rows = 0;
		image_bytes = 0;
		bits_per_pixel = 16;
	}
		
	/**
	********************************************************************************
	*	This method sends all the setup information to the PCI interface board.
	*	
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	private Object apply()
	{
		// Check that something has been applied and print a return. For asthetics.
		if (do_reset_controller || do_power_on || do_hardware_test || do_pci || do_timing || do_utility || do_dimensions)
			PrintMessage.message("SetupCommand", 0, "\n", 0, true);

		// Flush the PCI board reply buffer.
		try {
			dsp_cmd.flush_reply_buffer();
		}
		catch (Exception e) {
			PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
		}		

		// Update FITS header.
		parent.parent.fits_dialog.update_fits_info();
			
		if (do_pci && !abort) {
			try{
				// Load the specified DSP pci file.
				PrintMessage.message("SetupCommand", 0, "Loading Pci file...waiting.\n", 0, true);
				pci_boot.boot(pci_filename);
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				return "ERROR";
			}
			catch (IOException ioe) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: Cannot load file.\n", 0, true);
				return "ERROR";
			}
			catch (NoSuchElementException nsee) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: Invalid data element.\n", 0, true);
				return "ERROR";
			}
			catch (NullPointerException npe) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: Invalid (NULL) file.\n", 0, true);
				return "ERROR";
			}
			catch (Exception e) {
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
				return "ERROR";
			}
		}

		if (do_reset_controller && !abort) {
			try {
				PrintMessage.message("SetupCommand", 0, "Resetting the controller...waiting.\n", 0, true);
				dsp_cmd.hcvr(VoodooConstants.RESET_CONTROLLER, VoodooConstants.SYR);
				did_timing = false;
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.SYR) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				return "ERROR";
			}
			catch (Exception e) {
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
				return "ERROR";
			}
		}
				
		if (do_hardware_test && !abort) {
			int data = 0;
			int data_increment = 0;
			
			if (do_pci_hardware_test && !abort) {
				PrintMessage.message("SetupCommand", 0, "Performing PCI hardware test...waiting.\n", 0, true);
				data_increment = VoodooConstants.HARDWARE_DATA_MAX/number_of_pci_tests;

				// Set the board destination.	
				try {
					dsp_cmd.set_board_destination(VoodooConstants.TDL_ARG_NUM, VoodooConstants.PCI_ID);
				}
				catch (ReplyValueException rve) {
					PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
					PrintMessage.message("SetupCommand", 1, "ERROR: Board destination not set.\n", 0, true);
					PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
					if (rve.get_value() == VoodooConstants.TOUT)
						PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
					else
						PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				}
				catch(Exception e) {
					PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
					PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				}					

				// Do the test data links.
				for (int n=0; n<number_of_pci_tests; n++) {
					try {
						if (abort) { return "Done"; }
						dsp_cmd.test_data_link(data);
					}
					catch (ReplyValueException rve) {
						if (abort) { return "Done"; }
						PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
						PrintMessage.message("SetupCommand", 1, "     ERROR: Test " + Integer.toString(n) + "/" + Integer.toString(number_of_pci_tests) + "\n", 0, true);
						PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(data) + "\n", 0, true);
						if (rve.get_value() == VoodooConstants.TOUT)
							PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
						else
							PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
					}
					catch (Exception e) {
						if (abort) { return "Done"; }
						PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
						PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					}
					data += data_increment;
				}
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}

			if (do_timing_hardware_test && !abort) {
				PrintMessage.message("SetupCommand", 0, "Performing timing hardware test...waiting.\n", 0, true);
				data_increment = VoodooConstants.HARDWARE_DATA_MAX/number_of_timing_tests;

				// Set the board destination.	
				try {
					dsp_cmd.set_board_destination(VoodooConstants.TDL_ARG_NUM, VoodooConstants.TIM_ID);
				}
				catch (ReplyValueException rve) {
					PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
					PrintMessage.message("SetupCommand", 1, "ERROR: Board destination not set.\n", 0, true);
					PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
					if (rve.get_value() == VoodooConstants.TOUT)
						PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
					else
						PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				}
				catch(Exception e) {
					PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
					PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				}					

				// Do the test data links.
				for (int n=0; n<number_of_timing_tests; n++) {
					try {
						if (abort) { return "Done"; }
						dsp_cmd.test_data_link(data);
					}
					catch (ReplyValueException rve) {
						if (abort) { return "Done"; }
						PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
						PrintMessage.message("SetupCommand", 1, "     ERROR: Test " + Integer.toString(n) + "/" + Integer.toString(number_of_timing_tests) + "\n", 0, true);
						PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(data) + "\n", 0, true);
						if (rve.get_value() == VoodooConstants.TOUT)
							PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
						else
							PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
					}
					catch (Exception e) {
						if (abort) { return "Done"; }
						PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
						PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					}
					data += data_increment;
				}
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}

			if (do_utility_hardware_test && !abort) {
				PrintMessage.message("SetupCommand", 0, "Performing utility hardware test...waiting.\n", 0, true);
				data_increment = VoodooConstants.HARDWARE_DATA_MAX/number_of_utility_tests;

				// Set the board destination.	
				try {
					dsp_cmd.set_board_destination(VoodooConstants.TDL_ARG_NUM, VoodooConstants.UTIL_ID);
				}
				catch (ReplyValueException rve) {
					PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
					PrintMessage.message("SetupCommand", 1, "ERROR: Board destination not set.\n", 0, true);
					PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
					if (rve.get_value() == VoodooConstants.TOUT)
						PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
					else
						PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				}
				catch(Exception e) {
					PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
					PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				}					

				// Do the test data links.
				for (int n=0; n<number_of_utility_tests; n++) {
					try {
						if (abort) { return "Done"; }
						dsp_cmd.test_data_link(data);
					}
					catch (ReplyValueException rve)	{
						if (abort) { return "Done"; }
						PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
						PrintMessage.message("SetupCommand", 1, "     ERROR: Test " + Integer.toString(n) + "/" + Integer.toString(number_of_utility_tests) + "\n", 0, true);
						PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(data) + "\n", 0, true);
						if (rve.get_value() == VoodooConstants.TOUT)
							PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
						else
							PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
					}
					catch (Exception e) {
						if (abort) { return "Done"; }
						PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
						PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
					}
					data += data_increment; 
				}
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}		
		}

		if (do_timing && do_timing_file && !abort) {
			try{
				// Load the specified DSP timing file.
				PrintMessage.message("SetupCommand", 0, "Loading Timing file...waiting.\n", 0, true);
				dsp_file.load_file(timing_filename, "timing");
				did_timing = true;
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				if (abort) { return "Done"; }
				did_timing = false;
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				return "ERROR";
			}
			catch (IOException ioe) {
				if (abort) { return "Done"; }
				did_timing = false;
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: Cannot load file.\n", 0, true);
				return "ERROR";
			}
			catch (NoSuchElementException nsee) {
				if (abort) { return "Done"; }
				did_timing = false;
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: Invalid data element.\n", 0, true);
				return "ERROR";
			}
		 	catch (Exception e) {
				if (abort) { return "Done"; }
				did_timing = false;
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: Invalid type conversion.\n", 0, true);
				return "ERROR";
			}
		}
		
		if (do_timing && do_timing_app && !abort) {
			try {
				dsp_cmd.set_board_destination(VoodooConstants.TIM_APP_ARG_NUM, VoodooConstants.TIM_ID);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("SetupCommand", 1, "ERROR: Board destination not set.\n", 0, true);
			}
			catch(Exception e) {
				PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
			}
				
			if (abort) { return "Done"; }
				
			try {
				PrintMessage.message("SetupCommand", 0, "Loading Timing application...waiting.\n", 0, true);
				dsp_cmd.load_application(tim_app_number);
				did_timing = true;
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch(ReplyValueException rve) {
				if (abort) { return "Done"; }
				did_timing = false;
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				return "ERROR";
			}
			catch (Exception e) {
				if (abort) { return "Done"; }
				did_timing = false;
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				return "ERROR";
			}
		}

		if (do_utility && do_utility_file && !abort) {
			try{
				// Load the specified DSP utility file.
				PrintMessage.message("SetupCommand", 0, "Loading Utility file...waiting.\n", 0, true);
				dsp_file.load_file(utility_filename, "utility");
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch (ReplyValueException rve) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				return "ERROR";
			}
			catch (IOException ioe) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: Cannot load file.\n", 0, true);
				return "ERROR";
			}
			catch (NoSuchElementException nsee) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "ERROR: Invalid data element.\n", 0, true);
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				return "ERROR";
			}
		 	catch (Exception e) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: Invalid type conversion.\n", 0, true);
				return "ERROR";
			}
		}

		if (do_utility && do_utility_app && !abort) {
			try {
				dsp_cmd.set_board_destination(VoodooConstants.UTIL_APP_ARG_NUM, VoodooConstants.UTIL_ID);
			}
			catch (ReplyValueException rve) {
				PrintMessage.message("SetupCommand", 1, "ERROR: Board destination not set.\n", 0, true);
			}
			catch(Exception e) {
				PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
			}
				
			if (abort) { return "Done"; }

			try {
				PrintMessage.message("SetupCommand", 0, "Loading utility application...waiting.\n", 0, true);
				dsp_cmd.load_application(util_app_number);
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch(ReplyValueException rve) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				return "ERROR";
			}
			catch (Exception e) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				return "ERROR";
			}
		}
		
		if (do_power_on && !abort) {
			try {
				PrintMessage.message("SetupCommand", 0, "Doing power on...waiting.\n", 0, true);
				dsp_cmd.hcvr(VoodooConstants.POWER_ON, VoodooConstants.DON);
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch(ReplyValueException rve) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "     Expected value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("SetupCommand", 1, "     Actual value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				return "ERROR";
			}
			catch (Exception e) {
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
			}
		}

		if (do_dimensions && !abort) {
			try {
				PrintMessage.message("SetupCommand", 0, "Setting image dimensions...waiting.\n", 0, true);
				dsp_cmd.dimensions(array_rows, array_columns*(bits_per_pixel/16));	// This is how the DSP handles different bits per pixel.
				setup_applied = true;
				PrintMessage.message("SetupCommand", 0, "done.", 8, false);
			}
			catch(ReplyValueException rve) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "     Expected image columns value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("SetupCommand", 1, "     Actual image columns value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);

				PrintMessage.message("SetupCommand", 1, "     Expected image rows value = 0x" + Integer.toHexString(VoodooConstants.DON) + "\n", 0, true);
				if (rve.get_value() == VoodooConstants.TOUT)
					PrintMessage.message("SetupCommand", 1, "     Actual value = TIMEOUT\n", 0, true);
				else
					PrintMessage.message("SetupCommand", 1, "     Actual image rows value = 0x" + Integer.toHexString(rve.get_value()) + "\n", 0, true);
				return "ERROR";
			}
			catch (Exception e) {
				if (abort) { return "Done"; }
				PrintMessage.message("SetupCommand", 1, "failed.", 8, false);
				PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + ".\n", 0, true);
				return "ERROR";
			}
		}

		// Setup the controller configuration.
		if (do_timing || did_timing) {
			parent.view_config_options_button.setEnabled(true);
			read_controller_config();
		}
		
		return "Done";
	}

	/**
	********************************************************************************
	*	This method reads and sets the controller configuration. The camera type
	*	(IR or CCD) is determined here along with controller settings, such as
	*	binning, subimage readout, shutter exists, etc.
	*	
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	private void read_controller_config()
	{
		// Read the controller configuration.
		try {
			config_word = 0;
			config_word = dsp_cmd.hcvr(VoodooConstants.READ_CONTROLLER_CONFIG);
		}
		catch (ReplyValueException rve) {
			PrintMessage.message("SetupCommand", 1, "ERROR: Could not read controller status.\n", 0, true);
		}
		catch (Exception e) {
			PrintMessage.message("SetupCommand", 1, "ERROR: " + e.getMessage() + "\n", 0, true);
		}

		// Check for ERR and return if found.
		if (config_word == VoodooConstants.ERR) {
			parent.controller_config_dialog.set_config_data(VoodooConstants.DEFAULT_CONFIG_WORD);
			PrintMessage.message("SetupCommand", 1, "ERROR: Controller status returned \"ERR\" \n", 0, true);
			return;
		}

		// Check for an empty buffer and return if found.
		if (config_word == VoodooConstants.REPLY_BUFFER_EMPTY) {
			PrintMessage.message("SetupCommand", 1, "ERROR: Controller status returned \"EMPTY\" \n", 0, true);
			return;
		}
		
		// Setup the controller configuration window.
		parent.controller_config_dialog.set_config_data(config_word);
	}
}
