/**
********************************************************************************
*	A class to display a window from which the user may perform array setup
*	tasks (load DSP files/applications, set array dimensions, etc.).
*
*	CVM - View
*
*	@see JDialog
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import javax.swing.border.*;

class SetupDialog extends JDialog
{
	protected Container content_pane;
	protected HardwareTestDialog hardware_test_dialog = null;
	protected ControllerConfigDialog controller_config_dialog = null;
	protected VoodooMainWindow parent;
	protected JCheckBox reset_controller_chkbox;
	protected JCheckBox hardware_test_chkbox;
	protected JCheckBox pci_download_chkbox;
	protected JCheckBox timing_board_chkbox;
	protected JCheckBox utility_board_chkbox;
	protected JCheckBox power_on_chkbox;
	protected JCheckBox dimensions_chkbox;
	protected JRadioButton timing_file_rbutton;
	protected JRadioButton timing_app_rbutton;
	protected JRadioButton utility_file_rbutton;
      	protected JRadioButton utility_app_rbutton;
      	protected JRadioButton image_res_16bit_rbutton;
	protected JRadioButton image_res_32bit_rbutton;
	protected VTextField pci_file_txtfld;
	protected VTextField timing_file_txtfld;
	protected VTextField utility_file_txtfld;
	protected VTextField rows_txtfld;
	protected VTextField cols_txtfld;
	protected VTextField timing_app_txtfld;
	protected VTextField utility_app_txtfld;
	protected JButton get_pci_file_button;
	protected JButton get_timing_file_button;
	protected JButton get_utility_file_button;
	protected JButton apply_button;
	protected JButton view_config_options_button;
	protected int number_of_setup_windows;
	private JPanel panel_holder;
	private JPanel new_setup_panel;
	private JPanel new_button_panel;
	private JMenuBar menu_bar;
	protected SetupListener setup_listener;
	private Icon browse_icon;
	private Font font;
		
	/**
	********************************************************************************
	*	Class constructor. The window is created in this method.
	*
	*	@param owner The window from which this dialog window was opened.
	*
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	public SetupDialog(Frame owner)
	{
		// Call the superclass (JDialog) constructor.
		super((Frame)null, "Controller Setup", false);
		
		// Save the parent.
		this.parent = (VoodooMainWindow)owner;
		
		// Set the dialog box size.
		setSize(400, 440);
		
		// Make the window non-resizable.
      		setResizable(false);
		
		// Get a handle on the dialog box's content pane
		// and set it's layout.
		content_pane = this.getContentPane();
		content_pane.setLayout(new BorderLayout());

		// Set the components font.
		font = new Font("LucidaSansRegular", Font.PLAIN, 11);

		browse_icon = new ImageIcon(VoodooConstants.PIC_DIR + "folder.gif");

		// Create the event lister.
		setup_listener = new SetupListener(this);

		// Create all the child windows.
		hardware_test_dialog = new HardwareTestDialog(this);
		controller_config_dialog = new ControllerConfigDialog(this);
		
		// Create and add the menu/toolbar panel.
     		menu_bar = create_menu_bar();
     		setJMenuBar(menu_bar);
     		//content_pane.add(menu_bar, BorderLayout.NORTH);
		
		// Create the panel holder and set it's layout.
		panel_holder = new JPanel();
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		panel_holder.setLayout(gbl);
		
		// Create the setup panel and add it to the panel holder.
		new_setup_panel = create_setup_panel();
      		add_components(panel_holder, new_setup_panel, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 0, 1, 1);
				
		// Create the button panel and add it to the panel holder.
		new_button_panel = create_button_panel();
      		add_components(panel_holder, new_button_panel, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 1, 1, 1);
		
		// Add the panel holder to the dialog box.
		content_pane.add(panel_holder, BorderLayout.CENTER);

      		// Add a window listener.  This has to be here to allow the
      		// dialog box to be re-opened later when the "close" window
      		// menu item (from system, not java) is chosen.
      		addWindowListener(new WindowAdapter()
      		{
      			public void windowClosing(WindowEvent e)
      			{
      				number_of_setup_windows = 0;
			}
		});
				
		// Initialize any variables.
		number_of_setup_windows = 0;
	}
	
	/**
	********************************************************************************
	*	This method creates the panel that contains all the user input components.
	*
	*	@return Returns the panel that contains all the user input components.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	private JPanel create_setup_panel()
	{
		JPanel setup_panel = new JPanel();
		
		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		setup_panel.setLayout(gbl);
      		
		// Create the checkbox components.
		reset_controller_chkbox = new JCheckBox("Reset Controller");
		hardware_test_chkbox = new JCheckBox("Hardware Test");
		pci_download_chkbox = new JCheckBox("PCI Download:");
		timing_board_chkbox = new JCheckBox("Timing Board:");
		utility_board_chkbox = new JCheckBox("Utility Board:");
		power_on_chkbox = new JCheckBox("Power On");
		dimensions_chkbox = new JCheckBox("Image Dimensions:");

		// Create the button components.
		get_pci_file_button = new JButton(browse_icon);
		get_timing_file_button = new JButton(browse_icon);
		get_utility_file_button = new JButton(browse_icon);
		view_config_options_button = new JButton("Controller Configuration");
		
		// Set the button tooltip text.
		get_pci_file_button.setToolTipText("Browse For File");
		get_timing_file_button.setToolTipText("Browse For File");
		get_utility_file_button.setToolTipText("Browse For File");
		
		// Set the "Browse" buttons action command string.
      		get_pci_file_button.setActionCommand("PCI Browse");
      		get_timing_file_button.setActionCommand("TIM Browse");
      		get_utility_file_button.setActionCommand("UTIL Browse");

		// Set the button sizes.
		get_pci_file_button.setPreferredSize(new Dimension(23, 22));
		get_timing_file_button.setPreferredSize(new Dimension(23, 22));
		get_utility_file_button.setPreferredSize(new Dimension(23, 22));
		view_config_options_button.setPreferredSize(new Dimension(200, 26));

		// Start with the configuration button disabled.
		view_config_options_button.setEnabled(false);
		 
		// Create the hardware test options button and set its size.
		JButton hardware_options_button = new JButton("Options");
		hardware_options_button.setPreferredSize(new Dimension(90, 26));
		
		// Create the timing app. radio button components.
      		timing_file_rbutton = new JRadioButton("File", true);
      		timing_app_rbutton = new JRadioButton("Application", false);

      		// Group the timing app. radio buttons together so only one at a time may be selected.
      		ButtonGroup setup_timing_app_rbutton_group = new ButtonGroup();
      		setup_timing_app_rbutton_group.add(timing_file_rbutton);
      		setup_timing_app_rbutton_group.add(timing_app_rbutton);
      		
      		// Set the timing radiobuttons action command string.
      		timing_file_rbutton.setActionCommand("TIM File");
      		timing_app_rbutton.setActionCommand("TIM App");

		// Create the utility app. radio button components.
      		utility_file_rbutton = new JRadioButton("File", true);
      		utility_app_rbutton = new JRadioButton("Application", false);

      		// Group the utility app. radio buttons together so only one at a time may be selected.
      		ButtonGroup setup_utility_app_rbutton_group = new ButtonGroup();
      		setup_utility_app_rbutton_group.add(utility_file_rbutton);
      		setup_utility_app_rbutton_group.add(utility_app_rbutton);

		// Set the utility radiobutton action command strings.
      		utility_file_rbutton.setActionCommand("UTIL File");
      		utility_app_rbutton.setActionCommand("UTIL App");
      		
		// Create the image resolution radio button components.
      		image_res_16bit_rbutton = new JRadioButton("16 Bits/Pixel", true);
      		image_res_32bit_rbutton = new JRadioButton("32 Bits/Pixel", false);

		// Set the image resolution radiobutton action command strings.
      		image_res_16bit_rbutton.setActionCommand("16 Bits/Pixel");
      		image_res_32bit_rbutton.setActionCommand("32 Bits/Pixel");
      		
      		// Group the image resolution radio buttons together so only one at
      		// a time may be selected.
      		ButtonGroup setup_image_res_rbutton_group = new ButtonGroup();
      		setup_image_res_rbutton_group.add(image_res_16bit_rbutton);
      		setup_image_res_rbutton_group.add(image_res_32bit_rbutton);

		// Create the textfield label components.
		JLabel timing_file_label = new JLabel("Filename: ");
		JLabel utility_file_label = new JLabel("Filename: ");
		JLabel rows_label = new JLabel("Rows:");
		JLabel cols_label = new JLabel("Cols:");
		JLabel timing_app_label = new JLabel("Application: ");
		JLabel utility_app_label = new JLabel("Application: ");
		JLabel image_res_label = new JLabel("Image Resolution:");

		// Set the label colors.				
		timing_file_label.setForeground(Color.black);
		utility_file_label.setForeground(Color.black);
		rows_label.setForeground(Color.black);
		cols_label.setForeground(Color.black);
		timing_app_label.setForeground(Color.black);
		utility_app_label.setForeground(Color.black);
		image_res_label.setForeground(Color.black);
		
		// Create the textfield components.
		pci_file_txtfld = new VTextField("", 18);
		timing_file_txtfld = new VTextField("", 18);
		utility_file_txtfld = new VTextField("", 18);
		rows_txtfld = new VTextField("", 5);
		cols_txtfld = new VTextField("", 5);
		timing_app_txtfld = new VTextField("", 2);
		utility_app_txtfld = new VTextField("", 2);

		// Set the textfield names that will be used by the event listener.
		pci_file_txtfld.setName("PCI Textfield");
		timing_file_txtfld.setName("Timing Textfield");
		utility_file_txtfld.setName("Utility Textfield");
		rows_txtfld.setName("Array Rows Textfield");
		cols_txtfld.setName("Array Columns Textfield");
		timing_app_txtfld.setName("Timing App Textfield");
		utility_app_txtfld.setName("Utility App Textfield");
		
		// Set the timing and utility application input textfields to uneditable,
		// since the file download is the default option.
		timing_app_txtfld.setEditable(false);
		utility_app_txtfld.setEditable(false);

		// Add the setup components to the panel.
      		add_components(setup_panel, pci_download_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 0, 1, 1);
      		add_components(setup_panel, pci_file_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 0, 4, 1);
      		add_components(setup_panel, get_pci_file_button, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 0, 2, 1);
      		add_components(setup_panel, reset_controller_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 1, 2, 1);
      		add_components(setup_panel, hardware_test_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 2, 1, 1);
      		add_components(setup_panel, hardware_options_button, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 2, 2, 1);
      		add_components(setup_panel, timing_board_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 6, 1, 1);
      		add_components(setup_panel, timing_file_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1, 6, 1, 1);
      		add_components(setup_panel, timing_app_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 2, 6, 2, 1);
      		add_components(setup_panel, timing_file_label, gbl, gbc, 0, 60, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 7, 1, 1);
      		add_components(setup_panel, timing_file_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 7, 4, 1);
      		add_components(setup_panel, get_timing_file_button, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 7, 2, 1);
      		add_components(setup_panel, timing_app_label, gbl, gbc, 0, 49, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 8, 1, 1);
      		add_components(setup_panel, timing_app_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1, 8, 1, 1);
     		add_components(setup_panel, utility_board_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 9, 1, 1);
      		add_components(setup_panel, utility_file_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1, 9, 1, 1);
      		add_components(setup_panel, utility_app_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 2, 9, 2, 1);
      		add_components(setup_panel, utility_file_label, gbl, gbc, 0, 60, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 10, 1, 1);
      		add_components(setup_panel, utility_file_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 10, 4, 1);
      		add_components(setup_panel, get_utility_file_button, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 10, 2, 1);
      		add_components(setup_panel, utility_app_label, gbl, gbc, 0, 49, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 11, 1, 1);
      		add_components(setup_panel, utility_app_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1, 11, 1, 1);
      		add_components(setup_panel, power_on_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 12, 2, 1);
      		add_components(setup_panel, dimensions_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 17, 2, 1);
      		add_components(setup_panel, cols_label, gbl, gbc, 0, 85, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 18, 1, 1);
      		add_components(setup_panel, cols_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 18, 2, 1);
       		add_components(setup_panel, rows_label, gbl, gbc, 0, 85, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 19, 1, 1);
      		add_components(setup_panel, rows_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 19, 2, 1);
      		add_components(setup_panel, image_res_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 25, 1, 1);
      		add_components(setup_panel, image_res_16bit_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1, 25, 2, 1);
      		add_components(setup_panel, image_res_32bit_rbutton, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 3, 25, 2, 1);
      		add_components(setup_panel, view_config_options_button, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 27, 5, 1);

		// Add the event listener to the checkboxes.
		reset_controller_chkbox.addItemListener(setup_listener);
		hardware_test_chkbox.addItemListener(setup_listener);
		pci_download_chkbox.addItemListener(setup_listener);
		timing_board_chkbox.addItemListener(setup_listener);
		utility_board_chkbox.addItemListener(setup_listener);
		power_on_chkbox.addItemListener(setup_listener);
		dimensions_chkbox.addItemListener(setup_listener);
		
		// Add the event listener to the radio buttons.
		timing_file_rbutton.addActionListener(setup_listener);
		timing_app_rbutton.addActionListener(setup_listener);
		utility_file_rbutton.addActionListener(setup_listener);
		utility_app_rbutton.addActionListener(setup_listener);
		image_res_16bit_rbutton.addActionListener(setup_listener);
		image_res_32bit_rbutton.addActionListener(setup_listener);
					
		// Add the event listener to the buttons.
		get_pci_file_button.addActionListener(setup_listener);
		get_timing_file_button.addActionListener(setup_listener);
		get_utility_file_button.addActionListener(setup_listener);
		hardware_options_button.addActionListener(setup_listener);
		view_config_options_button.addActionListener(setup_listener);

		return setup_panel;
	}

	/**
	********************************************************************************
	*	This method creates the button control panel.
	*
	*	@return Returns the button control panel.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	private JPanel create_button_panel()
	{
		JPanel button_panel = new JPanel();

		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		button_panel.setLayout(gbl);
		
		// Create the buttons.
		apply_button = new JButton("Apply");
		JButton clear_all_button = new JButton("Clear");
		JButton load_setup_button = new JButton("Load");
		JButton save_setup_button = new JButton("Save");

		// Add the buttons to the panel.
      		add_components(button_panel, apply_button, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 0, 1, 1);
  		add_components(button_panel, clear_all_button, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 1, 0, 1, 1);
      		add_components(button_panel, load_setup_button, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 2, 0, 1, 1);
      		add_components(button_panel, save_setup_button, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 3, 0, 1, 1);

		// Make all the buttons the same size.
		apply_button.setPreferredSize(new Dimension(90, 26));
		clear_all_button.setPreferredSize(new Dimension(90, 26));
		load_setup_button.setPreferredSize(new Dimension(90, 26));
		save_setup_button.setPreferredSize(new Dimension(90, 26));

		// Add the action listener to the buttons.
		apply_button.addActionListener(setup_listener);
		save_setup_button.addActionListener(setup_listener);
		load_setup_button.addActionListener(setup_listener);
		clear_all_button.addActionListener(setup_listener);

		return button_panel;
	}
	
	/**
	********************************************************************************
	*	This method creates the menu bar.
	*
	*	@return Returns the menubar.
	*
	*	@see JMenuBar
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
	private JMenuBar create_menu_bar()
	{
		JMenuBar menu_bar = new JMenuBar();

		// Set the menubar's border.
		menu_bar.setBorder(new BevelBorder(BevelBorder.RAISED));
		menu_bar.setBorderPainted(true);
		
		// Add the Window menu to the menu bar.
		JMenu window_menu = new JMenu("Window");
		menu_bar.add(window_menu);
		
		// Add the submenus to the Window menu.
		JMenuItem close_item = new JMenuItem("Close", 'C');
		window_menu.add(close_item);

		// Set the menubar font.
		window_menu.setFont(font);
		close_item.setFont(font);
		
		// Add the event listeners to the menu items.
		close_item.addActionListener(setup_listener);
		
		return menu_bar;
	}

	/**
	*****************************************************************************
	*	Adds the specified component to the specified panel.
	*
	*	@param panel The panel to which the component will be added.
	*	@param compnt The component to add.
	*	@param gbl The GridBagLayout of the panel.
	*	@param gbl The GridBagConstraints of the panel.
	*	@param insetTOP The distance in pixels between the top of this
	*			component and the one above it.
	*	@param insetBOTTOM The distance in pixels between the bottom of
	*			this component and the one below it.
	*	@param insetLEFT The distance in pixels between the left side of
	*			this component and the one left of it.
	*	@param insetRIGHT The distance in pixels between the right side
	*			of this component and the one right of it.
	*	@param cfill Determines how the component "fills" the space of
	*		  	of its "cell".
	*	@param anchr The justification of the component within its "cell".
	*	@param col The x position of the component.
	*	@param row The y position of the component.
	*	@param gw The number of "cells" that the component spans in the
	*			width direction.
	*	@param gh The number of "cells" that the component spans in the
	*			height direction.
	*
	*	@see JPanel
	*	@see JComponent
	*	@see GridBagLayout
	*	@see GridBagConstraints
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
   	private void add_components(JPanel panel, JComponent compnt, GridBagLayout gbl, GridBagConstraints gbc, int insetTOP, int insetLEFT, int insetBOTTOM, int insetRIGHT, int cfill, int anchr, int col, int row, int gw, int gh)
   	{
      		// Set the grid layout constraints.
      		gbc.fill = cfill;
      		gbc.anchor = anchr;
      		gbc.insets = new Insets(insetTOP, insetLEFT, insetBOTTOM, insetRIGHT);
      		gbc.gridx = col;
      		gbc.gridy = row;
      		gbc.gridwidth = gw;
      		gbc.gridheight = gh;
      		
      		// Set the compontent font.
      		compnt.setFont(font);
      
      		// Add the component to the panel.
      		gbl.setConstraints(compnt, gbc);
      		panel.add(compnt);
      	}

	/**
	********************************************************************************
	*	This method performs a software "click" of the apply button. It does the
	*	same thing as if the button were "clicked" by a user with a mouse.
	*
	*	@see JButton
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
      	public void clickApply()
      	{
      		apply_button.doClick();
      	}
}
