/**
********************************************************************************
*	This class is the event listener for the SetupDialog window. All actions
*	handled by this class are passed to the SetupCommand class.
*
*	CVM - Controller
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import java.io.*;

public class SetupListener implements ActionListener, ItemListener
{
	protected SetupCommand setup_cmd = null;
	protected SetupDialog parent = null;
	protected SetupSave setup_save = null;
	private JFileChooser file_chooser = null;
	private File file;
	private String message;
	private int rows;
	private int cols;
	
	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@param parent The dialog window that this class listens for.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public SetupListener(SetupDialog parent)
	{
		this.parent = parent;
		setup_cmd = new SetupCommand(parent);
		setup_save = new SetupSave();
		file = new File(".");
	}

	/**
	********************************************************************************
	*	This method handles all the checkbox selection events.
	*
	*	@param ie The item event.
	*
	*	@see ItemListener
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void itemStateChanged(ItemEvent ie)
	{
		AbstractButton active_button = (AbstractButton) ie.getSource();

		// Reset the controller.
		if (active_button.getText().equals("Reset Controller")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_reset_controller(true);
			else
				setup_cmd.set_reset_controller(false);
		}
		
		// Power on the controller.
		else if (active_button.getText().equals("Power On")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_power_on(true);
			else
				setup_cmd.set_power_on(false);
		}
		
		// Run the hardware tests.
		else if (active_button.getText().equals("Hardware Test")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_hardware_test(true);
			else
				setup_cmd.set_hardware_test(false);
		}
		
		// Load the pci boot file into the DSP.
		else if (active_button.getText().equals("PCI Download:")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_pci(true);
			else
				setup_cmd.set_pci(false);
		}
		
		// Load the timing board file/app into the DSP.
		else if (active_button.getText().equals("Timing Board:")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_timing(true);
			else
				setup_cmd.set_timing(false);
		}

		// Load the utility board file/app into the DSP.
		else if (active_button.getText().equals("Utility Board:")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_utility(true);
			else
				setup_cmd.set_utility(false);
		}

		// Set the CCD dimensions.
		else if (active_button.getText().equals("Image Dimensions:")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_dimensions(true);
			else
				setup_cmd.set_dimensions(false);
		}
		
		// Do the pci hardware test.
		else if (active_button.getActionCommand().equals("PCI Board Hardware Test")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_pci_test(true);
			else
				setup_cmd.set_pci_test(false);
		}		

		// Do the timing hardware test.
		else if (active_button.getActionCommand().equals("Timing Board Hardware Test")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_timing_test(true);
			else
				setup_cmd.set_timing_test(false);
		}

		// Do the utility hardware test.
		else if (active_button.getActionCommand().equals("Utility Board Hardware Test")) {
			if (ie.getStateChange() == ItemEvent.SELECTED)
				setup_cmd.set_utility_test(true);
			else
				setup_cmd.set_utility_test(false);
		}
	}


	/**
	********************************************************************************
	*	This method handles all the non-selection events.
	*
	*	@param e The event.
	*
	*	@see ActionListener
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void actionPerformed(ActionEvent e)
	{
		String active_button = (String) e.getActionCommand();

		// Exit.
		if (active_button.equals("Close")) {
			parent.setVisible(false);
			parent.number_of_setup_windows = 0;
		}
		
		// Apply the current settings to the hardware.
		else if (active_button.equals("Apply")) {
			// Set the apply button text to "Abort".
			parent.apply_button.setText("Abort");

			// Re-update the shutter position to opposite the current
			// value. This will cause the shutter position to be
			// automatically updated on the next exposure.
			parent.parent.exposure_control_listener.exp_cmd.current_shutter_position = false;
			if (parent.parent.open_shutter_chkbox.isSelected())
				parent.parent.exposure_control_listener.exp_cmd.set_open_shutter(true);
			else
				parent.parent.exposure_control_listener.exp_cmd.set_open_shutter(false);

			// Check the options for a synthetic image
			// and reset the controller bit if true.
			if (OptionsCommand.get_image_check_mode())
				OptionsCommand.set_image_check_mode(true);

			// Set and check all textfields.
			try {
				setup_cmd.set_pci_filename(parent.pci_file_txtfld.getText());
				setup_cmd.set_timing_filename(parent.timing_file_txtfld.getText());
				setup_cmd.set_utility_filename(parent.utility_file_txtfld.getText());
				setup_cmd.set_timing_application(parent.timing_app_txtfld.getText());
				setup_cmd.set_utility_application(parent.utility_app_txtfld.getText());
				setup_cmd.set_hardware_test(VoodooConstants.PCI_ID, parent.hardware_test_dialog.get_pci_test_number());
				setup_cmd.set_hardware_test(VoodooConstants.TIM_ID, parent.hardware_test_dialog.get_timing_test_number());
				setup_cmd.set_hardware_test(VoodooConstants.UTIL_ID, parent.hardware_test_dialog.get_utility_test_number());
				setup_cmd.set_array_cols(parent.cols_txtfld.getText());
				setup_cmd.set_array_rows(parent.rows_txtfld.getText());
			}
			catch (NumberFormatException nfe) {
				PrintMessage.message("SetupListener", 1, nfe.getMessage() + ".\n", 0, true);
				parent.apply_button.setText("Apply");
				return;
			}
			catch (Exception exp) {
				PrintMessage.message("SetupListener", 1, exp.getMessage() + ".\n", 0, true);
				parent.apply_button.setText("Apply");
				return;
			}

			// Apply the current controller settings.
			setup_cmd.thread_it();
		}
		
		// Abort the current attempt to load the hardware settings.
		else if (active_button.equals("Abort")) {
			setup_cmd.stop();
			setup_cmd.interrupt();
			System.gc();

			try {
				DspCommand2.flush_reply_buffer();
			}
			catch (Exception ex) {}
						
			// Set the apply button text to "Apply".
			parent.apply_button.setText("Apply");
		}
		
		// Save the current CCD setup.
		else if (active_button.equals("Save")) {
			try {
				setup_save.write_to_file(parent.content_pane, parent, parent.hardware_test_dialog);
			}
			catch (IOException ioe) {
				PrintMessage.message("SetupListener", 1, "ERROR: File could not be opened.\n", 0, true);
			}
		}
		
		// Load a saved CCD setup.
		else if (active_button.equals("Load")) {
			try {
				// Read the setup from disk.
				setup_save.read_from_file(parent.content_pane, parent, parent.hardware_test_dialog);
			}
			catch (IOException ioe) {
				PrintMessage.message("SetupListener", 1, "ERROR: File could not be opened.", 0, true);
			}
		}
	
		// Clear all the current settings. Note: The variables in the SetupCommand
		// class are automatically updated by the setSelected() function.
		else if (active_button.equals("Clear")) {
			// Clear all the checkboxes.
			parent.reset_controller_chkbox.setSelected(false);
			parent.hardware_test_chkbox.setSelected(false);
			parent.pci_download_chkbox.setSelected(false);
			parent.timing_board_chkbox.setSelected(false);
			parent.utility_board_chkbox.setSelected(false);
			parent.power_on_chkbox.setSelected(false);
			parent.dimensions_chkbox.setSelected(false);
			
			// Clear the checkbox values in the command class.
			setup_cmd.set_reset_controller(false);
			setup_cmd.set_hardware_test(false);
			setup_cmd.set_pci(false);
			setup_cmd.set_timing(false);
			setup_cmd.set_utility(false);
			setup_cmd.set_power_on(false);
			setup_cmd.set_dimensions(false);
		}

		// Set the pci hardware to load a File.
		else if (active_button.equals("Options")) {
			parent.hardware_test_dialog.number_of_test_windows++;
			if (parent.hardware_test_dialog.number_of_test_windows == 1)
				parent.hardware_test_dialog.setVisible(true);
		}

		// Set the timing hardware to load a File.
		else if (active_button.equals("TIM File")) {
			parent.timing_file_txtfld.setEditable(true);
			parent.timing_app_txtfld.setEditable(false);
			parent.get_timing_file_button.setEnabled(true);
			setup_cmd.set_timing_file(true);
			setup_cmd.set_timing_app(false);
		}
		
		// Set the timing hardware to load an Application.
		else if (active_button.equals("TIM App")) {
			parent.timing_app_txtfld.setEditable(true);
			parent.timing_file_txtfld.setEditable(false);
			parent.get_timing_file_button.setEnabled(false);
			setup_cmd.set_timing_app(true);
			setup_cmd.set_timing_file(false);
		}
		
		// Set the utility hardware to load a File.
		else if (active_button.equals("UTIL File")) {
			parent.utility_file_txtfld.setEditable(true);
			parent.utility_app_txtfld.setEditable(false);
			parent.get_utility_file_button.setEnabled(true);
			setup_cmd.set_utility_file(true);
			setup_cmd.set_utility_app(false);
		}
		
		// Set the utility hardware to load a Application.
		else if (active_button.equals("UTIL App")) {
			parent.utility_app_txtfld.setEditable(true);
			parent.utility_file_txtfld.setEditable(false);
			parent.get_utility_file_button.setEnabled(false);
			setup_cmd.set_utility_app(true);
			setup_cmd.set_utility_file(false);
		}
		
		// Set the image resolution to 16 bits/pixel.
		else if (active_button.equals("16 Bits/Pixel")) {
			setup_cmd.set_image_resolution(16);
		}
		
		// Set the image resolution to 32 bits/pixel.
		else if (active_button.equals("32 Bits/Pixel"))
			setup_cmd.set_image_resolution(32);
				
		// Open a pci board DSP file.
		else if (active_button.equals("PCI Browse")) {
			// Open a file chooser that points to the current dir.
			file_chooser = new JFileChooser(file.getPath());
							
			// Add a file filter.
			file_chooser.addChoosableFileFilter(new LodFileFilter());
			
			// Assign the file filter and discard all others.
			file_chooser.setFileFilter(new LodFileFilter());
			
			// Set the Open dialog box size.
			file_chooser.setPreferredSize(new Dimension(500, 250));
			
			// Show the Open dialog box (returns the option selected)
			int selected = file_chooser.showOpenDialog(parent.content_pane);

			// If the Open button is pressed.
			if (selected == JFileChooser.APPROVE_OPTION) {
				// Get the selected file.
				file = file_chooser.getSelectedFile();
		
				// Display the file name in the frame.
				parent.pci_file_txtfld.setText(file.getPath());
				return;
			}
			// If the Cancel button is pressed.
			else if (selected == file_chooser.CANCEL_OPTION) {
				return;
			}
		}

		// Open a timing board DSP file.
		else if (active_button.equals("TIM Browse")) {
			// Open a file chooser that points to the current dir.
			file_chooser = new JFileChooser(file.getPath());
							
			// Add a file filter.
			file_chooser.addChoosableFileFilter(new LodFileFilter());
			
			// Assign the file filter and discard all others.
			file_chooser.setFileFilter(new LodFileFilter());
			
			// Set the Open dialog box size.
			file_chooser.setPreferredSize(new Dimension(500, 250));
			
			// Show the Open dialog box (returns the option selected)
			int selected = file_chooser.showOpenDialog(parent.content_pane);

			// If the Open button is pressed.
			if (selected == JFileChooser.APPROVE_OPTION) {
				// Get the selected file.
				file = file_chooser.getSelectedFile();
		
				// Display the file name in the frame.
				parent.timing_file_txtfld.setText(file.getPath());
				
				// If a new timing file has been loaded, remove the old controller
				// configuration window and replace it with a new one.
				Point location = parent.controller_config_dialog.getLocation();
				boolean show_it = false;
				
				if (parent.controller_config_dialog.isVisible())
					show_it = true;
				
				parent.controller_config_dialog.hide();
				parent.controller_config_dialog.dispose();
				parent.controller_config_dialog = null;
				parent.controller_config_dialog = new ControllerConfigDialog(parent);
				parent.controller_config_dialog.setLocation(location);
				
				if (show_it)
					parent.controller_config_dialog.setVisible(true);

				return;
			}
			// If the Cancel button is pressed.
			else if (selected == file_chooser.CANCEL_OPTION) {
				return;
			}
		}
		
		// Open a utility board DSP file.
		else if (active_button.equals("UTIL Browse")) {
			// Open a file chooser that points to the current dir.
			file_chooser = new JFileChooser(file.getPath());
							
			// Add a file filter.
			file_chooser.addChoosableFileFilter(new LodFileFilter());
			
			// Assign the file filter and discard all others.
			file_chooser.setFileFilter(new LodFileFilter());
			
			// Set the Open dialog box size.
			file_chooser.setPreferredSize(new Dimension(500, 250));
			
			// Show the Open dialog box (returns the option selected)
			int selected = file_chooser.showOpenDialog(parent.content_pane);
			
			// If the Open button is pressed.
			if (selected == JFileChooser.APPROVE_OPTION) {
				// Get the selected file.
				file = file_chooser.getSelectedFile();
		
				// Display the file name in the frame.
				parent.utility_file_txtfld.setText(file.getPath());
				return;
			}
			// If the Cancel button is pressed.
			else if (selected == file_chooser.CANCEL_OPTION) {
				return;
			}
		}

		// View the controller config options.
		else if (active_button.equals("Controller Configuration")) {
			parent.controller_config_dialog.setVisible(true);
		}

		// Close the Hardware test dialog box.
		else if (active_button.equals("Close Hardware Test")) {
			parent.hardware_test_dialog.number_of_test_windows = 0;
			parent.hardware_test_dialog.setVisible(false);
		}
	}
}