/**
********************************************************************************
*	A class to load or save the current controller properties as set in
*	the Controller Setup Window. The properties are loaded/saved to/from
*	a text file. This class is purely a user interface tool and
*	provides no hardware communication.
*
*	CVM - Model
*
*	@see SetupDialog
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import javax.swing.*;
import java.awt.*;
import java.io.*;

public class SetupSave
{
	File file;

	/**
	*****************************************************************************
	*	Default Constructor.
	*****************************************************************************
	*/
	public SetupSave()
	{
		file = new File(".");
	}

	/**
	*****************************************************************************
	*	Saves the current component settings in the Controller Setup Window
	*	to the specified text file. The file is specified via a popup
	*	JFileChooser dialog.
	*
	*	@param parent_component The container that holds the Controller Setup
	*		                Window components (i.e. content pane, panel, etc).
	*	@param setup_dialog The Controller Setup window.
	*	@param hardware_test_dialog The Hardware Test Options window.
	*
	*	@throws IOException
	*	@see SetupDialog
	*	@version 1.00
	*	@author Scott Streit
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	06/26/2000	sds	Changed file format from binary to text.
	*****************************************************************************
	*/
	public void write_to_file(Component parent_component, SetupDialog setup_dialog, HardwareTestDialog hardware_test_dialog) throws IOException
	{
		RandomAccessFile outFile;
		JFileChooser file_chooser;

		// Open a file chooser that points to the current dir.
		file_chooser = new JFileChooser(file.getPath());
		
		// Add a file filter.
		file_chooser.addChoosableFileFilter(new SetupFileFilter());
			
		// Assign the file filter and discard all others.
		file_chooser.setFileFilter(new SetupFileFilter());
			
		// Set the file chooser size.
		file_chooser.setPreferredSize(new Dimension(500, 250));

		// Show the Save dialog box (returns the option selected)
		int selected = file_chooser.showSaveDialog(parent_component);

		// If the Open button is pressed.
		if (selected == JFileChooser.APPROVE_OPTION) {
			// Get the selected file.
			file = file_chooser.getSelectedFile();
			
			outFile = new RandomAccessFile(file, "rw");
			
			// Set the file length to 1 byte. This is to ensure the new
			// data is properly written.
			outFile.setLength(1);

			// Set the informational message.
			outFile.writeBytes("**** NOTE: All labels and data values MUST be separated by a single tab. ****\n");
				
			// PCI download checkbox.
			outFile.writeBytes(setup_dialog.pci_download_chkbox.getText() + "\t" + String.valueOf(setup_dialog.pci_download_chkbox.isSelected()) + "\n");

			// Reset controller checkbox.
			outFile.writeBytes(setup_dialog.reset_controller_chkbox.getText() + ":" + "\t" + String.valueOf(setup_dialog.reset_controller_chkbox.isSelected()) + "\n");

			// Hardware test checkbox.
			outFile.writeBytes(setup_dialog.hardware_test_chkbox.getText() + ":" + "\t" + String.valueOf(setup_dialog.hardware_test_chkbox.isSelected()) + "\n");
							
			// PCI hardware test checkbox.
			outFile.writeBytes(hardware_test_dialog.pci_chkbox.getText() + "\t" + String.valueOf(hardware_test_dialog.pci_chkbox.isSelected()) + "\n");

			// Timing hardware test checkbox.
			outFile.writeBytes(hardware_test_dialog.timing_chkbox.getText() + "\t" + String.valueOf(hardware_test_dialog.timing_chkbox.isSelected()) + "\n");

			// Utility hardware test checkbox.
			outFile.writeBytes(hardware_test_dialog.utility_chkbox.getText() + "\t" + String.valueOf(hardware_test_dialog.utility_chkbox.isSelected()) + "\n");

			// Timing board download checkbox.
			outFile.writeBytes(setup_dialog.timing_board_chkbox.getText() + "\t" + String.valueOf(setup_dialog.timing_board_chkbox.isSelected()) + "\n");
				
			// Utility board download checkbox.
			outFile.writeBytes(setup_dialog.utility_board_chkbox.getText() + "\t" + String.valueOf(setup_dialog.utility_board_chkbox.isSelected()) + "\n");
				
			// Power on checkbox.
			outFile.writeBytes(setup_dialog.power_on_chkbox.getText() + ":" + "\t" + String.valueOf(setup_dialog.power_on_chkbox.isSelected()) + "\n");

			// Dimensions checkbox.
			outFile.writeBytes(setup_dialog.dimensions_chkbox.getText() + "\t" + String.valueOf(setup_dialog.dimensions_chkbox.isSelected()) + "\n");
				
			// Timing file radio button.
			outFile.writeBytes("Timing " + setup_dialog.timing_file_rbutton.getText() + ":" + "\t" + String.valueOf(setup_dialog.timing_file_rbutton.isSelected()) + "\n");

			// Utility file radio button.
			outFile.writeBytes("Utility " + setup_dialog.utility_file_rbutton.getText() + ":" + "\t" + String.valueOf(setup_dialog.utility_file_rbutton.isSelected()) + "\n");
			
			// Image resolution 16bit radio button.
			outFile.writeBytes(setup_dialog.image_res_16bit_rbutton.getText() + ":" + "\t" + String.valueOf(setup_dialog.image_res_16bit_rbutton.isSelected()) + "\n");
				
			// Save all the textbox info.
			outFile.writeBytes("PCI Hardware Test:" + "\t" + hardware_test_dialog.pci_test_txtfld.getText() + "\n");
			outFile.writeBytes("Timing Hardware Test:" + "\t" + hardware_test_dialog.timing_test_txtfld.getText() + "\n");
			outFile.writeBytes("Utility Hardware Test:" + "\t" + hardware_test_dialog.utility_test_txtfld.getText() + "\n");
			outFile.writeBytes("PCI Filename:" + "\t" + setup_dialog.pci_file_txtfld.getText() + "\n");
			outFile.writeBytes("Timing Filename:" + "\t" + setup_dialog.timing_file_txtfld.getText() + "\n");
			outFile.writeBytes("Timing App Number:" + "\t" + setup_dialog.timing_app_txtfld.getText() + "\n");
			outFile.writeBytes("Utility Filename:" + "\t" + setup_dialog.utility_file_txtfld.getText() + "\n");
			outFile.writeBytes("Utility App Number:" + "\t" + setup_dialog.utility_app_txtfld.getText() + "\n");
			outFile.writeBytes("Image Rows:" + "\t" + setup_dialog.rows_txtfld.getText() + "\n");
			outFile.writeBytes("Image Cols:" + "\t" + setup_dialog.cols_txtfld.getText() + "\n");

			// Save controller configuration components.
			write_file(setup_dialog.controller_config_dialog, outFile);
			
			outFile.close();

			// Inform user that setup is complete.
			PrintMessage.message("SetupSave", 0, "Setup saved.\n", 0, true);
				
			return;
		}
		// If the Cancel button is pressed.
		else if (selected == file_chooser.CANCEL_OPTION) {
			return;
		}
	}

	/**
	*****************************************************************************
	*	Saves the current component settings in the Controller Configuration
	*	Window to the specified text file. 
	*
	*	@param controller_config_dialog The Controller Configuration Setup window.
	*	@param file The setup file to write to.
	*
	*	@throws IOException
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	protected void write_file(ControllerConfigDialog controller_config_dialog, RandomAccessFile outFile) throws IOException
	{
		outFile.writeBytes("***************** Controller Configuration ************************\n");
		// Every component in this section must be checked against null since the
		// components are added dynamically and not all components may exist.
		
		//*********************************
		//	BINNING
		//*********************************
		// Check that the binning components exist.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("Binning") != -1) {
			// Get the column binning parameter.
			outFile.writeBytes("Column Binning (value -1):" + "\t" + Integer.toString(controller_config_dialog.binning_parameters_combo_Bx.getSelectedIndex()) + "\n");

			// Get the row binning parameter.
			outFile.writeBytes("Row Binning (value -1):" + "\t" + Integer.toString(controller_config_dialog.binning_parameters_combo_By.getSelectedIndex()) + "\n");
		}
		
		else {
			outFile.writeBytes("Column Binning (value -1):\t0\n");
			outFile.writeBytes("Row Binning (value -1):\t0\n");
		}

		//*********************************
		//	READOUT
		//*********************************
		// Check that the readout components exist.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("Readout") != -1) {

			// Check that the parallel/serial readout components exist.
			if (controller_config_dialog.single_readout_A_rbutton != null) {

				// Single readout Amp A radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.single_readout_A_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.single_readout_A_rbutton.isSelected()) + "\n");

				// Single readout Amp B radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.single_readout_B_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.single_readout_B_rbutton.isSelected()) + "\n");

				// Single readout Amp C radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.single_readout_C_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.single_readout_C_rbutton.isSelected()) + "\n");

				// Single readout Amp D radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.single_readout_D_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.single_readout_D_rbutton.isSelected()) + "\n");

				// Dual readout Amp AB radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.dual_readout_AB_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.dual_readout_AB_rbutton.isSelected()) + "\n");

				// Dual readout Amp CD radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.dual_readout_CD_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.dual_readout_CD_rbutton.isSelected()) + "\n");

				// Quad readout Amp ABCD radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.quad_readout_ABCD_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.quad_readout_ABCD_rbutton.isSelected()) + "\n");

			}
	
			else {
				outFile.writeBytes("Amp A:\tfalse\n");
				outFile.writeBytes("Amp B:\tfalse\n");
				outFile.writeBytes("Amp C:\tfalse\n");
				outFile.writeBytes("Amp D:\tfalse\n");
				outFile.writeBytes("Amp AB:\tfalse\n");
				outFile.writeBytes("Amp CD:\tfalse\n");
				outFile.writeBytes("Amp ABCD:\tfalse\n");
			}		

			// Otherwise, must only be serial components that exist.
			if (controller_config_dialog.serial_readout_L_rbutton != null) {
				// Single readout Amp L radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.serial_readout_L_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.serial_readout_L_rbutton.isSelected()) + "\n");

				// Single readout Amp R radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.serial_readout_R_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.serial_readout_R_rbutton.isSelected()) + "\n");

				// Dual readout Amp LR radio button.
				outFile.writeBytes("Amp " + controller_config_dialog.serial_readout_LR_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.serial_readout_LR_rbutton.isSelected()) + "\n");
			}

			else {
				outFile.writeBytes("Amp L:\tfalse\n");
				outFile.writeBytes("Amp R:\tfalse\n");
				outFile.writeBytes("Amp LR:\tfalse\n");
			}
		}

		else {
			outFile.writeBytes("Amp A:\tfalse\n");
			outFile.writeBytes("Amp B:\tfalse\n");
			outFile.writeBytes("Amp C:\tfalse\n");
			outFile.writeBytes("Amp D:\tfalse\n");
			outFile.writeBytes("Amp AB:\tfalse\n");
			outFile.writeBytes("Amp CD:\tfalse\n");
			outFile.writeBytes("Amp ABCD:\tfalse\n");
		}		
		
		//*********************************
		//	GAIN & SPEED
		//*********************************
		// Check that the gain and speed components exist.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("Gain") != -1) {

			// Gain setting 1 radio button.
			outFile.writeBytes("Gain " + controller_config_dialog.gain_1_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.gain_1_rbutton.isSelected()) + "\n");

			// Gain setting 2 radio button.
			outFile.writeBytes("Gain " + controller_config_dialog.gain_2_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.gain_2_rbutton.isSelected()) + "\n");

			// Gain setting 4.75 radio button.
			outFile.writeBytes("Gain " + controller_config_dialog.gain_475_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.gain_475_rbutton.isSelected()) + "\n");

			// Gain setting 9.5 radio button.
			outFile.writeBytes("Gain " + controller_config_dialog.gain_95_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.gain_95_rbutton.isSelected()) + "\n");

			// Slow speed radio button.
			outFile.writeBytes("Speed " + controller_config_dialog.slow_speed_rbutton.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.slow_speed_rbutton.isSelected()) + "\n");
		}
		
		else {
			outFile.writeBytes("Gain 1.0:\tfalse\n");
			outFile.writeBytes("Gain 2.0:\tfalse\n");
			outFile.writeBytes("Gain 4.75:\tfalse\n");
			outFile.writeBytes("Gain 9.5:\tfalse\n");
			outFile.writeBytes("Slow Speed:\tfalse\n");
		}

		//*********************************
		//	MPP
		//*********************************
		// Check that the MPP component exists.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("MPP") != -1) {
			// MPP checkbox.
			outFile.writeBytes(controller_config_dialog.mpp_chkbox.getText() + ":" + "\t" + String.valueOf(controller_config_dialog.mpp_chkbox.isSelected()) + "\n");
		}
		
		else
			outFile.writeBytes("Use MPP mode:\tfalse\n");

		//*********************************
		//	TEMPERATURE
		//*********************************
		// Check that the Temperature component exists.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("Temperature") != -1) {
			// Temperature checkbox.
			outFile.writeBytes(controller_config_dialog.array_temperature_chkbox.getText() + "\t" + String.valueOf(controller_config_dialog.array_temperature_chkbox.isSelected()) + "\n");
			
			// Temperature textfield.
			outFile.writeBytes("Array Temperature Value:" + "\t" + controller_config_dialog.temperature_txtfld.getText() + "\n");

			// Linear coefficient #1 textfield.
			outFile.writeBytes("Linear Coefficient Value #1:" + "\t" + controller_config_dialog.linear_coeff1_txtfld.getText() + "\n");

			// Linear coefficient #1 textfield.
			outFile.writeBytes("Linear Coefficient Value #2:" + "\t" + controller_config_dialog.linear_coeff2_txtfld.getText() + "\n");
		}
		
		else {
			outFile.writeBytes("Array Temperature:\tfalse\n");
			outFile.writeBytes("Array Temperature Value:\t0\n");
			outFile.writeBytes("Linear Coefficient Value #1:\t0.0\n");
			outFile.writeBytes("Linear Coefficient Value #2:\t0.0\n");
		}

		outFile.close();
	}

	/**
	*****************************************************************************
	*	Loads the component settings, previously stored in the specified
	*	text file, into the Controller Setup Window. The file is specified
	*	via a popup JFileChooser dialog. Calls the read_file() method.
	*
	*	@param parent_component The container that holds the Controller Setup
	*		      		Window components (i.e. content pane, panel, etc).
	*	@param setup_dialog The Controller Setup window.
	*	@param hardware_test_dialog The Hardware Test Options window.
	*
	*	@throws IOException
	*	@see read_file()
	*	@see SetupDialog
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	public void read_from_file(Component parent_component, SetupDialog setup_dialog, HardwareTestDialog hardware_test_dialog) throws IOException
	{
		JFileChooser file_chooser;
		
		// Open a file chooser that points to the current dir.
		file_chooser = new JFileChooser(file.getPath());
		
		// Add a file filter.
		file_chooser.addChoosableFileFilter(new SetupFileFilter());
			
		// Assign the file filter and discard all others.
		file_chooser.setFileFilter(new SetupFileFilter());

		// Set the file chooser size.
		file_chooser.setPreferredSize(new Dimension(500, 250));
			
		// Show the Save dialog box (returns the option selected)
		int selected = file_chooser.showOpenDialog(parent_component);

		// If the Open button is pressed.
		if (selected == JFileChooser.APPROVE_OPTION) {
			// Get the selected file.
			file = file_chooser.getSelectedFile();
				
			// Read the file.
			read_file(file, setup_dialog, hardware_test_dialog);
			
			// Inform user that setup is complete.
			PrintMessage.message("SetupSave", 0, "Setup loaded.\n", 0, true);
		}

		// If the Cancel button is pressed.
		else if (selected == file_chooser.CANCEL_OPTION) {
			return;
		}
	}

	/**
	*****************************************************************************
	*	Loads the component settings, previously stored in the specified
	*	text file, into the Controller Setup Window.
	*
	*	@param file The file that contains the saved settings.
	*	@param setup_dialog The Controller Setup window.
	*	@param hardware_test_dialog The Hardware Test Options window.
	*
	*	@throws IOException
	*	@see read_from_file()
	*	@see SetupDialog
	*	@version 1.00
	*	@author Scott Streit
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	06/26/2000	sds	Changed file format from binary to text.
	*****************************************************************************
	*/
	protected void read_file(File file, SetupDialog setup_dialog, HardwareTestDialog hardware_test_dialog) throws IOException
	{
		RandomAccessFile inFile;
		SetupCommand setup_cmd = setup_dialog.setup_listener.setup_cmd;
		boolean selected;
		String line;

			// Open the file and output.
			inFile = new RandomAccessFile(file, "r");
				
			// Read informational message and throw it away.
			line = inFile.readLine();

			// PCI download checkbox.
			line = inFile.readLine();
			selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
			setup_dialog.pci_download_chkbox.setSelected(selected);

			// Reset controller checkbox.
			line = inFile.readLine();
			setup_dialog.reset_controller_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());

			// Hardware test checkbox.
			line = inFile.readLine();
			setup_dialog.hardware_test_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());
			
			// PCI hardware test checkbox.
			line = inFile.readLine();
			hardware_test_dialog.pci_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());

			// Timing hardware test checkbox.
			line = inFile.readLine();
			hardware_test_dialog.timing_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());

			// Utility hardware test checkbox.
			line = inFile.readLine();
			hardware_test_dialog.utility_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());

			// Timing board download checkbox.
			line = inFile.readLine();
			setup_dialog.timing_board_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());

			// Utility board download checkbox.
			line = inFile.readLine();
			setup_dialog.utility_board_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());
			
			// Power on checkbox.
			line = inFile.readLine();
			setup_dialog.power_on_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());

			// Dimensions checkbox.
			line = inFile.readLine();
			setup_dialog.dimensions_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());
					
			// Timing file radio button.
			line = inFile.readLine();
			selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
			setup_dialog.timing_file_rbutton.setSelected(selected);
			if (selected) {
				setup_dialog.get_timing_file_button.setEnabled(true);
				setup_dialog.timing_app_txtfld.setEditable(false);
				setup_dialog.timing_file_txtfld.setEditable(true);
				setup_cmd.set_timing_file(true);
				setup_cmd.set_timing_app(false);
			}
			else {
				setup_dialog.get_timing_file_button.setEnabled(false);
				setup_dialog.timing_app_rbutton.setSelected(true);
				setup_dialog.timing_app_txtfld.setEditable(true);
				setup_dialog.timing_file_txtfld.setEditable(false);
				setup_cmd.set_timing_file(false);
				setup_cmd.set_timing_app(true);
			}
				
			// Utility file radio button.
			line = inFile.readLine();
			selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
			setup_dialog.utility_file_rbutton.setSelected(selected);
			if (selected) {
				setup_dialog.get_utility_file_button.setEnabled(true);
				setup_dialog.utility_app_txtfld.setEditable(false);
				setup_dialog.utility_file_txtfld.setEditable(true);
				setup_cmd.set_utility_file(true);
				setup_cmd.set_utility_app(false);
			}
			else {
				setup_dialog.get_utility_file_button.setEnabled(false);
				setup_dialog.utility_app_rbutton.setSelected(true);
				setup_dialog.utility_app_txtfld.setEditable(true);
				setup_dialog.utility_file_txtfld.setEditable(false);
				setup_cmd.set_utility_file(false);
				setup_cmd.set_utility_app(true);
			}
				
			// Image resolution 16bit radio button.
			line = inFile.readLine();
			selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
			setup_dialog.image_res_16bit_rbutton.setSelected(selected);
			if (selected)
				setup_cmd.set_image_resolution(16);
			else {
				setup_dialog.image_res_32bit_rbutton.setSelected(true);
				setup_cmd.set_image_resolution(32);
			}

			// PCI hardware test textfield.
			line = inFile.readLine();
			hardware_test_dialog.pci_test_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Timing hardware test textfield.
			line = inFile.readLine();
			hardware_test_dialog.timing_test_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Utility hardware test textfield.
			line = inFile.readLine();
			hardware_test_dialog.utility_test_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// PCI filename textfield.
			line = inFile.readLine();
			setup_dialog.pci_file_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Timing filename textfield.
			line = inFile.readLine();
			setup_dialog.timing_file_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Timing application textfield.
			line = inFile.readLine();
			setup_dialog.timing_app_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Utility filename textfield.
			line = inFile.readLine();
			setup_dialog.utility_file_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Utility application textfield.
			line = inFile.readLine();
			setup_dialog.utility_app_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Image rows textfield.
			line = inFile.readLine();
			setup_dialog.rows_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Image cols textfield.
			line = inFile.readLine();
			setup_dialog.cols_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			inFile.close();

			// Set the filename in the global setup dialog.
			setup_dialog.parent.voodoo_global_setup_dialog.set_controller_setup_txtfld(file.getAbsolutePath());

			return;
	}

	/**
	*****************************************************************************
	*	Loads the component settings, previously stored in the specified
	*	text file, into the Controller Configuration Setup Window.
	*
	*	@param controller_config_dialog The Controller Configuration Setup window.
	*	@param file The file that contains the saved settings.
	*
	*	@throws IOException
	*	@see read_from_file()
	*	@see SetupDialog
	*	@version 1.00
	*	@author Scott Streit
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	06/26/2000	sds	Changed file format from binary to text.
	*****************************************************************************
	*/
	protected void read_file(File file, ControllerConfigDialog controller_config_dialog) throws IOException
	{
		RandomAccessFile inFile;
		boolean done;
		boolean selected;
		String line;

		// Check that controller configuration tabs exist.
		if (controller_config_dialog.controller_config_tabbed_pane == null)
			return;

		// Open the file for input.
		inFile = new RandomAccessFile(file, "r");

		// Search for the start of the controller configuration data.
		done = false;
		while (!done) {
			if (inFile.readLine().equals("***************** Controller Configuration ************************"))
				done = true;
		}
			
		//*********************************
		//	BINNING
		//*********************************
		// Check if the binning components exist.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("Binning") != -1) {
			
			// Get the column binning parameter.
			line = inFile.readLine();
			controller_config_dialog.binning_parameters_combo_Bx.setSelectedIndex(new Integer(line.substring(line.lastIndexOf("\t")+1)).intValue());

			// Get the row binning parameter.
			line = inFile.readLine();
			controller_config_dialog.binning_parameters_combo_By.setSelectedIndex(new Integer(line.substring(line.lastIndexOf("\t")+1)).intValue());
		}

		//*********************************
		//	READOUT
		//*********************************
		// Check that the readout components exist.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("Readout") != -1) {

			// Check that the parallel/serial readout components exist.
			if (controller_config_dialog.single_readout_A_rbutton != null) {

				// Get single readout Amp A radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.single_readout_A_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.single_readout_A_rbutton.doClick();

				// Get single readout Amp B radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.single_readout_B_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.single_readout_B_rbutton.doClick();

				// Get single readout Amp C radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.single_readout_C_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.single_readout_C_rbutton.doClick();

				// Get single readout Amp D radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.single_readout_D_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.single_readout_D_rbutton.doClick();

				// Get dual readout Amp AB radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.dual_readout_AB_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.dual_readout_AB_rbutton.doClick();

				// Get dual readout Amp CD radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.dual_readout_CD_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.dual_readout_CD_rbutton.doClick();

				// Get quad readout Amp ABCD radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.quad_readout_ABCD_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.quad_readout_ABCD_rbutton.doClick();
					
				// Skip the next three (serial readout) lines.
				line = inFile.readLine();
				line = inFile.readLine();
				line = inFile.readLine();
			}
			
			// Otherwise, must be serial readout components that exist.
			else {
					
				// Skip the first seven (both readouts) lines.
				line = inFile.readLine();
				line = inFile.readLine();
				line = inFile.readLine();
				line = inFile.readLine();
				line = inFile.readLine();
				line = inFile.readLine();
				line = inFile.readLine();

				// Get serial readout Amp L radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.serial_readout_L_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.serial_readout_L_rbutton.doClick();

				// Get serial readout Amp R radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.serial_readout_R_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.serial_readout_R_rbutton.doClick();

				// Get serial readout Amp LR radio button.
				line = inFile.readLine();
				selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
				controller_config_dialog.serial_readout_LR_rbutton.setSelected(selected);
				if (selected)
					controller_config_dialog.serial_readout_LR_rbutton.doClick();
			}
		}

		//*********************************
		//	GAIN & SPEED
		//*********************************
		// Check if the gain and speed buttons exist.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("Gain") != -1) {

			// Gain setting 1 radio button.
			line = inFile.readLine();
			selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
			controller_config_dialog.gain_1_rbutton.setSelected(selected);
			if (selected)
				controller_config_dialog.controller_config_listener.controller_config_cmd.set_gain(VoodooConstants.ONE);

			// Gain setting 2 radio button.
			line = inFile.readLine();
			selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
			controller_config_dialog.gain_2_rbutton.setSelected(selected);
			if (selected)
				controller_config_dialog.controller_config_listener.controller_config_cmd.set_gain(VoodooConstants.TWO);

			// Gain setting 4.75 radio button.
			line = inFile.readLine();
			selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
			controller_config_dialog.gain_475_rbutton.setSelected(selected);
			if (selected)
				controller_config_dialog.controller_config_listener.controller_config_cmd.set_gain(VoodooConstants.FIVE);

			// Gain setting 9.5 radio button.
			line = inFile.readLine();
			selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
			controller_config_dialog.gain_95_rbutton.setSelected(selected);
			if (selected)
				controller_config_dialog.controller_config_listener.controller_config_cmd.set_gain(VoodooConstants.TEN);
					
			// Slow speed radio button.
			line = inFile.readLine();
			selected = new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue();
			controller_config_dialog.slow_speed_rbutton.setSelected(selected);
			if (selected)
				controller_config_dialog.controller_config_listener.controller_config_cmd.set_speed(VoodooConstants.SLOW);
			else {
				controller_config_dialog.fast_speed_rbutton.setSelected(true);
				controller_config_dialog.controller_config_listener.controller_config_cmd.set_speed(VoodooConstants.FAST);
			}
		}

		//*********************************
		//	MPP
		//*********************************
		// Check that the MPP component exists.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("MPP") != -1) {

			// MPP checkbox.
			line = inFile.readLine();
			controller_config_dialog.mpp_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());
		}

		//*********************************
		//	TEMPERATURE
		//*********************************
		// Check if the temperature buttons exist.
		if (controller_config_dialog.controller_config_tabbed_pane.indexOfTab("Temperature") != -1) {
			
			// Temperature checkbox.
			line = inFile.readLine();
			controller_config_dialog.array_temperature_chkbox.setSelected(new Boolean(line.substring(line.lastIndexOf("\t")+1)).booleanValue());

			// Array temperature textfield.
			line = inFile.readLine();
			controller_config_dialog.temperature_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Linear coefficient #1 textfield.
			line = inFile.readLine();
			controller_config_dialog.linear_coeff1_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));

			// Linear coefficient #2 textfield.
			line = inFile.readLine();
			controller_config_dialog.linear_coeff2_txtfld.setText(line.substring(line.lastIndexOf("\t")+1));
		}
		
		inFile.close();
	}
}
