import javax.swing.SwingUtilities;

/**
 * An abstract class that you subclass to perform
 * GUI-related work in a dedicated thread.
 * For instructions on using this class, see 
 * http://java.sun.com/products/jfc/swingdoc/threads.html
 * This version has an extra method called <code>interrupt()</code>,
 * see the Example1 class below for more information.
 */
abstract class SwingWorker 
{
    private Object value;
    private volatile Thread thread;
    protected boolean abort;
    
    /** 
     * Compute the value to be returned by the <code>get</code> method. 
     */
    public abstract Object construct();


    /**
     * Called on the event dispatching thread (not on the worker thread)
     * after the <code>construct</code> method has returned.
     */
    public void finished() {}


     /**
     * Check if the current thread is alive.
     */
    public boolean alive() {
	if (thread != null)
		return thread.isAlive();
	else
		return false;
    }


     /**
     * Abort the current thread.
     */
    public void stop() {
	abort = true;
    }


    /**
     * A new method that interrupts the worker thread.  Call this method
     * to force the worker to abort what it's doing.
     */
    public void interrupt() {
	Thread t = thread;
	if (t != null) {
	    t.interrupt();
	}
	thread = null;
    }

    /**
     * Return the value created by the <code>construct</code> method.  
     */
    public Object get() {
        while (true) {  // keep trying if we're interrupted
            Thread t;
            synchronized (SwingWorker.this) {
                t = thread;
                if (t == null) {
                    return value;
                }
            }
            try {
                t.join();
            }
            catch (InterruptedException e) {
            }
        }
    }

    /**
     * Modified to do nothing. To start the thread, the subclass must
     * call the "thread_it" method.
     */
    public SwingWorker() {}

    /**
     * Start a thread that will call the <code>construct</code> method
     * and then exit.
     */    
    public synchronized void thread_it() {
        abort = false;
        final Runnable doFinished = new Runnable() {
           public void run() { finished(); }
        };

        Runnable doConstruct = new Runnable() { 
            public void run() {
                synchronized(SwingWorker.this) {
                    value = construct();
                    thread = null;
                }
                SwingUtilities.invokeLater(doFinished);
            }
        };

        thread = new Thread(doConstruct);
        thread.start();
    }
}
