/**
********************************************************************************
*	A class to display a dialog window that will allow the user to dynamically
*	change and save global program options.
*
*	CVM - View
*
*	@see JDialog
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

class VoodooGlobalSetupDialog extends JDialog
{
	protected Container content_pane;
	private JMenuBar new_menu_bar;
	private VoodooGlobalSetupListener global_setup_listener = null;
	private Font font;
	private Icon browse_icon;
	protected VoodooMainWindow parent = null;
	protected VTextField width_txtfld;
	protected VTextField height_txtfld;
	protected VTextField statusWin_font_txtfld;
	protected VTextField statusWin_fontsize_txtfld;
	protected VTextField compnt_font_txtfld;
	protected VTextField compnt_fontsize_txtfld;
	protected VTextField controller_file_txtfld;
	public int number_of_global_setup_windows;
	
	/**
	********************************************************************************
	*	Constructor. Creates and sets all dialog window properties.
	*
	*	@param owner The parent window for this dialog box.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public VoodooGlobalSetupDialog(Frame owner)
	{
		// Call the superclass (JDialog) constructor.
		super(owner, "Voodoo Options", false);
		
		// Save the parent info.
		parent = (VoodooMainWindow)owner;
		
		// Set the dialog box size.
		setSize(375, 250);
		
		// Make the window non-resizable.
      		setResizable(false);

		// Get a handle on the dialog box's content pane.
		content_pane = this.getContentPane();
		
		// Set the components font.
		font = new Font("LucidaSansRegular", Font.PLAIN, 11);

		browse_icon = new ImageIcon(VoodooConstants.PIC_DIR + "folder.gif");
		
		// Create the event listener for this dialog box.
		global_setup_listener = new VoodooGlobalSetupListener(this);

		// Create the menu bar and add it to the dialog box.
		new_menu_bar = create_menu_bar();
		setJMenuBar(new_menu_bar);
		
		// Create the tabbed pane.
		JTabbedPane tp = create_tabbed_pane();
		content_pane.add(tp);
		
      		// Add a window listener.  This has to be here to allow the
      		// dialog box to be re-opened later when the "close" window
      		// menu item (from system, not java) is chosen.
      		addWindowListener(new WindowAdapter()
      		{
      			public void windowClosing(WindowEvent e)
      			{
      				number_of_global_setup_windows = 0;
			}
		});
		
		// Initialize any variables.
		number_of_global_setup_windows = 0;
	}

	/**
	*****************************************************************************
	*	Creates the "tabbed" panels portion of the window.
	*
	*	@return The created "tabbed pane" is returned.
	*
	*	@see JTabbedPane
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	private JTabbedPane create_tabbed_pane()
	{
		// Create a new tabbed pane and set the tabs to be on top.
		JTabbedPane global_tabbed_pane = new JTabbedPane(JTabbedPane.TOP);
		
		// Set the tab event listener.
      		global_tabbed_pane.addChangeListener(global_setup_listener);
      		
      		Component save_panel = create_save_panel();
      		global_tabbed_pane.addTab("Save", save_panel);

      		Component properties_panel = create_properties_panel();
      		global_tabbed_pane.addTab("Windows", properties_panel);
      		
      		return global_tabbed_pane;
      	}

	/**
	********************************************************************************
	*	This function creates the menu bar for this dialog window.
	*
	*	@return Returns the constructed menubar as a JMenuBar class.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JMenuBar create_menu_bar()
	{
		JMenuBar menu_bar = new JMenuBar();

		// Set the menubar's border.
		menu_bar.setBorder(new BevelBorder(BevelBorder.RAISED));
		menu_bar.setBorderPainted(true);
		
		// Add the Window menu to the menu bar.
		JMenu window_menu = new JMenu("Window");
		menu_bar.add(window_menu);

		// Add the submenus to the Window menu.
		JMenuItem close_item = new JMenuItem("Close", 'C');
		window_menu.add(close_item);
		
		// Set the menubar font.
		window_menu.setFont(font);
		close_item.setFont(font);

		// Add the event listeners to the menu items.
		close_item.addActionListener(global_setup_listener);
		
		return menu_bar;
	}

	/**
	********************************************************************************
	*	This function creates the program window options panel for this dialog
	*	window.
	*
	*	@return Returns the constructed window options panel as a JPanel class.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JPanel create_properties_panel()
	{
     		JPanel properties_panel = new JPanel();
		
		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		properties_panel.setLayout(gbl);

		// Create the combo box options.
		String windows[] = {"Choose A Window Here ...", "Voodoo Main Window",
				    "Controller Setup Window", "DSP Commands Window",
				    "FITS Window", "Image Buffer Window",
				    "Global Options Window", "Hardware Test Window",
				    "Controller Config Window"};

		// Create the combo box.
		JComboBox window_cbox = new JComboBox(windows);
		window_cbox.setSelectedItem((Object)(new String("Choose A Window Here ...")));
		
    		// Create the label components.
      		JLabel width_label = new JLabel("Width: ");
      		JLabel height_label = new JLabel("Height: ");
      		JLabel statusWin_font_label = new JLabel("Textfield Font: ");
      		JLabel statusWin_fontsize_label = new JLabel("Size: ");
      		JLabel compnt_font_label = new JLabel("Component Font: ");
      		JLabel compnt_fontsize_label = new JLabel("Size: ");

     		// Create the textfield components.
		width_txtfld = new VTextField("", 4);
		height_txtfld = new VTextField("", 4);
		statusWin_font_txtfld = new VTextField("", 10);
		statusWin_fontsize_txtfld = new VTextField("", 2);
		compnt_font_txtfld = new VTextField("LucidaSansRegular", 10);
		compnt_fontsize_txtfld = new VTextField("11", 2);
		
		// Create the button components.
		JButton apply_winSize_button = new JButton("Apply Window Size");

		// Add the components to the panel.
      		add_components(properties_panel, window_cbox, gbl, gbc, 0, 0, 15, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 4, 1);
      		add_components(properties_panel, width_label, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 1, 1, 1);
      		add_components(properties_panel, width_txtfld, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 1, 1, 1);
      		add_components(properties_panel, height_label, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 1, 1, 1);
      		add_components(properties_panel, height_txtfld, gbl, gbc, 0, 0, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 3, 1, 1, 1);
/*
      		add_components(properties_panel, statusWin_font_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 3, 2, 1);
      		add_components(properties_panel, statusWin_font_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 3, 2, 1);
      		add_components(properties_panel, statusWin_fontsize_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 4, 3, 1, 1);
      		add_components(properties_panel, statusWin_fontsize_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 3, 1, 1);
      		add_components(properties_panel, compnt_font_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 0, 4, 2, 1);
      		add_components(properties_panel, compnt_font_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 4, 2, 1);
      		add_components(properties_panel, compnt_fontsize_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 4, 4, 1, 1);
      		add_components(properties_panel, compnt_fontsize_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 5, 4, 1, 1);
*/
      		add_components(properties_panel, apply_winSize_button, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 6, 4, 1);

		// Add the event listener to the components.
		window_cbox.addItemListener(global_setup_listener);
		apply_winSize_button.addActionListener(global_setup_listener);
		
    		return properties_panel;
     	}

	/**
	********************************************************************************
	*	This function creates the save global options panel for this dialog
	*	window.
	*
	*	@return Returns the constructed save options panel as a JPanel class.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JPanel create_save_panel()
	{
     		JPanel save_panel = new JPanel();
		
		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		save_panel.setLayout(gbl);

		// Create the textfield labels.
		JLabel controller_file_label = new JLabel("Controller Setup File: ");
		
		// Create the textfields.
		controller_file_txtfld = new VTextField("", 15);

		// Create the button components.
		JButton save_button = new JButton("Save Settings");
		JButton browse_controller_button = new JButton(browse_icon);
		
		// Set the button tooltip text.
		browse_controller_button.setToolTipText("Browse For File");
		
		// Set the "Browse" buttons action command string.
      		browse_controller_button.setActionCommand("Controller Setup Browse");

		// Set the button sizes.
		browse_controller_button.setPreferredSize(new Dimension(23, 22));
		
		// Add the components to the panel.
      		add_components(save_panel, controller_file_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 0, 1, 1);
      		add_components(save_panel, controller_file_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 0, 1, 1);
       		add_components(save_panel, browse_controller_button, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 2, 0, 1, 1);
     		add_components(save_panel, save_button, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 2, 3, 1);

		// Add the event listener to the components.
		save_button.addActionListener(global_setup_listener);
		browse_controller_button.addActionListener(global_setup_listener);
		
    		return save_panel;
	}

	/**
	********************************************************************************
	*	This method sets the controller setup filename text in the controller
	*	setup textfield.
	*
	*	@param setup_filename The filename to be set in the controller textfield.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_controller_setup_txtfld(String setup_filename)
	{
		controller_file_txtfld.setText(setup_filename);
	}
	
	/**
	*****************************************************************************
	*	Adds the specified component to the specified panel.
	*
	*	@param panel The panel to which the component will be added.
	*	@param compnt The component to add.
	*	@param gbl The GridBagLayout of the panel.
	*	@param gbl The GridBagConstraints of the panel.
	*	@param insetTOP The distance in pixels between the top of this
	*			component and the one above it.
	*	@param insetBOTTOM The distance in pixels between the bottom of
	*			this component and the one below it.
	*	@param insetLEFT The distance in pixels between the left side of
	*			this component and the one left of it.
	*	@param insetRIGHT The distance in pixels between the right side
	*			of this component and the one right of it.
	*	@param cfill Determines how the component "fills" the space of
	*		  	of its "cell".
	*	@param anchr The justification of the component within its "cell".
	*	@param col The x position of the component.
	*	@param row The y position of the component.
	*	@param gw The number of "cells" that the component spans in the
	*			width direction.
	*	@param gh The number of "cells" that the component spans in the
	*			height direction.
	*
	*	@see JPanel
	*	@see JComponent
	*	@see GridBagLayout
	*	@see GridBagConstraints
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
      	private void add_components(JPanel panel, JComponent compnt, GridBagLayout gbl, GridBagConstraints gbc, int insetTOP, int insetLEFT, int insetBOTTOM, int insetRIGHT, int cfill, int anchr, int col, int row, int gw, int gh)
   	{
      		// Set the grid layout constraints.
      		gbc.fill = cfill;
      		gbc.anchor = anchr;
      		gbc.insets = new Insets(insetTOP, insetLEFT, insetBOTTOM, insetRIGHT);
      		gbc.gridx = col;
      		gbc.gridy = row;
      		gbc.gridwidth = gw;
      		gbc.gridheight = gh;
      		
      		// Set the component font.
      		compnt.setFont(font);
      
      		// Add the component to the panel.
      		gbl.setConstraints(compnt, gbc);
      		panel.add(compnt);
      	}
}
