/**
********************************************************************************
*	This class creates the main application window and all of it's input
*	components.
*
*	CVM - View
*
*	@see CloseableFrame
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import CloseableFrame.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.text.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.Toolkit;

public class VoodooMainWindow extends CloseableFrame
{
	private Container content_pane;
	private Container panel_holder;
	private Frame parent_frame;
	protected ImageBuffersDialog image_buffers_dialog = null;
	protected DspDialog dsp_dialog = null;
	protected SetupDialog setup_dialog = null;
	protected FitsDialog fits_dialog = null;
	protected VoodooGlobalSetupDialog voodoo_global_setup_dialog = null;
	protected VoodooMainWindowListener main_window_listener = null;
	protected ExposureControlListener exposure_control_listener = null;
	protected ArrayControlListener array_control_listener;
	private VoodooGlobalSetupSave global_setup = null;
	private JPanel new_ccd_control_panel;
	private JPanel new_image_control_panel;
	private JPanel new_exposure_panel;
	private JPanel sub_panel_holder;
	private JPanel button_panel;
	private JMenuBar new_menu_bar;
	private JToolBar new_toolbar;
	private static JTextPane info_output_textarea;
	private static Document contentModel;
	private static MutableAttributeSet charStyle1;
	private static MutableAttributeSet charStyle2;
	private static MutableAttributeSet charStyle3;
	private static JEditorPane info_pane;
	private static JScrollPane scroll_pane;
	private static JProgressBar progress_bar;
	private static JLabel elapsed_label;
	private static int line_counter;
	private JLabel temperature1_label;
	private JLabel temperature2_label;
	private Image minimize_icon;
	private Font font;
	protected JButton expose_button;
	protected JButton pause_button;
	protected JButton change_exp_time_button;
	protected JButton open_shutter_now_button;
	protected JButton clear_array_button;
	protected JButton idle_array_button;
	protected JButton read_temperature_button;
	protected JButton reset_controller_button;
	protected JButton reset_pci_pc_button;
	protected JCheckBox save_disk_chkbox;
	protected JCheckBox display_image_chkbox;
	protected JCheckBox delay_chkbox;
	protected JCheckBox multi_exp_chkbox;
	protected JCheckBox beep_chkbox;
	protected JCheckBox open_shutter_chkbox;
	protected JCheckBox autoincr_chkbox;
	protected JLabel image_incr_start_number_label;
	protected VTextField exposure_time_txtfld;
	protected VTextField mult_exp_txtfld;
	protected VTextField image_filename_txtfld;
	protected VTextField delay_txtfld;
	protected VTextField image_incr_start_number_txtfld;
	protected JComboBox deinterlace_combo_box;

	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@param window_width The desired width of the application window.
	*	@param window_height The desired height of the application window.
	*	@param window_title The text to display at the top of the application
	*			    window.
	*
	*	@see CloseableFrame
	*	@version 1.1
	*	@author Scott Streit
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	02/08/2000	sds	Removed reply buffer ready bit set.
	********************************************************************************
	*/
	public VoodooMainWindow(String window_title)
	{
		// Call superclass constructor.
		super(480, 620, window_title);
		
		// Get a reference to the content pane.
		content_pane = this.getContentPane();

		// Set the component font.
		font = new Font("LucidaSansRegular", Font.PLAIN, 11);

		// Create all the child windows.
		image_buffers_dialog = new ImageBuffersDialog(this);
		dsp_dialog = new DspDialog(this);
		setup_dialog = new SetupDialog(this);
		fits_dialog = new FitsDialog(this);
		voodoo_global_setup_dialog = new VoodooGlobalSetupDialog(this);

		// Set the event listeners for this window.
		main_window_listener = new VoodooMainWindowListener(this);
		array_control_listener = new ArrayControlListener(this);
		exposure_control_listener = new ExposureControlListener(this);

		// Add the key listener for voodoo.
		addKeyListener(main_window_listener);
		
     		// Set the layout for the main window.
     		//content_pane.setLayout(new GridLayout(2,1));
     		content_pane.setLayout(new BorderLayout());
     		
     		// Create and add the menu/toolbar panel.
     		// 11-2-99 menu_panel = new JPanel(false);
     		// 11-2-99 content_pane.add(menu_panel, BorderLayout.NORTH);
		// 11-2-99 menu_panel.setLayout(new BorderLayout());
     		//menu_panel.setLayout(new GridLayout(2,1));

		// Create and add the toolbar to the menu/toolbar panel.
// 11-2-99		new_toolbar = create_toolbar();
// 11-2-99		content_pane.add(new_toolbar, BorderLayout.NORTH);


		// Create and add the menu bar to the menu/toolbar panel.
		new_menu_bar = create_menu_bar();
		setJMenuBar(new_menu_bar);

		// Create the container to hold the exposure and info panels.
		panel_holder = new Container();
		panel_holder.setLayout(new GridLayout(2,1));

		// Create the container to hold the CCD and image control and info panels.
		sub_panel_holder = new JPanel();
		GridBagLayout mgbl = new GridBagLayout();
      		GridBagConstraints mgbc = new GridBagConstraints();
      		sub_panel_holder.setLayout(mgbl);

		// Create and add the CCD exposure control panel.
		new_ccd_control_panel = create_camera_control_panel();
      		add_components(sub_panel_holder, new_ccd_control_panel, mgbl, mgbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1, 0, 1, 1);
		
		// Create and add the image control panel.
		new_image_control_panel = create_image_control_panel();
      		add_components(sub_panel_holder, new_image_control_panel, mgbl, mgbc, 0, 0, 0, 5, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 0, 1, 1);

		// Create and add the exposure control panel.
		new_exposure_panel = create_exposure_control_panel();
      		add_components(sub_panel_holder, new_exposure_panel, mgbl, mgbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 2, 2, 1);

		// Add the sub container to the main panel container.
		panel_holder.add(sub_panel_holder);
			
  		// Create and add the info/status panel.
		JPanel jp = new JPanel(new GridLayout(1,1), true);
		scroll_pane = create_info_panel();
		jp.add(scroll_pane);
		panel_holder.add(jp); //scroll_pane);

		// Add the panel container to the window.
		content_pane.add(panel_holder, BorderLayout.CENTER);

		// Set the minimized icon.
		minimize_icon = Toolkit.getDefaultToolkit().getImage(VoodooConstants.PIC_DIR + "voodooicon.gif");
		setIconImage(minimize_icon);
		
      		// Add a window listener.  This has to be here and not in
      		// CloseableFrame to allow the memory resources to be released
      		// when the "close" window menu item (from system, not java) is
      		// chosen.
      		addWindowListener(new WindowAdapter()
      		{
      			public void windowClosing(WindowEvent e)
      			{
      				try {
      					dispose();
					MemorySetup.free_memory();
					VoodooSetup.close_device_driver();
					System.exit(0);
				}
		    		catch (Exception ex) {
					JOptionPane.showMessageDialog(content_pane, ex.getMessage(), "Device Driver Error", JOptionPane.ERROR_MESSAGE);
       					System.exit(1);
    				}
			}
		});

   		// Initialize any variables.
   		line_counter = 0;

		// Auto-setup global program parameters.
 		try {
 			global_setup = new VoodooGlobalSetupSave(voodoo_global_setup_dialog);
 			global_setup.read_from_file();
		}
		catch (IOException ioe) {}

  		
/************* WILL BE RE-IMPLEMENTED LATER ****************
 		// Initialize camera properties, PCI DSP does not do this on it's own.
		try {
			DspCommand2.set_shutter_position(false);
		}
		catch (ReplyValueException rve) {
			PrintMessage.message("VoodooMainWindow", 1, "ERROR: Cannot set \"close shutter\".\n", 0, true);
		}
		catch (Exception e) {
			PrintMessage.message("VoodooMainWindow", 1, "ERROR: PCI board not ready to process commands. \n", 0, true);
		}
*/

    		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		show();
	}
	
	/**
	********************************************************************************
	*	This method returns the parent frame for the specified component. USED ???
	*
	*	@param component The component who's parent frame is wanted.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private Frame getParentFrame(Component component)
	{
		if (component instanceof Frame)
			return (JFrame)component;
		
		for (Component c = component; c!=null; c.getParent()) {
			// Check only for Frame, not JFrame!
			if (c instanceof Frame)
				return (Frame)c;
		}
		
		// No frames, return null.
		return null;
	}
	
	/**
	********************************************************************************
	*	This method creates the menu bar.
	*
	*	@return Returns the constructed menu bar.
	*
	*	@see JMenuBar
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JMenuBar create_menu_bar()
	{
		JMenuBar menu_bar = new JMenuBar();
		
		// Set the menubar's border.
		menu_bar.setBorder(new BevelBorder(BevelBorder.RAISED));
		menu_bar.setBorderPainted(true);
		
		// Add the File menu to the menu bar.
		JMenu file_menu = new JMenu("File");
		menu_bar.add(file_menu);
		
		// Add the submenus to the File menu.
		JMenuItem exit_item = new JMenuItem("Exit", 'E');
		JMenuItem global_setup_item = new JMenuItem("Global Setup...", 'G');
		JMenuItem about_item = new JMenuItem("About", 'A');
		
		file_menu.add(global_setup_item);
		file_menu.add(about_item);
		file_menu.addSeparator();
		file_menu.add(exit_item);
				
		// Add the Setup menu to the menu bar.
		JMenu setup_menu = new JMenu("Setup");
		menu_bar.add(setup_menu);
		
		// Add the submenus to the Setup menu.
		JMenuItem setup_item = new JMenuItem("Controller Setup...", 'C');
		JMenuItem fits_item = new JMenuItem("FITS Setup...", 'F');
		JMenuItem dsp_item = new JMenuItem("DSP Commands...", 'D');
		JMenuItem clear_info_item = new JMenuItem("Clear Info Window", 'C');
		JMenuItem image_buffers_item = new JMenuItem("Image Buffers...", 'I');
		JCheckBoxMenuItem exposure_time_item = new JCheckBoxMenuItem("Use Controller Exposure Time", true);
		JCheckBoxMenuItem debug_item = new JCheckBoxMenuItem("Debug Mode", false);
		JCheckBoxMenuItem synthetic_item = new JCheckBoxMenuItem("Synthetic Image Mode", false);
		setup_menu.add(setup_item);
		setup_menu.add(fits_item);
		setup_menu.add(dsp_item);
		setup_menu.add(image_buffers_item);
		setup_menu.addSeparator();
		setup_menu.add(clear_info_item);
		setup_menu.addSeparator();
		setup_menu.add(exposure_time_item);
		setup_menu.add(debug_item);
		setup_menu.add(synthetic_item);

		// Set the menubar font.
		file_menu.setFont(font);
		global_setup_item.setFont(font);
		exit_item.setFont(font);
		setup_menu.setFont(font);
		setup_item.setFont(font);
		fits_item.setFont(font);
		dsp_item.setFont(font);
		clear_info_item.setFont(font);
		image_buffers_item.setFont(font);
		exposure_time_item.setFont(font);
		debug_item.setFont(font);
		synthetic_item.setFont(font);
		about_item.setFont(font);

		// Add the event listeners to the menu items.
		global_setup_item.addActionListener(main_window_listener);
		exit_item.addActionListener(main_window_listener);
		setup_item.addActionListener(main_window_listener);
		fits_item.addActionListener(main_window_listener);
		dsp_item.addActionListener(main_window_listener);
		image_buffers_item.addActionListener(main_window_listener);
		clear_info_item.addActionListener(main_window_listener);
		about_item.addActionListener(main_window_listener);
		exposure_time_item.addActionListener(main_window_listener);
		debug_item.addActionListener(main_window_listener);
		synthetic_item.addActionListener(main_window_listener);
		
		return menu_bar;
	}

	/**
	********************************************************************************
	*	This method creates the tool bar.
	*
	*	@return Returns the constructed tool bar.
	*
	*	@see JToolBar
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JToolBar create_toolbar()
	{
		JToolBar toolbar = new JToolBar();
		
		// Set the margin between toolbar border and its components.
		toolbar.setMargin(new Insets(5, 5, 5, 5));
		
		// Set the toolbar to unfloatable.
		toolbar.setFloatable(true);
		
		// Create the toolbar buttons.
		JButton exit_button = new JButton("    Exit    ");
		JButton setup_button = new JButton("  Controller Setup  ");
		JButton fits_button = new JButton("    FITS    ");
		JButton dsp_button = new JButton("DSP Commands");
	
		// Set the button size.
		exit_button.setSize(new Dimension(90, 26));
		setup_button.setSize(new Dimension(90, 26));
		fits_button.setSize(new Dimension(90, 26));
		dsp_button.setSize(new Dimension(90, 26));

		// Add the buttons to the toolbar.
		// exit_button = toolbar.add(setup_button);
		toolbar.add(fits_button);
		toolbar.add(dsp_button);
		toolbar.add(exit_button);
		
		return toolbar;
	}

	/**
	********************************************************************************
	*	This method creates the array control panel.
	*
	*	@return Returns the constructed array control panel.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JPanel create_camera_control_panel()
	{
		JPanel camera_control_panel = new JPanel();
     		
     		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		camera_control_panel.setLayout(gbl);
      		
		// Create the CCD exposure options border.
		TitledBorder camera_options_border = new TitledBorder(new EtchedBorder(), "Camera");
		camera_options_border.setTitleFont(new Font(camera_control_panel.getFont().getFontName(), Font.BOLD, 14));
		camera_options_border.setTitleColor(Color.black);
		camera_control_panel.setBorder(camera_options_border);

		// Create buttons.
		open_shutter_now_button = new JButton("Open Shutter");
		clear_array_button = new JButton("Clear Array");
		idle_array_button = new JButton("Idle Array");
		reset_controller_button = new JButton("Reset Controller");
		reset_pci_pc_button = new JButton("Reset PCI");
		read_temperature_button = new JButton("Read Temp");

		open_shutter_now_button.setPreferredSize(new Dimension(120, 26));
		clear_array_button.setPreferredSize(new Dimension(120, 26));
		idle_array_button.setPreferredSize(new Dimension(120, 26));
		reset_controller_button.setPreferredSize(new Dimension(120, 26));
		reset_pci_pc_button.setPreferredSize(new Dimension(120, 26));
		read_temperature_button.setPreferredSize(new Dimension(120, 26));
		
		Insets insets = reset_controller_button.getMargin();
		insets.left = 0;
		insets.right = 0;
		reset_controller_button.setMargin(insets);
		
		// Create labels.
		temperature1_label = new JLabel("Temp: ");
		temperature2_label = new JLabel("0 C");

		temperature1_label.setForeground(Color.black);
		temperature2_label.setForeground(Color.black);

      		add_components(camera_control_panel, open_shutter_now_button, gbl, gbc, 0, 10, 0, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 1, 2, 1);
      		add_components(camera_control_panel, clear_array_button, gbl, gbc, 0, 10, 0, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 2, 2, 1);
      		add_components(camera_control_panel, idle_array_button, gbl, gbc, 0, 10, 0, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 3, 2, 1);
      		add_components(camera_control_panel, reset_controller_button, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 4, 2, 1);
      		add_components(camera_control_panel, reset_pci_pc_button, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 5, 2, 1);
      		add_components(camera_control_panel, read_temperature_button, gbl, gbc, 0, 10, 5, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER, 0, 6, 2, 1);
      		add_components(camera_control_panel, temperature1_label, gbl, gbc, 0, 10, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 0, 7, 1, 1);
      		add_components(camera_control_panel, temperature2_label, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 7, 1, 1);

		// Add the event listeners to the components.
		open_shutter_now_button.addActionListener(array_control_listener);
		clear_array_button.addActionListener(array_control_listener);
		idle_array_button.addActionListener(array_control_listener);
		reset_controller_button.addActionListener(array_control_listener);
		reset_pci_pc_button.addActionListener(array_control_listener);
		read_temperature_button.addActionListener(array_control_listener);

		return camera_control_panel;
	}
	
	/**
	********************************************************************************
	*	This method creates the image control panel.
	*
	*	@return Returns the constructed image control panel.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JPanel create_image_control_panel()
	{
     		JPanel image_panel = new JPanel();
     		
     		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		image_panel.setLayout(gbl);
      		
		// Create the exposure type border.
		TitledBorder exposure_options_border = new TitledBorder(new EtchedBorder(), "Exposure Options");
		exposure_options_border.setTitleFont(new Font(image_panel.getFont().getFontName(), Font.BOLD, 14));
		exposure_options_border.setTitleColor(Color.black);
		image_panel.setBorder(exposure_options_border);

     		// Create the checkboxes.
		beep_chkbox = new JCheckBox("Beep");
     		open_shutter_chkbox = new JCheckBox("Open Shutter");
     		save_disk_chkbox = new JCheckBox("Save To Disk: ");
     		display_image_chkbox = new JCheckBox("Display Image");
		delay_chkbox = new JCheckBox("Delay (secs): ");
		multi_exp_chkbox = new JCheckBox("Multiple Exp: ");
		autoincr_chkbox = new JCheckBox("Auto Incr");
		
     		// Create the labels.
     		image_incr_start_number_label = new JLabel("Start #: ");

		// Set the label colors.
		image_incr_start_number_label.setForeground(Color.black);

     		// Create the textfields.
     		image_filename_txtfld = new VTextField((new JFileChooser()).getCurrentDirectory().getAbsolutePath() + "/Image.fit", 14);
     		delay_txtfld = new VTextField("0", 3);
     		mult_exp_txtfld = new VTextField("1", 3);
     		image_incr_start_number_txtfld = new VTextField("1", 3);

		// Create the deinterlacing images.
		ImageIcon images[] = new ImageIcon[5];
		images[0] = new ImageIcon(VoodooConstants.PIC_DIR + "SDEINT1.gif");
		images[1] = new ImageIcon(VoodooConstants.PIC_DIR + "SDEINT2.gif");
		images[2] = new ImageIcon(VoodooConstants.PIC_DIR + "SDEINT3.gif");
		images[3] = new ImageIcon(VoodooConstants.PIC_DIR + "SDEINT4.gif");
		images[4] = new ImageIcon(VoodooConstants.PIC_DIR + "SDEINT5.gif");
		images[0].setDescription("None");
		images[1].setDescription("Parallel");
		images[2].setDescription("Serial");
		images[3].setDescription("Quad");
		images[4].setDescription("IR Quad");

		// Create the deinterlacing combo box.
		deinterlace_combo_box = new JComboBox(images);
		DeinterlaceCellRenderer renderer = new DeinterlaceCellRenderer();
		renderer.setPreferredSize(new Dimension(70,70));
		deinterlace_combo_box.setRenderer(renderer);
		deinterlace_combo_box.setMaximumRowCount(5);

		// Disable all components that need it.
		mult_exp_txtfld.setEnabled(false);
		delay_txtfld.setEnabled(false);
		autoincr_chkbox.setEnabled(false);
		image_filename_txtfld.setEnabled(false);
		image_incr_start_number_txtfld.setEnabled(false);
		image_incr_start_number_label.setEnabled(false);

     		// Add the components to the panel.
      		add_components(image_panel, delay_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 0, 1, 1);
      		add_components(image_panel, delay_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 0, 1, 1);
      		add_components(image_panel, multi_exp_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 1, 1, 1);
      		add_components(image_panel, mult_exp_txtfld, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 1, 1, 1);
      		add_components(image_panel, open_shutter_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 2, 1, 1);
     		add_components(image_panel, deinterlace_combo_box, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER, 2, 0, 4, 4);
      		add_components(image_panel, beep_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 3, 1, 1);
      		add_components(image_panel, display_image_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 7, 1, 1);
   		add_components(image_panel, save_disk_chkbox, gbl, gbc, 0, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 8, 1, 1);
      		add_components(image_panel, image_filename_txtfld, gbl, gbc, 0, 0, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 8, 4, 1);
      		add_components(image_panel, autoincr_chkbox, gbl, gbc, 0, 0, 20, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1, 9, 2, 1);
      		add_components(image_panel, image_incr_start_number_label, gbl, gbc, 0, 0, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 3, 9, 1, 1);
      		add_components(image_panel, image_incr_start_number_txtfld, gbl, gbc, 0, 0, 20, 0, GridBagConstraints.NONE, GridBagConstraints.WEST, 4, 9, 1, 1);

		// Add the event listeners to the components.
		display_image_chkbox.addItemListener(exposure_control_listener);
		multi_exp_chkbox.addItemListener(exposure_control_listener);
		save_disk_chkbox.addItemListener(exposure_control_listener);
		autoincr_chkbox.addItemListener(exposure_control_listener);
		delay_chkbox.addItemListener(exposure_control_listener);
		open_shutter_chkbox.addItemListener(exposure_control_listener);

		// Initialize the components.
		open_shutter_chkbox.doClick();
		save_disk_chkbox.doClick();

		return image_panel;
	}

	/**
	********************************************************************************
	*	This method creates the exposure control panel.
	*
	*	@return Returns the constructed exposure control panel.
	*
	*	@see JPanel
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JPanel create_exposure_control_panel()
	{
     		JPanel exposure_panel = new JPanel();
     		
     		// Set the panel layout.
      		GridBagLayout gbl = new GridBagLayout();
      		GridBagConstraints gbc = new GridBagConstraints();
      		exposure_panel.setLayout(gbl);
      		
		// Create the exposure type border.
		Border exposure_control_border = new EtchedBorder();
		exposure_panel.setBorder(exposure_control_border);

     		// Create the labels.
     		JLabel exposure_time_label = new JLabel("Exp Time (sec): ");
     		JLabel progress_bar_label = new JLabel("Readout Progress: ");
		JLabel elapsed_time_label = new JLabel("Elapsed Time: ");
		elapsed_label = new JLabel("0");
		
		// Set the label colors.
     		exposure_time_label.setForeground(Color.black);
     		progress_bar_label.setForeground(Color.black);
		elapsed_time_label.setForeground(Color.black);
		elapsed_label.setForeground(Color.black);
		
		// Create the textfields.
     		exposure_time_txtfld = new VTextField("1.000", 5);
     		
     		// Create the buttons.
     		expose_button = new JButton("Expose");
		pause_button = new JButton("Pause ");
		change_exp_time_button = new JButton("Change Exp Time");
		
		// Set the button size.
		expose_button.setPreferredSize(new Dimension(150, 26));
		Dimension current_size = pause_button.getPreferredSize();
		pause_button.setPreferredSize(new Dimension(75, 26));
		change_exp_time_button.setPreferredSize(new Dimension(150, 26));

		// Set the button text position relative to button edge.
		change_exp_time_button.setMargin(new Insets(2,2,2,2));
		pause_button.setMargin(new Insets(2,2,2,2));

		// Disable the pause button.
		pause_button.setEnabled(false);
		change_exp_time_button.setEnabled(false);
		
     		// Create the progress bar.
     		progress_bar = new JProgressBar();

     		// Add the components to the exposure control panel.
      		add_components(exposure_panel, exposure_time_label, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 1, 1, 1);
      		add_components(exposure_panel, exposure_time_txtfld, gbl, gbc, 10, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST, 1, 1, 1, 1);
     		add_components(exposure_panel, expose_button, gbl, gbc, 10, 10, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 2, 1, 1, 1);
      		add_components(exposure_panel, elapsed_time_label, gbl, gbc, 0, 0, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST, 0, 2, 1, 1);
      		add_components(exposure_panel, elapsed_label, gbl, gbc, 0, 0, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST, 1, 2, 2, 1);
      		add_components(exposure_panel, change_exp_time_button, gbl, gbc, 0, 10, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST, 2, 2, 1, 1);
     		add_components(exposure_panel, pause_button, gbl, gbc, 0, 0, 0, 10, GridBagConstraints.NONE, GridBagConstraints.EAST, 3, 2, 1, 1);
//      		add_components(exposure_panel, progress_bar_label, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.EAST, 0, 2, 1, 1);
//      		add_components(exposure_panel, progress_bar, gbl, gbc, 20, 0, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER, 1, 2, 2, 1);

		// Add the event listeners to the components.
		beep_chkbox.addItemListener(exposure_control_listener);
		expose_button.addActionListener(exposure_control_listener);
		pause_button.addActionListener(exposure_control_listener);
		change_exp_time_button.addActionListener(exposure_control_listener);
		
		return exposure_panel;
	}
	
	/**
	********************************************************************************
	*	This method creates the info/status panel.
	*
	*	@return Returns the constructed info/status panel.
	*
	*	@see JScrollPane
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	private JScrollPane create_info_panel()
	{
		// Initialize the information output text pane.
		info_output_textarea = new JTextPane();
	     	
	     	// Set window size.
	     	info_output_textarea.setPreferredSize(new Dimension(10, 36));
	     	
	     	// Make the text area uneditable by the user.
	     	info_output_textarea.setEditable(false);
	
		// Enable double buffering.
		info_output_textarea.setDoubleBuffered(true);
			
		// Get a reference to the content model of the text pane.
		contentModel = info_output_textarea.getDocument();
		
		// Prepare the character attribute set called style1.
		charStyle1 = new SimpleAttributeSet();
		StyleConstants.setFontFamily(charStyle1, "Courier");
		StyleConstants.setFontSize(charStyle1, 14);

		// Prepare the character attribute set called style2.
		charStyle2 = new SimpleAttributeSet();
		StyleConstants.setFontFamily(charStyle2, "Courier");
		StyleConstants.setFontSize(charStyle2, 14);
		StyleConstants.setForeground(charStyle2, Color.red);

		// Prepare the character attribute set called style3.
		charStyle3 = new SimpleAttributeSet();
		StyleConstants.setFontFamily(charStyle3, "Courier");
		StyleConstants.setFontSize(charStyle3, 14);
		StyleConstants.setForeground(charStyle3, Color.blue);
		
      		try {
      			contentModel.insertString(contentModel.getLength(), "Welcome To Voodoo \n", charStyle1);
      			line_counter++;
      		}
      		catch (BadLocationException ble) {}

     		// Create a scrollpane around the text area and add it to the infoPanel.
     		scroll_pane = new JScrollPane(info_output_textarea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
     		return scroll_pane;
	}
	

	/**
	*****************************************************************************
	*	Adds the specified component to the specified panel.
	*
	*	@param panel The panel to which the component will be added.
	*	@param compnt The component to add.
	*	@param gbl The GridBagLayout of the panel.
	*	@param gbl The GridBagConstraints of the panel.
	*	@param insetTOP The distance in pixels between the top of this
	*			component and the one above it.
	*	@param insetBOTTOM The distance in pixels between the bottom of
	*			this component and the one below it.
	*	@param insetLEFT The distance in pixels between the left side of
	*			this component and the one left of it.
	*	@param insetRIGHT The distance in pixels between the right side
	*			of this component and the one right of it.
	*	@param cfill Determines how the component "fills" the space of
	*		  	of its "cell".
	*	@param anchr The justification of the component within its "cell".
	*	@param col The x position of the component.
	*	@param row The y position of the component.
	*	@param gw The number of "cells" that the component spans in the
	*			width direction.
	*	@param gh The number of "cells" that the component spans in the
	*			height direction.
	*
	*	@see JPanel
	*	@see JComponent
	*	@see GridBagLayout
	*	@see GridBagConstraints
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	private void add_components(JPanel panel, JComponent compnt, GridBagLayout gbl, GridBagConstraints gbc, int insetTOP, int insetLEFT, int insetBOTTOM, int insetRIGHT, int cfill, int anchr, int col, int row, int gw, int gh)
   	{
      		// Set the grid layout constraints.
      		gbc.fill = cfill;
      		gbc.anchor = anchr;
      		gbc.insets = new Insets(insetTOP, insetLEFT, insetBOTTOM, insetRIGHT);
      		gbc.gridx = col;
      		gbc.gridy = row;
      		gbc.gridwidth = gw;
      		gbc.gridheight = gh;
      
      		// Set the component font.
      		compnt.setFont(font);

      		// Add the component to the panel.
      		gbl.setConstraints(compnt, gbc);
      		panel.add(compnt);
      	}

	/**
	*****************************************************************************
	*	Adds the specified component to the specified panel.
	*
	*	@param container The container to which the component will be added.
	*	@param compnt The component to add.
	*	@param gbl The GridBagLayout of the panel.
	*	@param gbl The GridBagConstraints of the panel.
	*	@param insetTOP The distance in pixels between the top of this
	*			component and the one above it.
	*	@param insetBOTTOM The distance in pixels between the bottom of
	*			this component and the one below it.
	*	@param insetLEFT The distance in pixels between the left side of
	*			this component and the one left of it.
	*	@param insetRIGHT The distance in pixels between the right side
	*			of this component and the one right of it.
	*	@param cfill Determines how the component "fills" the space of
	*		  	of its "cell".
	*	@param anchr The justification of the component within its "cell".
	*	@param col The x position of the component.
	*	@param row The y position of the component.
	*	@param gw The number of "cells" that the component spans in the
	*			width direction.
	*	@param gh The number of "cells" that the component spans in the
	*			height direction.
	*
	*	@see JPanel
	*	@see JComponent
	*	@see GridBagLayout
	*	@see GridBagConstraints
	*	@version 1.00
	*	@author Scott Streit
	*****************************************************************************
	*/
	private void add_components(Container container, JComponent compnt, GridBagLayout gbl, GridBagConstraints gbc, int insetTOP, int insetLEFT, int insetBOTTOM, int insetRIGHT, int cfill, int anchr, int col, int row, int gw, int gh)
   	{
      		// Set the grid layout constraints.
      		gbc.fill = cfill;
      		gbc.anchor = anchr;
      		gbc.insets = new Insets(insetTOP, insetLEFT, insetBOTTOM, insetRIGHT);
      		gbc.gridx = col;
      		gbc.gridy = row;
      		gbc.gridwidth = gw;
      		gbc.gridheight = gh;
      
      		// Add the component to the panel.
      		gbl.setConstraints(compnt, gbc);
      		container.add(compnt);
      	}
      	
	/**
	********************************************************************************
	*	This method sets the info/status text.
	*
	*	@param text The text to display in the info/status panel.
	*	@param style The style of the text to display. Style = 1, means use
	*		     charStyle1 (Courier, plain, 14, black). Style = 2, means
	*		     use charStyle2 (Courier, plain, 14, red). Style = 3,
	*		     means use charStyle3 (Courier, plain, 14, blue).
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static void set_info_panel_text(String text, int style)
      	{
       		try {
	      		switch(style) {
      				case 1:
      					contentModel.insertString(contentModel.getLength(), text, charStyle1);
      					break;
      				case 2:
      					contentModel.insertString(contentModel.getLength(), text, charStyle2);
      					break;
      				case 3:
      					contentModel.insertString(contentModel.getLength(), text, charStyle3);
      					break;
			}
      			line_counter++;
      		}
      		catch (BadLocationException ble) {}
      	}

	/**
	********************************************************************************
	*	This method sets the info/status text.
	*
	*	@param text The text to display in the info/status panel.
	*	@param style The style of the text to display. Style = 1, means use
	*		     charStyle1 (Courier, plain, 14, black). Style = 2, means
	*		     use charStyle2 (Courier, plain, 14, red). Style = 3,
	*		     means use charStyle3 (Courier, plain, 14, blue).
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static void set_info_panel_text(String text, int style, int offset)
      	{
       		try {
	      		switch(style) {
      				case 1:
      					//contentModel.remove(contentModel.getLength()-offset, offset);
      					contentModel.insertString(contentModel.getLength(), text, charStyle1);
      					break;
      				case 2:
      					//contentModel.remove(contentModel.getLength()-offset, offset);
	      				contentModel.insertString(contentModel.getLength(), text, charStyle2);
      					break;
      				case 3:
      					//contentModel.remove(contentModel.getLength()-offset, offset);
	      				contentModel.insertString(contentModel.getLength(), text, charStyle3);
      					break;
			}
			line_counter++;
      		}
      		catch (BadLocationException ble) {}
      	}

	/**
	********************************************************************************
	*	This method searches out and replaces the last "waiting" string with
	*	text.
	*
	*	@param text The text to display in the info/status panel.
	*	@param style The style of the text to display. Style = 1, means use
	*		     charStyle1 (Courier, plain, 14, black). Style = 2, means
	*		     use charStyle2 (Courier, plain, 14, red). Style = 3,
	*		     means use charStyle3 (Courier, plain, 14, blue).
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static void set_info_panel_text(int style, String text)
      	{
       		try {
			contentModel.remove(contentModel.getText(0, contentModel.getLength()).lastIndexOf("waiting."), 8);

	      		switch(style) {
      				case 1:
      					contentModel.insertString(contentModel.getText(0, contentModel.getLength()).lastIndexOf("...")+3, text, charStyle1);
      					break;
      				case 2:
	      				contentModel.insertString(contentModel.getText(0, contentModel.getLength()).lastIndexOf("...")+3, text, charStyle2);
      					break;
      				case 3:
	      				contentModel.insertString(contentModel.getText(0, contentModel.getLength()).lastIndexOf("...")+3, text, charStyle3);
      					break;
			}
      		}
      		catch (BadLocationException ble) {}
      	}

	/**
	********************************************************************************
	*	This method clears the info/status text.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
      	public static void clear_info_panel_text()
      	{
      		try {
      			// Remove the contents of the info/status window.
      			contentModel.remove(0, contentModel.getLength()-1);
      			
      			// Put a blank line at the start of info/status window.
      			// This is to prevent a NullPointer to the info/status
      			// window.
      			set_info_panel_text("   ", 1);

	      		line_counter = 0;
      		}
      		catch (BadLocationException ble) {}
      	}

	/**
	********************************************************************************
	*	This method downwardly increments the info/status panel scrollbar. To
	*	ensure proper scrolling, this method must be called BEFORE any text is
	*	written to the window.
	*
	*	@version 1.1
	*	@author Scott Streit
	********************************************************************************
	*/
      	public static void increment_info_panel_scrollbar()
      	{
		// Get a reference to the info panel's viewport (visible window).
		JViewport viewport = scroll_pane.getViewport();

		if (line_counter > 12) {
			// Get the coordinates of the viewport's upper left hand corner
			// and increment the Y ordinate by 20.
			Point point = viewport.getViewPosition();

			double y = point.getY() + 15;
			double x = point.getX();
			
			// Set the viewport's upper left hand corner coordinate.
			point.setLocation(x, y);
			viewport.setViewPosition(viewport.toViewCoordinates(point));
		}
      	}
      	
	/**
	********************************************************************************
	*	This method updates the readout progress bar. NOT USED !!!
	*
	*	@param i The current value to be set.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
      	public static void set_progress_bar(final int i)
      	{
      		progress_bar.setValue(i);
      	}

	/**
	********************************************************************************
	*	This method gets the maximum value the progress bar can have. NOT USED !!!
	*
	*	@return Returns the maximum progress bar value.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static int get_progress_bar_max_value()
	{
		return progress_bar.getMaximum();
	}

	/**
	********************************************************************************
	*	This method sets the elapsed exposure time string.
	*
	*	@param elapsed_time The elapsed exposure time (in seconds).
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public static void set_elapsed_time(int elapsed_time)
	{
		elapsed_label.setText(Integer.toString(elapsed_time));	
	}

	/**
	********************************************************************************
	*	This method sets the current array temperature string.
	*
	*	@param temperature The current array temperature (in Celcius).
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_temperature_label(double temperature)
	{
		temperature2_label.setText(Double.toString(temperature).substring(0, Double.toString(temperature).indexOf(".")+2) + " C");	
	}

	/**
	********************************************************************************
	*	This method sets the current exposure event listener, which is determined
	*	by the controller configuration used.
	*
	*	@param array_type CCD (0) or IR (1). Bit 12 of the READ_CONTROLLER_CONFIG
	*			  command's return value.
	*
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public void set_exposure_event_listener(int array_type)
	{
		if (array_type == VoodooConstants.IR) {
			// Remove the previous event listeners.
			display_image_chkbox.removeItemListener(exposure_control_listener);
			multi_exp_chkbox.removeItemListener(exposure_control_listener);
			save_disk_chkbox.removeItemListener(exposure_control_listener);
			autoincr_chkbox.removeItemListener(exposure_control_listener);
			delay_chkbox.removeItemListener(exposure_control_listener);
			open_shutter_chkbox.removeItemListener(exposure_control_listener);
			beep_chkbox.removeItemListener(exposure_control_listener);
			expose_button.removeActionListener(exposure_control_listener);
			pause_button.removeActionListener(exposure_control_listener);
			change_exp_time_button.removeActionListener(exposure_control_listener);

			exposure_control_listener = new IRExposureControlListener(this);

			// Install the new event listeners.
			display_image_chkbox.addItemListener(exposure_control_listener);
			multi_exp_chkbox.addItemListener(exposure_control_listener);
			save_disk_chkbox.addItemListener(exposure_control_listener);
			autoincr_chkbox.addItemListener(exposure_control_listener);
			delay_chkbox.addItemListener(exposure_control_listener);
			open_shutter_chkbox.addItemListener(exposure_control_listener);
			beep_chkbox.addItemListener(exposure_control_listener);
			expose_button.addActionListener(exposure_control_listener);
			pause_button.addActionListener(exposure_control_listener);
			change_exp_time_button.addActionListener(exposure_control_listener);

			// The event listeners may have changed, so re-initialize the IR
			// checkboxes to equal the previous chosen values. "exp_cmd" is set
			// to "ir_exp_cmd" in the IRExposureControlListener() constructor.
			exposure_control_listener.exp_cmd.set_save_to_disk(save_disk_chkbox.isSelected());
			exposure_control_listener.exp_cmd.set_multiple_exposures(multi_exp_chkbox.isSelected());
			exposure_control_listener.exp_cmd.set_auto_increment(autoincr_chkbox.isSelected());
			exposure_control_listener.exp_cmd.set_delay(delay_chkbox.isSelected());
			exposure_control_listener.exp_cmd.set_display_image(display_image_chkbox.isSelected());
			exposure_control_listener.exp_cmd.set_beep(beep_chkbox.isSelected());
			exposure_control_listener.exp_cmd.set_open_shutter(open_shutter_chkbox.isSelected());
		}
	}

	/**
	********************************************************************************
	*	This class renderes the images for the deinterlace option.
	*
	*	@see JLabel
	*	@see ListCellRenderer
	*	@version 1.00
	*	@author Scott Streit	
	********************************************************************************
	*/
       	class DeinterlaceCellRenderer extends JLabel implements ListCellRenderer
	{
		/**
		********************************************************************************
		*	Class constructor.
		*
		*	@version 1.00
		*	@author Scott Streit	
		********************************************************************************
		*/
		public DeinterlaceCellRenderer()
		{
			setOpaque(true);
			setHorizontalAlignment(CENTER);
			setVerticalAlignment(CENTER);
		}
	
		/**
		********************************************************************************
		*	This methods renders the images for the deinterlace option.
		*
		*	@param list The list of options.
		*	@param value The image to display for one of the options.
		*	@param index The index into the list of options for the current option.
		*	@param isSelected True if the current option is selected, false otherwise.
		*	@param cellHasFocus True if the current option has the focus, else false.
		*
		*	Note: The deinterlace value is set in the exposure control listener to
		*	      the correct array type. It is meant for only the CCD exposure
		*	      command to be in use here, the exposure control listener routes
		*	      the value to the correct array command class.
		*
		*	@see ListCellRenderer
		*	@version 1.00
		*	@author Scott Streit	
		********************************************************************************
		*/
		public Component getListCellRendererComponent(JList list,
						      	      Object value,
						      	      int index,
						      	      boolean isSelected,
						      	      boolean cellHasFocus)
		{
			// What happens when a selection is made.
			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			}
			
			else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}

			ImageIcon icon = (ImageIcon)value;
               		setIcon(icon);

               		String description = icon.getDescription();
       			if (description.equals("None")) {
				exposure_control_listener.exp_cmd.set_deinterlacing(false);
       			}
       			else if (description.equals("Parallel")) {
				exposure_control_listener.exp_cmd.set_deinterlacing(true);
       				exposure_control_listener.exp_cmd.set_deinterlace_value(1);
       			}
       			else if (description.equals("Serial")) {
				exposure_control_listener.exp_cmd.set_deinterlacing(true);
       				exposure_control_listener.exp_cmd.set_deinterlace_value(2);
       			}
       			else if (description.equals("Quad")) {
				exposure_control_listener.exp_cmd.set_deinterlacing(true);
       				exposure_control_listener.exp_cmd.set_deinterlace_value(3);
       			}
       			else if (description.equals("IR Quad")) {
				exposure_control_listener.exp_cmd.set_deinterlacing(true);
       				exposure_control_listener.exp_cmd.set_deinterlace_value(4);
       			}

			return this;
		}
	}
}
