/**
********************************************************************************
*	This class is the event listener for the main Voodoo window. This class
*	handles both mouse and keyboard events.
*
*	CVM - Controller
*
*	@version 1.00
*	@author Scott Streit
********************************************************************************
*/
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

public class VoodooMainWindowListener extends KeyAdapter implements ActionListener
{
	private VoodooMainWindow parent = null;
	private String message;
	
	/**
	********************************************************************************
	*	Class constructor.
	*
	*	@param parent The window this class listens for.
	*
	*	@see ActionListener, KeyAdapter
	*	@version 1.00
	*	@author Scott Streit
	********************************************************************************
	*/
	public VoodooMainWindowListener(VoodooMainWindow parent)
	{
		this.parent = parent;
	}

	/**
	********************************************************************************
	*	This method handles all non-keyboard events.
	*
	*	@param e The event.
	*
	*	@see ActionListener
	*	@version 1.00
	*	@author Scott Streit
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	02/08/2000	sds	Removed reply buffer ready bit set.
	*	06/21/2000	sds	Added "else if" to check if a window is being
	*				opened while it is already open, if so, bring
	*				the window to the front of all other windows.
	********************************************************************************
	*/
	public void actionPerformed(ActionEvent e)
	{
		AbstractButton active_button = (AbstractButton) e.getSource();
						
		// Exit Voodoo.
		if (active_button.getText().equals("Exit")) {
			try {
				parent.dispose();
				MemorySetup.free_memory();
				VoodooSetup.close_device_driver();
				System.exit(0);
			}
	    		catch (Exception ex) {
				JOptionPane.showMessageDialog(parent, ex.getMessage(), "Device Driver Error", JOptionPane.ERROR_MESSAGE);
       				System.exit(1);
    			}
		}
		
		// Display the Voodoo global setup window.
		else if (active_button.getText().equals("Global Setup...")) {
			parent.voodoo_global_setup_dialog.number_of_global_setup_windows++;
			if (parent.voodoo_global_setup_dialog.number_of_global_setup_windows == 1)
				parent.voodoo_global_setup_dialog.setVisible(true);

			// Make sure the window is brought to the front if an attempted re-open occurs.
			else if (parent.voodoo_global_setup_dialog.number_of_global_setup_windows >= 1)
				parent.voodoo_global_setup_dialog.toFront();
		}

		// Display the Voodoo options window.
		else if (active_button.getText().equals("Image Buffers...")) {
			parent.image_buffers_dialog.setVisible(true);
		}
			
		// Display the Controller setup window.
		else if (active_button.getText().equals("Controller Setup...")) {
			parent.setup_dialog.number_of_setup_windows++;
			if (parent.setup_dialog.number_of_setup_windows == 1)
				parent.setup_dialog.setVisible(true);

			// Make sure the window is brought to the front if an attempted re-open occurs.
			else if (parent.setup_dialog.number_of_setup_windows >= 1)
				parent.setup_dialog.toFront();
		}
			
		// Display the FITS window.
		else if (active_button.getText().equals("FITS Setup...")) {
			parent.fits_dialog.number_of_fits_windows++;
			if (parent.fits_dialog.number_of_fits_windows == 1)
				parent.fits_dialog.setVisible(true);

			// Make sure the window is brought to the front if an attempted re-open occurs.
			else if (parent.fits_dialog.number_of_fits_windows >= 1)
				parent.fits_dialog.toFront();
		}
		
		// Display the DSP window.
		else if (active_button.getText().equals("DSP Commands...")) {
			parent.dsp_dialog.number_of_dsp_windows++;
			if (parent.dsp_dialog.number_of_dsp_windows == 1)
				parent.dsp_dialog.setVisible(true);

			// Make sure the window is brought to the front if an attempted re-open occurs.
			else if (parent.dsp_dialog.number_of_dsp_windows >= 1)
				parent.dsp_dialog.toFront();
		}
		
		// Clear the Info/Status window.
		else if (active_button.getText().equals("Clear Info Window"))
			parent.clear_info_panel_text();

		// Set the debug mode to "On/Off".
		else if (active_button.getText().equals("Debug Mode")) {
			if (active_button.isSelected())
				OptionsCommand.set_debug_mode(VoodooConstants.DEBUG_ON);
			else
				OptionsCommand.set_debug_mode(VoodooConstants.DEBUG_OFF);
		}

		// Set the synthetic image mode to "On/Off".
		else if (active_button.getText().equals("Synthetic Image Mode")) {
			if (active_button.isSelected())
				OptionsCommand.set_image_check_mode(VoodooConstants.ON);
			else
				OptionsCommand.set_image_check_mode(VoodooConstants.OFF);
		}
		
		// Set which should do the exposure timing, the controller or the host CPU.
		else if (active_button.getText().equals("Use Controller Exposure Time")) {
			if (active_button.isSelected())
				ExposureCommand.set_exposure_controller_mode(VoodooConstants.ON);
			else
				ExposureCommand.set_exposure_controller_mode(VoodooConstants.OFF);
		}
		
		// Display the About window.
		else if (active_button.getText().equals("About")) {
			message = "Program: " + Version.PROGRAM + "\n" +
				  "Author: " + Version.AUTHOR + "\n" +
				  "Release Date: " + Version.RELEASE_DATE + "\n" +
				  "Version: " + Version.VERSION;
			JOptionPane.showMessageDialog(parent, message, "About", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	********************************************************************************
	*	This method handles all the keyboard events. In particular, it allows
	*	a Ctrl-C to free the image buffers.
	*
	*	@param e The event.
	*
	*	@see ActionListener
	*	@version 1.00
	*	@author Scott Streit
	*
	*	DATE		AUTHOR	DESCRIPTION
	*	-----------------------------------
	*	02/08/2000	sds	Removed reply buffer ready bit set.
	********************************************************************************
	*/
	public void keyPressed(KeyEvent e)
	{
		int key_code = e.getKeyCode();

		if (e.isControlDown() && key_code == e.VK_C) {
			try {
				parent.dispose();
				MemorySetup.free_memory();
				VoodooSetup.close_device_driver();
				System.exit(0);
			}
	    		catch (Exception ex) {
				JOptionPane.showMessageDialog(parent, ex.getMessage(), "Device Driver Error", JOptionPane.ERROR_MESSAGE);
       				System.exit(1);
    			}
		}
	}
}